package com.hcloud.microserver.bank;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.fastjson.JSONObject;
import com.hcloud.microserver.commoncore.util.string.StringUtils;


import java.util.ArrayList;
import java.util.List;


// 有个很重要的点 DemoDataListener 不能被spring管理，要每次读取excel都要new,然后里面用到spring可以构造方法传进去
public class DemoDataListener extends AnalysisEventListener<BankApiExcel> {

    /**
     * 每隔5条存储数据库，实际使用中可以3000条，然后清理list ，方便内存回收
     */
    private static final int BATCH_COUNT = 5;
    List<BankApiExcel> list = new ArrayList<BankApiExcel>();
    /**
     * 假设这个是一个DAO，当然有业务逻辑这个也可以是一个service。当然如果不用存储这个对象没用。
     */
    private BankApiExcel bankApiExcel;
    public DemoDataListener() {
        // 这里是demo，所以随便new一个。实际使用如果到了spring,请使用下面的有参构造函数
        bankApiExcel = new BankApiExcel();
    }
    /**
     * 如果使用了spring,请使用这个构造方法。每次创建Listener的时候需要把spring管理的类传进来
     *
     * @param bankApiExcel
     */
    public DemoDataListener(BankApiExcel bankApiExcel) {
        this.bankApiExcel = bankApiExcel;
    }
    /**
     * 这个每一条数据解析都会来调用
     *
     * @param bankApiExcel
     *            one row value. Is is same as {@link AnalysisContext#readRowHolder()}
     * @param context
     */
    @Override
    public void invoke(BankApiExcel bankApiExcel, AnalysisContext context) {
        list.add(bankApiExcel);
        // 达到BATCH_COUNT了，需要去存储一次数据库，防止数据几万条数据在内存，容易OOM
        if (list.size() >= BATCH_COUNT) {
            saveData();
            // 存储完成清理 list
            list.clear();
        }
    }
    /**
     * 所有数据解析完成了 都会来调用
     *
     * @param context
     */
    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 这里也要保存数据，确保最后遗留的数据也存储到数据库
        saveData();
    }
    /**
     * 加上存储数据库
     */
    private void saveData() {
        String type = "String";
        for (BankApiExcel bankApiExcel:list) {
            String s = bankApiExcel.getType();
            if ( StringUtils.isNotEmpty(s) &&s.contains("P")){
                type = "BigDecimal";

            }
            System.out.println("/**"+bankApiExcel.getChName()+";"+bankApiExcel.getRemark()+";"+bankApiExcel.getType()+
            ";*/");
            System.out.println("private "+type+" "+bankApiExcel.getPro()+";");
        }

//        System.out.println(JSONObject.toJSONString(list));
    }
}
