package com.hcloud.microserver.facade.carbon.client;

import com.alibaba.fastjson.JSONObject;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.fallback.impl.GoodsInfoFallback;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsInfoForm;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

/**
 * @author xiezt
 */
@FeignClient(name = "carbon-service", path = "/goods", fallbackFactory = GoodsInfoFallback.class)
public interface GoodsInfoClient {

    @PostMapping("/shelves")
    ResultVO shelves(@RequestBody List<GoodsInfoForm> goodsInfoForms);

    @PostMapping("/delete")
    ResultVO delete(@RequestBody GoodsInfoForm goodsInfoForm);

    @PostMapping("update")
    ResultVO update(@RequestBody GoodsInfoForm goodsInfoForm);

    @PostMapping("/searchByPage")
    ResultVO searchByPage(@RequestBody GoodsInfoForm goodsInfoForm);

    @PostMapping("/searchJoinListByPage")
    ResultVO searchJoinListByPage(@RequestBody GoodsInfoForm goodsInfoForm);

    /**
     * 分页查询商品类型信息（移动端展示）
     * @param goodsInfoForm
     * @return
     */
    @PostMapping("/searchGoodsInfoByPage")
    ResultVO searchGoodsInfoByPage(@RequestBody GoodsInfoForm goodsInfoForm);



    /**
     * 获取商品详情
     * @param goodsId
     * @return
     */
    @GetMapping("/getGoodsInfoById")
    ResultVO getGoodsInfoById(@RequestParam("goodsId") String goodsId);

    /**
     * 根据需求碳汇量计算商品列表
     * @param carbon
     * @return
     */
    @GetMapping("/getGoodsInfoByCarbon")
    ResultVO getGoodsInfoByCarbon(@RequestParam("carbon") String carbon);

    /**
     * 更新商品
     * @param jsonObject
     * @return
     */
    @PostMapping("/updateGoods")
    ResultVO updateGoods(@RequestBody JSONObject jsonObject);

    /**
     * 通过农户Id查询所属商品
     * @param farmerInfoForm
     * @return
     */
    @PostMapping("/getGoodsInfoByFarmerId")
    ResultVO getGoodsInfoByFarmerId(@RequestBody FarmerInfoForm farmerInfoForm);

}
