package com.hcloud.microserver.facade.carbon.client;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.fallback.impl.CustomerFallback;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import javax.xml.transform.Result;

/**
 * @author xiezt
 */
@FeignClient(name = "carbon-service", path = "/customer",fallbackFactory = CustomerFallback.class)
public interface CustomerClient {

    /**
     * 分页查询爱心用户信息
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/searchCustomerByPage")
    ResultVO searchCustomerInfoByPage(@RequestBody CustomerInfoForm customerInfoForm);

    /**
     * 查询爱心用户明细
     * @param guid
     * @return
     */
    @GetMapping("/getCustomerInfoById")
    ResultVO getCustomerInfoById(@RequestParam("guid") String guid);

    /**
     * 保存爱心用户信息
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/saveCustomerInfo")
    ResultVO saveCustomerInfo(@RequestBody CustomerInfoForm customerInfoForm);

    /**
     * 修改爱心用户信息
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/updateCustomerInfo")
    ResultVO modifiedCustomerInfo(@RequestBody CustomerInfoForm customerInfoForm);

    /**
     * 通过登录账号查询爱心用户信息
     * @param account
     * @return
     */
    @GetMapping("/getCustomerInfoByAccount")
    ResultVO getCustomerInfoByAccount(@RequestParam("account") String account);

    /**
     * 多条件查询爱心用户信息
     * @param queryStr
     * @return
     */
    @GetMapping("/getCustomerInfoByInfo")
    ResultVO getCustomerInfoByInfo(@RequestParam("queryStr") String queryStr);

    /**
     * 通过爱心用户ID,查询爱心用户企业认证信息
     * @param customerId
     * @return
     */
    @GetMapping("/getCustomerCompanyInfo")
    ResultVO getCustomerCompanyInfo(@RequestParam("customerId") String customerId);

    /**
     * 通过登录渠道编码查询用户信息
     * @param type
     * @param threeCode
     * @return
     */
    @GetMapping("/getCustomerInfoByChannel")
    ResultVO getCustomerInfoByChannelInfo(@RequestParam("channelType") Integer type, @RequestParam("code") String threeCode);

    /**
     * 获取离线支付凭证通过用户信息
     * @param customerId
     * @return
     */
    @GetMapping("/getOfflinePayVoucherById")
    ResultVO getOfflinePayVoucherById(@RequestParam("customerId") String customerId);

    /**
     * 企业分页
     * @param customerCompanyInfoForm
     * @return
     */
    @PostMapping("/searchCompanyByPage")
    ResultVO searchCompanyByPage(@RequestBody CustomerCompanyInfoForm customerCompanyInfoForm);


    /**
     * 验证爱心用户企业认证信息
     * @param customerCompanyInfoForm
     * @return
     */
    @PostMapping("/validateCompanyInfo")
    ResultVO validateCustomerCompany(@RequestBody CustomerCompanyInfoForm customerCompanyInfoForm);

    /**
     * 重置密码
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/resetPassword")
    ResultVO resetPassword(@RequestBody CustomerInfoForm customerInfoForm);

    /**
     * 更新用户信息（微信登录渠道及用户信息内的微信支付信息）
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/updateCustomerInfoForWeChat")
    ResultVO updateCustomerInfoForWeChat(@RequestBody CustomerInfoForm customerInfoForm);

    /**
     * 绑定用户手机号，并合并订单相同手机号的订单信息
     * @param customerId
     * @param phone
     * @return
     */
    @GetMapping("/bindCustomerPhone")
    ResultVO bindCustomerPhone(@RequestParam("customerId") String customerId,
                               @RequestParam("phone") String phone);
}
