package com.hcloud.microserver.facade.carbon.client;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.fallback.impl.FarmerInfoFallback;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.PovertyAreaForm;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

@FeignClient(name = "carbon-service", path = "/farmer", fallbackFactory = FarmerInfoFallback.class)
public interface FarmerInfoClient {

    @PostMapping("/save")
    ResultVO save(@RequestBody FarmerInfoForm farmerInfoForm);

    @PostMapping("/delete")
    ResultVO delete(@RequestBody FarmerInfoForm farmerInfoForm);

    @PostMapping("/update")
    ResultVO update(@RequestBody FarmerInfoForm farmerInfoForm);

    @PostMapping("/searchByPage")
    ResultVO searchByPage(@RequestBody FarmerInfoForm farmerInfoForm);

    @PostMapping("/searchByCollectiveId")
    ResultVO searchByCollectiveId(@RequestParam(name = "collectiveId") String collectiveId);

    /**
     * 查询贫困区域(分页查询)
     * @param povertyAreaForm
     * @return
     */
    @PostMapping("/searchAreaByPage")
    ResultVO searchPovertyAreaByPage(@RequestBody PovertyAreaForm povertyAreaForm);

    /**
     * 通过贫困区域查询贫困户信息
     * @param farmerInfoForm
     * @return
     */
    @PostMapping("/getFarmerInfoByAreaId")
    ResultVO getFarmerInfoByAreaId(@RequestBody FarmerInfoForm farmerInfoForm);

    /**
     * 查询贫困区域明细信息(含贫困户统计信息,贫困区域贫困户列表,贫困区域基本信息)
     * @param areaId
     * @return
     */
    @GetMapping("/getPovertyAreaById")
    ResultVO getPovertyAreaById(@RequestParam("areaId") String areaId);

    /**
     * 获取农户信息通过产品Id
     * @param productGuid
     * @return
     */
    @PostMapping("/searchByProductGuid")
    ResultVO searchByProductGuid(@RequestParam(name = "productGuid") String productGuid);


    /**
     * 获取农户明细信息
     * @param farmerId
     * @return
     */
    @GetMapping("/getFarmerInfoById")
    ResultVO getFarmerInfoById(@RequestParam("guid") String farmerId);
}
