package com.hcloud.microserver.facade.carbon.client;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.fallback.impl.OrderFallback;
import com.hcloud.microserver.facade.carbon.forms.GoodsCredentialInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @author JYJ
 */
@FeignClient(name = "carbon-service", path = "/order",fallbackFactory = OrderFallback.class)
public interface OrderClient {
    /**
     * 分页查询订单信息
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/searchOrderInfoPage")
    ResultVO searchOrderInfoByPage(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm);

    /**
     * 分页获取爱心用户订单信息
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/getOrderInfoByCustomerId")
    ResultVO getOrderInfoByCustomerId(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm);

    /**
     * 查询订单明细信息
     * @param orderId
     * @return
     */
    @GetMapping("/getOrderDetail")
    ResultVO getOrderDetailInfoByOrderId(@RequestParam("orderId") String orderId);

    /**
     * 获取某订单基本信息
     * @param orderId
     * @return
     */
    @GetMapping("/getOrderInfoById")
    ResultVO getOrderInfoById(@RequestParam("guid") String orderId);

    /**
     * 删除订单信息
     * @param orderId
     * @return
     */
    @GetMapping("/delOderInfoById")
    ResultVO delOrderInfo(@RequestParam("guid") String orderId);

    /**
     * 更新订单信息
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/modifiedOrderInfo")
    ResultVO updateOrderInfo(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm);

    /**
     * 统一下单生成商品订单，并生成支付订单。返回支付订单信息
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/insertOrderInfoForPay")
    ResultVO saveOrderInfoAndPayOrderInfo(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm);

    /**
     * 支付回调后，统一更新订单信息及支付订单信息
     * @param payOrderNO
     * @return
     */
    @GetMapping("/modifiedOrderInfoForPay")
    ResultVO updateOrderInfoAndPayOrderInfo(@RequestParam("payNo") String payOrderNO);

    /**
     * 获取用户证书列表
     * @param goodsCredentialInfoForm
     * @return
     */
    @GetMapping("/getCustomerCredentialList")
    ResultVO getCustomerCredentialList(@RequestBody GoodsCredentialInfoForm goodsCredentialInfoForm);

    /**
     * 订单支付超时取消
     * @param orderId
     * @return
     */
    @GetMapping("/callbackOrderInfo")
    ResultVO callbackOrderInfo(@RequestParam("orderId") String orderId);

    /**
     * 通过订单编号查询订单信息
     * @param orderNo
     * @return
     */
    @GetMapping("/getOrderInfoByNo")
    ResultVO getOrderInfoByOrderNo(@RequestParam("orderNo") String orderNo);

    /**
     * 取消用户订单
     * @param orderId
     * @return
     */
    @GetMapping("/cancelOrderInfo")
    ResultVO cancelOrderInfo(@RequestParam("orderId") String orderId);

    /**
     * 通过碳汇需求量购买指定农户下的碳汇商品
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/saveOrderInfoForFarmer")
    ResultVO saveOrderInfoForFarmer(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm);

    /**
     * 根据证书guid获取证书
     * @param guid
     * @return
     */
    @GetMapping("/getCredential")
    ResultVO getCredentialInfo(@RequestParam(name = "guid") String guid);

    /**
     * 根据订单id返回证书
     * @param orderId
     * @return
     */
    @GetMapping("/getCredentialByOrderId")
    ResultVO getCredentialByOrderId(@RequestParam(name = "orderId") String orderId);
}
