package com.hcloud.microserver.facade.carbon.client;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.fallback.impl.OrderPayFallback;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsPayOrderForm;
import com.hcloud.microserver.facade.carbon.forms.PaymentForm;
import jdk.nashorn.internal.objects.annotations.Getter;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * @author JYJ
 */
@FeignClient(name = "carbon-service", path = "/order/pay", fallbackFactory = OrderPayFallback.class)
public interface OrderPayClient {

    /**
     * 查询某订单，支付流水明细
     * @param orderId
     * @return
     */
    @GetMapping("/getListById")
    ResultVO getOrderPayListByOrderId(@RequestParam("orderId") String orderId);

    /**
     * 保存支付订单信息
     * @param goodsPayOrderForm
     * @return
     */
    @PostMapping("/saveOrderPayInfo")
    ResultVO savePayOrderInfo(@RequestBody GoodsPayOrderForm goodsPayOrderForm);

    /**
     * 更新支付订单明细
     * @param goodsPayOrderForm
     * @return
     */
    @PostMapping("/modifiedOrderPayInfo")
    ResultVO updateOrderPayInfo(@RequestBody GoodsPayOrderForm goodsPayOrderForm);

    /**
     * 通过支付订单编号查询支付信息
     * @param payNo
     * @return
     */
    @GetMapping("/getPayOrderInfoByNo")
    ResultVO getPayOrderInfoByPayNo(@RequestParam("payNo") String payNo);

    @PostMapping("/queryPaymentByPage")
    ResultVO queryPaymentByPage(@RequestBody PaymentForm paymentForm);
}
