package com.hcloud.microserver.facade.carbon.fallback.impl;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.fallback.CustomerFallbackFactory;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import feign.hystrix.FallbackFactory;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

/**
 * @author xiezt
 */
@Component
public class CustomerFallback implements FallbackFactory<CustomerFallbackFactory> {

    @Override
    public CustomerFallbackFactory create(Throwable cause) {
        return new CustomerFallbackFactory() {
            /**
             * 分页查询爱心用户信息
             *
             * @param customerInfoForm
             * @return
             */
            @Override
            public ResultVO searchCustomerInfoByPage(CustomerInfoForm customerInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 查询爱心用户明细
             *
             * @param guid
             * @return
             */
            @Override
            public ResultVO getCustomerInfoById(String guid) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 保存爱心用户信息
             *
             * @param customerInfoForm
             * @return
             */
            @Override
            public ResultVO saveCustomerInfo(CustomerInfoForm customerInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 修改爱心用户信息
             *
             * @param customerInfoForm
             * @return
             */
            @Override
            public ResultVO modifiedCustomerInfo(CustomerInfoForm customerInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 通过登录账号查询爱心用户信息
             *
             * @param account
             * @return
             */
            @Override
            public ResultVO getCustomerInfoByAccount(String account) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 多条件查询爱心用户信息
             *
             * @param queryStr
             * @return
             */
            @Override
            public ResultVO getCustomerInfoByInfo(String queryStr) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 通过爱心用户ID,查询爱心用户企业认证信息
             *
             * @param customerId
             * @return
             */
            @Override
            public ResultVO getCustomerCompanyInfo(String customerId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 通过登录渠道编码查询用户信息
             *
             * @param type
             * @param threeCode
             * @return
             */
            @Override
            public ResultVO getCustomerInfoByChannelInfo(Integer type, String threeCode) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 获取离线支付凭证通过用户信息
             *
             * @param customerId
             * @return
             */
            @Override
            public ResultVO getOfflinePayVoucherById(String customerId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 企业分页
             * @param customerCompanyInfoForm
             * @return
             */
            @Override
            public ResultVO searchCompanyByPage(CustomerCompanyInfoForm customerCompanyInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 验证爱心用户企业认证信息
             *
             * @param customerCompanyInfoForm
             * @return
             */
            @Override
            public ResultVO validateCustomerCompany(CustomerCompanyInfoForm customerCompanyInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 重置密码
             * @param customerInfoForm
             * @return
             */
            @Override
            public ResultVO resetPassword(CustomerInfoForm customerInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 更新用户信息（微信登录渠道及用户信息内的微信支付信息）
             *
             * @param customerInfoForm
             * @return
             */
            @PostMapping("/updateCustomerInfoForWeChat")
            @Override
            public ResultVO updateCustomerInfoForWeChat(@RequestBody CustomerInfoForm customerInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 绑定用户手机号，并合并订单相同手机号的订单信息
             *
             * @param customerId
             * @param phone
             * @return
             */
            @Override
            public ResultVO bindCustomerPhone(String customerId, String phone) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }
        };
    }
}
