package com.hcloud.microserver.facade.carbon.fallback.impl;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.fallback.OrderFallbackFactory;
import com.hcloud.microserver.facade.carbon.forms.GoodsCredentialInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsPayOrderForm;
import feign.hystrix.FallbackFactory;
import org.springframework.stereotype.Component;

/**
 * @author xiezt
 */
@Component
public class OrderFallback implements FallbackFactory<OrderFallbackFactory> {

    @Override
    public OrderFallbackFactory create(Throwable cause) {
        return new OrderFallbackFactory(){
            /**
             * 分页查询订单信息
             *
             * @param goodsOrderInfoForm
             * @return
             */
            @Override
            public ResultVO searchOrderInfoByPage(GoodsOrderInfoForm goodsOrderInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            @Override
            public ResultVO getOrderInfoByCustomerId(GoodsOrderInfoForm goodsOrderInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 查询订单明细信息
             *
             * @param orderId
             * @return
             */
            @Override
            public ResultVO getOrderDetailInfoByOrderId(String orderId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 获取某订单基本信息
             *
             * @param orderId
             * @return
             */
            @Override
            public ResultVO getOrderInfoById(String orderId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 删除订单信息
             *
             * @param orderId
             * @return
             */
            @Override
            public ResultVO delOrderInfo(String orderId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 更新订单信息
             *
             * @param goodsOrderInfoForm
             * @return
             */
            @Override
            public ResultVO updateOrderInfo(GoodsOrderInfoForm goodsOrderInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 统一下单生成商品订单，并生成支付订单。返回支付订单信息
             *
             * @param goodsOrderInfoForm
             * @return
             */
            @Override
            public ResultVO saveOrderInfoAndPayOrderInfo(GoodsOrderInfoForm goodsOrderInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 支付回调后，统一更新订单信息及支付订单信息
             *
             * @param payOrderNO
             * @return
             */
            @Override
            public ResultVO updateOrderInfoAndPayOrderInfo(String payOrderNO) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 获取用户证书列表
             *
             * @param goodsCredentialInfoForm
             * @return
             */
            @Override
            public ResultVO getCustomerCredentialList(GoodsCredentialInfoForm goodsCredentialInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 订单支付超时取消
             *
             * @param orderId
             * @return
             */
            @Override
            public ResultVO callbackOrderInfo(String orderId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 通过订单编号查询订单信息
             *
             * @param orderNo
             * @return
             */
            @Override
            public ResultVO getOrderInfoByOrderNo(String orderNo) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 取消用户订单
             *
             * @param orderId
             * @return
             */
            @Override
            public ResultVO cancelOrderInfo(String orderId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 通过碳汇需求量购买指定农户下的碳汇商品
             *
             * @param goodsOrderInfoForm
             * @return
             */
            @Override
            public ResultVO saveOrderInfoForFarmer(GoodsOrderInfoForm goodsOrderInfoForm) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 根据证书guid获取证书
             *
             * @param guid
             * @return
             */
            @Override
            public ResultVO getCredentialInfo(String guid) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 根据订单id返回证书
             *
             * @param orderId
             * @return
             */
            @Override
            public ResultVO getCredentialByOrderId(String orderId) {
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }
        };
    }
}
