package com.hcloud.microserver.bank.controller;

import com.alibaba.fastjson.JSONObject;
import com.hcloud.microserver.bank.service.CompanyService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.facade.carbon.client.CompanyClient;
import com.hcloud.microserver.facade.carbon.entity.CustomerCompanyInfo;
import com.hcloud.microserver.facade.carbon.entity.CustomerInfo;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@Slf4j
@RestController
@RequestMapping("/company")
public class CompanyManageController extends BaseController implements CompanyClient {

    @Autowired
    private CompanyService companyService;

    @Override
    @PostMapping("/add")
    public ResultVO addCompany(@RequestBody JSONObject jsonObject) {
        log.info("前端的参数: {}", jsonObject.toJSONString());
        CustomerInfoForm customerInfoForm = jsonObject.getObject("customerInfoForm", CustomerInfoForm.class);
        CustomerCompanyInfoForm customerCompanyInfoForm = jsonObject.getObject("customerCompanyInfoForm", CustomerCompanyInfoForm.class);

        CustomerInfo customerInfo = BeanCopyUtil.convertBean(customerInfoForm, CustomerInfo.class);
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(customerCompanyInfoForm, CustomerCompanyInfo.class);

        int success = this.companyService.saveCompany(customerInfo, customerCompanyInfo);
        if (success >= 1)
            return success();
        return failure();
    }

    @Override
    @PostMapping("/update")
    public ResultVO updateCompany(@RequestBody JSONObject jsonObject) {
        log.info("前端的参数: {}", jsonObject.toJSONString());
        CustomerInfoForm customerInfoForm = jsonObject.getObject("customerInfoForm", CustomerInfoForm.class);
        CustomerCompanyInfoForm customerCompanyInfoForm = jsonObject.getObject("customerCompanyInfoForm", CustomerCompanyInfoForm.class);

        CustomerInfo customerInfo = BeanCopyUtil.convertBean(customerInfoForm, CustomerInfo.class);
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(customerCompanyInfoForm, CustomerCompanyInfo.class);

        int success = this.companyService.modifyCompany(customerInfo, customerCompanyInfo);
        if (success >= 1)
            return success();
        return failure();
    }

    @Override
    @PostMapping("/delete")
    public ResultVO deleteCompany(@RequestBody JSONObject jsonObject) {
        log.info("前端的参数: {}", jsonObject.toJSONString());
        CustomerInfoForm customerInfoForm = jsonObject.getObject("customerInfoForm", CustomerInfoForm.class);
        CustomerCompanyInfoForm customerCompanyInfoForm = jsonObject.getObject("customerCompanyInfoForm", CustomerCompanyInfoForm.class);

        CustomerInfo customerInfo = BeanCopyUtil.convertBean(customerInfoForm, CustomerInfo.class);
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(customerCompanyInfoForm, CustomerCompanyInfo.class);

        int success = this.companyService.removeCompany(customerInfo, customerCompanyInfo);
        if (success >= 1)
            return success();
        return failure();
    }
}
