package com.hcloud.microserver.bank.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.service.CustomerCompanyInfoService;
import com.hcloud.microserver.bank.service.CustomerInfoService;
import com.hcloud.microserver.bank.service.OrderPayVoucherService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.TracedCodeUtils;
import com.hcloud.microserver.facade.carbon.client.CustomerClient;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.OrderPayVoucherForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/customer")
public class CustomerInfoController extends BaseController implements CustomerClient {

    /**
     * 默认初始密码
     */
    private static final String DEFAULT_PWD="000000";

    @Autowired
    private CustomerInfoService customerInfoService;

    @Autowired
    private CustomerCompanyInfoService customerCompanyInfoService;

    @Autowired
    private OrderPayVoucherService orderPayVoucherService;

    /**
     * 分页查询爱心用户信息
     *
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/searchCustomerByPage")
    @Override
    public ResultVO searchCustomerInfoByPage(@RequestBody CustomerInfoForm customerInfoForm) {
        PageInfo pageInfo = customerInfoService.searchCustomerByPage(customerInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 查询爱心用户明细
     *
     * @param guid
     * @return
     */
    @GetMapping("/getCustomerInfoById")
    @Override
    public ResultVO getCustomerInfoById(@RequestParam("guid") String guid) {
        CustomerInfoForm customerInfoForm = customerInfoService.getCustomerInfoById(guid);
        return success(customerInfoForm);
    }

    /**
     * 保存爱心用户信息
     *
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/saveCustomerInfo")
    @Override
    public ResultVO saveCustomerInfo(@RequestBody CustomerInfoForm customerInfoForm) {
        customerInfoForm.setAccount(TracedCodeUtils.getCommonRandomCode());
        //customerInfoForm.setPasswd(Md5Util.toMD5(CodeUtil.getSearchId()));
        customerInfoForm.setPasswd(Md5Util.toMD5(DEFAULT_PWD));
        int i = customerInfoService.saveCustomerInfoByChannelInfo(customerInfoForm);
        if(i==0){return failure();}
        return success(customerInfoForm);
    }

    /**
     * 修改爱心用户信息
     *
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/updateCustomerInfo")
    @Override
    public ResultVO modifiedCustomerInfo(@RequestBody CustomerInfoForm customerInfoForm) {
        int i = customerInfoService.modifyByPrimaryKeySelective(customerInfoForm);
        if(i==0){return failure();}
        return success();
    }

    /**
     * 通过登录账号查询爱心用户信息
     * @param account
     * @return
     */
    @GetMapping("/getCustomerInfoByAccount")
    @Override
    public ResultVO getCustomerInfoByAccount(@RequestParam("account") String account) {
        CustomerInfoForm customerInfoForm = customerInfoService.getCustomerInfoByAccount(account);
        if(customerInfoForm==null){return new ResultVO(ResultEnum.LOGIN_ACCOUNT_MISS);}
        return success(customerInfoForm);
    }

    /**
     * 多条件查询爱心用户信息
     * @param queryStr
     * @return
     */
    @GetMapping("/getCustomerInfoByInfo")
    @Override
    public ResultVO getCustomerInfoByInfo(@RequestParam("queryStr") String queryStr) {
       CustomerInfoForm customerInfo =  customerInfoService.getCustomerInfoByInfo(queryStr);
       if(Objects.nonNull(customerInfo)){
           return success(customerInfo);
       }
        return failure();
    }

    /**
     * 通过爱心用户ID,查询爱心用户企业认证信息
     * @param customerId
     * @return
     */
    @GetMapping("/getCustomerCompanyInfo")
    @Override
    public ResultVO getCustomerCompanyInfo(@RequestParam("customerId") String customerId) {
       CustomerCompanyInfoForm customerCompanyInfoForm =  customerCompanyInfoService.getCustomerCompanyInfoByCustomerId(customerId);
        return success(customerCompanyInfoForm);
    }

    /**
     * 通过登录渠道编码查询用户信息
     * @param type
     * @param threeCode
     * @return
     */
    @GetMapping("/getCustomerInfoByChannel")
    @Override
    public ResultVO getCustomerInfoByChannelInfo(@RequestParam("channelType") Integer type, @RequestParam("code") String threeCode) {
        CustomerInfoForm customerInfoForm = customerInfoService.getCustomerInfoByChannelInfo(type,threeCode);
        return success(customerInfoForm);
    }

    /**
     * 获取离线支付凭证通过用户信息
     * @param customerId
     * @return
     */
    @GetMapping("/getOfflinePayVoucherById")
    @Override
    public ResultVO getOfflinePayVoucherById(@RequestParam("customerId") String customerId) {
       List<OrderPayVoucherForm> list =  orderPayVoucherService.getPayVoucherInfoByOrderId(customerId);
        return success(list);
    }

    /**
     * 企业分页
     * @param customerCompanyInfoForm
     * @return
     */
    @Override
    @PostMapping("/searchCompanyByPage")
    public ResultVO searchCompanyByPage(@RequestBody CustomerCompanyInfoForm customerCompanyInfoForm) {
        PageInfo pageInfo = this.customerCompanyInfoService.queryListByPage(customerCompanyInfoForm);
        if (Objects.isNull(pageInfo)){ return success(null, 0);}
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    /**
     * 验证爱心用户企业认证信息
     * @param customerCompanyInfoForm
     * @return
     */
    @PostMapping("/validateCompanyInfo")
    @Override
    public ResultVO validateCustomerCompany(@RequestBody CustomerCompanyInfoForm customerCompanyInfoForm) {
        int i =customerCompanyInfoService.validateCompanyInfo(customerCompanyInfoForm);
        if (i==0){return failure();}
        return success();
    }

    @Override
    @PostMapping("/resetPassword")
    public ResultVO resetPassword(@RequestBody CustomerInfoForm customerInfoForm) {
        int i = this.customerInfoService.resetPassword(customerInfoForm);
        if (i > 0){ return success();}
        return failure();
    }

    /**
     * 更新用户信息（微信登录渠道及用户信息内的微信支付信息）
     *
     * @param customerInfoForm
     * @return
     */
    @PostMapping("/updateCustomerInfoForWeChat")
    @Override
    public ResultVO updateCustomerInfoForWeChat(@RequestBody CustomerInfoForm customerInfoForm) {
        int i =customerInfoService.updateCustomerInfoForWeChat(customerInfoForm);
        if(i==0){return failure();}
        return success();
    }

    /**
     * 绑定用户手机号，并合并订单相同手机号的订单信息
     *
     * @param customerId
     * @param phone
     * @return
     */
    @GetMapping("/bindCustomerPhone")
    @Override
    public ResultVO bindCustomerPhone(@RequestParam("customerId") String customerId,
                                      @RequestParam("phone") String phone) {
        if(null!=phone && !"".equals(phone)){
            int i = customerInfoService.bindCustomerPhone(customerId,phone);
            if(i==0){return failure();}
        }
        return success();
    }
}
