package com.hcloud.microserver.bank.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.service.FarmerInfoService;
import com.hcloud.microserver.bank.service.PovertyAreaService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.client.FarmerInfoClient;
import com.hcloud.microserver.facade.carbon.entity.FarmerInfo;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.PovertyAreaForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/farmer")
public class FarmerInfoManageController extends BaseController implements FarmerInfoClient {


    @Autowired
    private FarmerInfoService farmerInfoService;

    @Autowired
    private PovertyAreaService povertyAreaService;

    @Override
    @PostMapping("/save")
    public ResultVO save(@RequestBody FarmerInfoForm farmerInfoForm) {
        int success = this.farmerInfoService.save(farmerInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/delete")
    public ResultVO delete(@RequestBody FarmerInfoForm farmerInfoForm) {
        int success = this.farmerInfoService.softRemove(farmerInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/update")
    public ResultVO update(@RequestBody FarmerInfoForm farmerInfoForm) {
        int success = this.farmerInfoService.modify(farmerInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/searchByPage")
    public ResultVO searchByPage(@RequestBody FarmerInfoForm farmerInfoForm) {
        PageInfo pageInfo = this.farmerInfoService.queryListByPage(farmerInfoForm);
        if (Objects.isNull(pageInfo)){ return success(null, 0);}
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    /**
     * 查询贫困区域(分页查询)
     *
     * @param povertyAreaForm
     * @return
     */
    @PostMapping("/searchAreaByPage")
    @Override
    public ResultVO searchPovertyAreaByPage(@RequestBody PovertyAreaForm povertyAreaForm) {
       PageInfo pageInfo = povertyAreaService.queryListByPage(povertyAreaForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 通过贫困区域询贫困户信息
     *
     * @param farmerInfoForm
     * @return
     */
    @PostMapping("/getFarmerInfoByAreaId")
    @Override
    public ResultVO getFarmerInfoByAreaId(@RequestBody FarmerInfoForm farmerInfoForm) {
       PageInfo pageInfo =  farmerInfoService.searchFarmerInfoByPovertyArea(farmerInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 查询贫困区域明细信息(含贫困户统计信息,贫困区域贫困户列表,贫困区域基本信息)
     *
     * @param areaId
     * @return
     */
    @GetMapping("/getPovertyAreaById")
    @Override
    public ResultVO getPovertyAreaById(@RequestParam("areaId") String areaId) {
       PovertyAreaForm povertyAreaForm =povertyAreaService.getFullPovertyAreaInfoById(areaId);
       if(povertyAreaForm==null){return new ResultVO(ResultEnum.FAILURE);}
        return success(povertyAreaForm);
    }

    @Override
    @PostMapping("/searchByProductGuid")
    public ResultVO searchByProductGuid(@RequestParam(name = "productGuid") String productGuid) {
        List<FarmerInfo> farmerInfos = this.farmerInfoService.queryByProductGuid(productGuid);
        return success(farmerInfos);
    }

    /**
     * 获取农户明细信息
     *
     * @param farmerId
     * @return
     */
    @GetMapping("/getFarmerInfoById")
    @Override
    public ResultVO getFarmerInfoById(@RequestParam("guid") String farmerId) {
        FarmerInfo farmerInfo = farmerInfoService.queryByPrimaryKey(farmerId);
        if(Objects.nonNull(farmerInfo)){
            return success(farmerInfo);
        }
        return failure();
    }

    @Override
    @PostMapping("/searchByCollectiveId")
    public ResultVO searchByCollectiveId(@RequestParam(name = "collectiveId") String collectiveId) {
        List<FarmerInfo> farmerInfos = this.farmerInfoService.queryByCollectiveId(collectiveId);
        return success(farmerInfos);
    }
}
