package com.hcloud.microserver.bank.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.GoodsInfoBO;
import com.hcloud.microserver.bank.service.GoodsInfoService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.facade.carbon.client.GoodsInfoClient;
import com.hcloud.microserver.facade.carbon.entity.GoodsInfo;
import com.hcloud.microserver.facade.carbon.entity.GoodsSaleInfo;
import com.hcloud.microserver.facade.carbon.entity.ProductInfo;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsInfoForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/goods")
public class GoodsInfoManageController extends BaseController implements GoodsInfoClient {



    @Autowired
    private GoodsInfoService goodsInfoService;


    @Autowired
    private RedisUtils redisUtils;

    @Override
    @PostMapping("/shelves")
    public ResultVO shelves(@RequestBody List<GoodsInfoForm> goodsInfoForms) {
        int success = this.goodsInfoService.batchShelves(goodsInfoForms);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/delete")
    public ResultVO delete(@RequestBody GoodsInfoForm goodsInfoForm) {
        int success = this.goodsInfoService.remove(goodsInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/update")
    public ResultVO update(@RequestBody GoodsInfoForm goodsInfoForm) {
        int success = this.goodsInfoService.modify(goodsInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/searchByPage")
    public ResultVO searchByPage(@RequestBody GoodsInfoForm goodsInfoForm) {
        PageInfo pageInfo = this.goodsInfoService.queryListByPage(goodsInfoForm);
        if (Objects.isNull(pageInfo)){ return success(null, 0);}
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    @Override
    @PostMapping("/searchJoinListByPage")
    public ResultVO searchJoinListByPage(@RequestBody GoodsInfoForm goodsInfoForm) {
        PageInfo pageInfo = this.goodsInfoService.queryJoinListByPage(goodsInfoForm);
        if (Objects.isNull(pageInfo)){ return success(null, 0);}
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    /**
     * 分页查询商品类型信息（移动端展示）
     *
     * @param goodsInfoForm
     * @return
     */
    @PostMapping("/searchGoodsInfoByPage")
    @Override
    public ResultVO searchGoodsInfoByPage(@RequestBody GoodsInfoForm goodsInfoForm) {
        PageInfo pageInfo = goodsInfoService.searchGoodsInfoByPage(goodsInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 获取商品详情
     * @param goodsId
     * @return
     */
    @GetMapping("/getGoodsInfoById")
    @Override
    public ResultVO getGoodsInfoById(@RequestParam("goodsId") String goodsId) {
        GoodsInfoBO goodsInfoBO = goodsInfoService.getGoodsInfoByGuid(goodsId);
        return success(goodsInfoBO);
    }

    /**
     * 根据需求碳汇量计算商品列表
     *
     * @param carbon
     * @return
     */
    @GetMapping("/getGoodsInfoByCarbon")
    @Override
    public ResultVO getGoodsInfoByCarbon(@RequestParam("carbon") String carbon) {
        List<GoodsInfoForm> goodsList = goodsInfoService.getGoodsInfoByCarbon(carbon);
        return success(goodsList);
    }

    @Override
    @PostMapping("/updateGoods")
    public ResultVO updateGoods(@RequestBody JSONObject jsonObject) {
        JSONArray jsonArray = jsonObject.getJSONArray("productInfos");
        List<ProductInfo> productInfos = JSONObject.parseArray(jsonArray.toJSONString(), ProductInfo.class);
        GoodsInfoForm goodsInfoForm = jsonObject.getObject("goodsInfoForm", GoodsInfoForm.class);
        boolean cond1 = Objects.nonNull(productInfos) && productInfos.size() > 0;
        boolean cond2 = Objects.nonNull(goodsInfoForm);
        if (cond1 && cond2) {
            GoodsInfo goodsInfo = new GoodsInfo();
            GoodsSaleInfo goodsSaleInfo = new GoodsSaleInfo();
            goodsInfo.setGuid(goodsInfoForm.getGuid());
            goodsInfo.setCarbonVal(goodsInfoForm.getCarbonVal());
            goodsInfo.setPrice(goodsInfoForm.getPrice());
            goodsInfo.setGoodsSaleDate(goodsInfoForm.getGoodsSaleDate());
            goodsInfo.setGoodsUse(goodsInfoForm.getGoodsUse());
            goodsSaleInfo.setSaleTargets(goodsInfoForm.getSaleTargets());
            goodsSaleInfo.setSaleType(goodsInfoForm.getSaleType());
            int success = this.goodsInfoService.modifyGoods(goodsInfo, goodsSaleInfo, productInfos);
            if (success > 0) {
                return success();
            }
        }

        return failure();
    }

    /**
     * 通过农户Id查询所属商品
     *
     * @param farmerInfoForm
     * @return
     */
    @PostMapping("/getGoodsInfoByFarmerId")
    @Override
    public ResultVO getGoodsInfoByFarmerId(@RequestBody FarmerInfoForm farmerInfoForm) {
       PageInfo pageInfo = goodsInfoService.searchGoodsInfoByFarmerId(farmerInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

}
