package com.hcloud.microserver.bank.controller;

import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.bank.bo.CredentialInfoBO;
import com.hcloud.microserver.bank.service.*;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.facade.carbon.client.OrderClient;
import com.hcloud.microserver.facade.carbon.entity.GoodsInfo;
import com.hcloud.microserver.facade.carbon.forms.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/order")

public class OrderInfoManagerController extends BaseController implements OrderClient {

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private OrderDetailInfoService orderDetailInfoService;

    @Autowired
    private CredentialInfoService credentialInfoService;

    @Autowired
    private CustomerInfoService customerInfoService;

    @Autowired
    private GoodsInfoService goodsInfoService;

    /**
     * 分页查询订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/searchOrderInfoPage")
    @Override
    public ResultVO searchOrderInfoByPage(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm) {
        PageInfo pageInfo = orderInfoService.searchOrderInfoByPage(goodsOrderInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 分页获取爱心用户订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/getOrderInfoByCustomerId")
    @Override
    public ResultVO getOrderInfoByCustomerId(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm) {
        PageInfo pageInfo =orderInfoService.searchOrderInfoByCustomerId(goodsOrderInfoForm);
        List list = pageInfo.getList();
        if(null!=list && list.size()>0){
            List<GoodsOrderInfoForm> orderInfoFormList = BeanCopyUtil.convertList(list,GoodsOrderInfoForm.class);
            for (GoodsOrderInfoForm form:orderInfoFormList) {
                String guid = form.getGuid();
                List<GoodsOrderDetailForm> detailFormList =orderDetailInfoService.getOrderDetailListByOrderId(guid);
                form.setGoodsOrderDetailForm(detailFormList);
            }
            return success(orderInfoFormList,pageInfo.getTotal());
        }
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 查询订单明细信息
     *
     * @param orderId
     * @return
     */
    @GetMapping("/getOrderDetail")
    @Override
    public ResultVO getOrderDetailInfoByOrderId(@RequestParam("orderId") String orderId) {
        List<GoodsOrderDetailForm> list =  orderDetailInfoService.getOrderDetailListByOrderId(orderId);
        return success(list);
    }

    /**
     * 获取某订单基本信息
     *
     * @param orderId
     * @return
     */
    @GetMapping("/getOrderInfoById")
    @Override
    public ResultVO getOrderInfoById(@RequestParam("guid") String orderId) {
       GoodsOrderInfoForm goodsOrderInfoForm =  orderInfoService.getOrderInfoById(orderId);
        return success(goodsOrderInfoForm);
    }

    /**
     * 删除订单信息
     *
     * @param orderId
     * @return
     */
    @GetMapping("/delOderInfoById")
    @Override
    public ResultVO delOrderInfo(@RequestParam("guid") String orderId) {
        GoodsOrderInfoForm goodsOrderInfoForm = orderInfoService.getOrderInfoById(orderId);
        if(Objects.nonNull(goodsOrderInfoForm)){
            Integer orderStatus = goodsOrderInfoForm.getOrderStatus();
            if(orderStatus==1){
                int i = orderInfoService.rollbackOrderInfo(orderId);
                if(i==0){return failure();}
            }else {
                int j = orderInfoService.delOrderInfoByLogical(orderId);
                if(j==0){return failure();}
            }
        }
        return success();
    }

    /**
     * 更新订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/modifiedOrderInfo")
    @Override
    public ResultVO updateOrderInfo(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm) {
        int i = orderInfoService.modifyByPrimaryKeySelective(goodsOrderInfoForm);
        if(i==0){return failure();}
        return success();
    }

    /**
     * 统一下单生成商品订单，并生成支付订单。返回支付订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/insertOrderInfoForPay")
    @Override
    public ResultVO saveOrderInfoAndPayOrderInfo(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm) {
        int i = convertOrderInfoForm(goodsOrderInfoForm);
        if(i==1){
            String orderId = orderInfoService.saveFullOrderInfo(goodsOrderInfoForm);
            if(orderId==null){return failure();}
            return success(goodsOrderInfoForm);
        }else if(i==-1){
            return new ResultVO(ResultEnum.GOODS_BUY_AUTH);
        }
        return failure();
    }

    /**
     * 转换客户订单信息,用于生成客户订单(含快照信息)
     * @param goodsOrderInfoForm
     * @return
     */
    private int convertOrderInfoForm(GoodsOrderInfoForm goodsOrderInfoForm){
        //生成订单信息的类别
        List<GoodsInfoForm> goodsList = goodsOrderInfoForm.getGoodsList();
        CustomerInfoForm customerInfoForm = customerInfoService.getCustomerInfoById(goodsOrderInfoForm.getFkCustomerGuid());
        if(Objects.nonNull(customerInfoForm)){
            Integer customerType = customerInfoForm.getCustomerType();
            String  typeName = "个人账户";
            if(customerType==2){
                typeName="履约企业";
            }else if(customerType==3) {
                typeName="普通履约企业";
            }
            goodsOrderInfoForm.setCustomerName(customerInfoForm.getCustomerName());
            goodsOrderInfoForm.setCustomerType(typeName);
            goodsOrderInfoForm.setOrderStatus(1);
            //计算碳汇量及碳汇销售总价,生成订单详情列表
            List<GoodsOrderDetailForm> goodsOrderDetailList = Lists.newArrayList();
            Double amountMoney = 0d;
            Double carbonAmount = 0d;
            if(null!=goodsList && goodsList.size()>0){
                for (GoodsInfoForm bean:goodsList) {
                    BigDecimal carbon = bean.getCarbonSkin();
                    String goodsId = bean.getGuid();
                    Integer goodsUse =Integer.valueOf(bean.getGoodsUse()) ;
                    //判断用户类型是否超出购买权限，如果超出购买权限将不允许直接购买
                    if(customerType==1 && goodsUse==1){
                        return -1;
                    }
                    //有非法商品数量,不生成订单详情
                    if(carbon.intValue()==0 || Objects.isNull(carbon)){continue;}
                    GoodsOrderDetailForm detailForm = new GoodsOrderDetailForm();
                    GoodsInfo goodsInfo = goodsInfoService.queryByPrimaryKey(goodsId);
                    BigDecimal detailMoney = goodsInfo.getPrice().multiply(carbon);
                    detailForm.setFkGoodsGuid(goodsId);
                    detailForm.setAmount(detailMoney);
                    detailForm.setSaleNum(carbon);
                    detailForm.setGoodsName(goodsInfo.getGoodsName());
                    detailForm.setGoodsPrice(goodsInfo.getPrice());
                    carbonAmount+=carbon.doubleValue();
                    amountMoney+=detailMoney.doubleValue();
                    goodsOrderDetailList.add(detailForm);
                }
                goodsOrderInfoForm.setOrderAmount(BigDecimal.valueOf(amountMoney));
                goodsOrderInfoForm.setOrderCarbonAmount(BigDecimal.valueOf(carbonAmount));
                goodsOrderInfoForm.setGoodsOrderDetailForm(goodsOrderDetailList);
                return 1;
            }
        }
        return 0;
    }

    /**
     * 支付回调后，统一更新订单信息及支付订单信息
     *
     * @param payOrderNO
     * @return
     */
    @GetMapping("/modifiedOrderInfoForPay")
    @Override
    public ResultVO updateOrderInfoAndPayOrderInfo(@RequestParam("payNo") String payOrderNO) {
        int i = orderInfoService.updateFullOrderInfo(payOrderNO,2);
        if(i==0){return failure();}
        return success();
    }

    /**
     * 获取用户证书列表
     *
     * @param goodsCredentialInfoForm
     * @return
     */
    @PostMapping("/getCustomerCredentialList")
    @Override
    public ResultVO getCustomerCredentialList(@RequestBody GoodsCredentialInfoForm goodsCredentialInfoForm) {
        PageInfo pageInfo = credentialInfoService.searchCredentialByPage(goodsCredentialInfoForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 订单支付超时取消
     *
     * @param orderId
     * @return
     */
    @GetMapping("/callbackOrderInfo")
    @Override
    public ResultVO callbackOrderInfo(@RequestParam("orderId") String orderId) {
        int i = orderInfoService.rollbackOrderInfo(orderId);
        if(i==0){return failure();}
        return success();
    }

    /**
     * 通过订单编号查询订单信息
     *
     * @param orderNo
     * @return
     */
    @GetMapping("/getOrderInfoByNo")
    @Override
    public ResultVO getOrderInfoByOrderNo(@RequestParam("orderNo") String orderNo) {
      GoodsOrderInfoForm goodsOrderInfoForm =  orderInfoService.getOrderInfoByNo(orderNo);
      if(Objects.nonNull(goodsOrderInfoForm)){
          success(goodsOrderInfoForm);
      }
        return failure();
    }

    /**
     * 取消用户订单
     *
     * @param orderId
     * @return
     */
    @GetMapping("/cancelOrderInfo")
    @Override
    public ResultVO cancelOrderInfo(@RequestParam("orderId") String orderId) {
        //取消订单主体信息

        //取消支付订单信息

        //取消离线支付申请信息

        return null;
    }

    /**
     * 通过碳汇需求量购买指定农户下的碳汇商品
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @Override
    public ResultVO saveOrderInfoForFarmer(GoodsOrderInfoForm goodsOrderInfoForm) {
        //TODO 查询农户下的产品信息

        //TODO 根据需求的碳汇量计算出产品列表
        return null;
    }

    @GetMapping("/getCredential")
    @Override
    public ResultVO getCredentialInfo(@RequestParam(name = "guid") String guid) {
        GoodsCredentialInfoForm goodsCredentialInfoForm = this.credentialInfoService.queryByPrimaryKey(guid);
        return success(goodsCredentialInfoForm);
    }

    @GetMapping("/getCredentialByOrderId")
    @Override
    public ResultVO getCredentialByOrderId(@RequestParam(name = "orderId") String orderId) {
        CredentialInfoBO credentialInfoBO = this.credentialInfoService.queryByOrderId(orderId);
        return success(credentialInfoBO);
    }
}
