package com.hcloud.microserver.bank.controller;


import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.service.OrderInfoService;
import com.hcloud.microserver.bank.service.OrderPayApplyService;
import com.hcloud.microserver.bank.service.OrderPayVoucherService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.client.OrderPayApplyClient;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import com.hcloud.microserver.facade.carbon.forms.OrderPayApplyForm;
import com.hcloud.microserver.facade.carbon.forms.OrderPayVoucherForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/pay/apply")
public class PayApplyManagerController extends BaseController implements OrderPayApplyClient {

    /**
     * 订单未支付状态,
     */
    private static final Integer ORDER_NO_PAY_STATUS=1;

    @Autowired
    private OrderPayApplyService orderPayApplyService;

    @Autowired
    private OrderPayVoucherService orderPayVoucherService;

    @Autowired
    private OrderInfoService orderInfoService;
    /**
     * 分页查询离线支付申请信息
     *
     * @param orderPayApplyForm
     * @return
     */
    @PostMapping("/searchByPage")
    @Override
    public ResultVO searchPayApplyByPage(@RequestBody OrderPayApplyForm orderPayApplyForm) {
       PageInfo pageInfo =  orderPayApplyService.searchPayApplyInfoByPage(orderPayApplyForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    /**
     * 更新离线支付申请信息
     *
     * @param orderPayApplyForm
     * @return
     */
    @PostMapping("/updatePayApply")
    @Override
    public ResultVO updatePayApplyById(@RequestBody OrderPayApplyForm orderPayApplyForm) {
        int i = orderPayApplyService.modifyByPrimaryKeySelective(orderPayApplyForm);
        if (i==0){return failure();}
        return success();
    }

    /**
     * 删除离线支付申请信息
     *
     * @param guid
     * @return
     */
    @GetMapping("/delPayApply")
    @Override
    public ResultVO delPayApply(@RequestParam("guid") String guid) {
        OrderPayApplyForm orderPayApplyForm = orderPayApplyService.queryByPrimaryKey(guid);
        if(orderPayApplyForm!=null){
            orderPayApplyForm.setState(0);
            int i = orderPayApplyService.modifyByPrimaryKeySelective(orderPayApplyForm);
            if(i==0){return failure();}
            return success();
        }
        return failure();
    }

    /**
     * 生成离线支付申请
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @PostMapping("/savePayApplyInfo")
    @Override
    public ResultVO savePayApplyInfo(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm) {
        String orderId =  goodsOrderInfoForm.getGuid();
        if(null!=orderId && !"".equals(orderId)){
          GoodsOrderInfoForm orderInfoForm = orderInfoService.getOrderInfoById(orderId);
          Integer orderSatus = orderInfoForm.getOrderStatus();
          if(orderSatus!=1){
              return new ResultVO(ResultEnum.ORDER_PAY_COMPLETE);
          }
            int i= orderPayApplyService.savePayApplyInfo(goodsOrderInfoForm);
            if(i==0){return failure();}
            return success();
        }
       return failure();
    }

    /**
     * 生成离线支付凭证
     *
     * @param payVoucherForm
     * @return
     */
    @PostMapping("/saveOffLienVoucher")
    @Override
    public ResultVO saveOffLienPayVoucher(@RequestBody OrderPayVoucherForm payVoucherForm) {
        int i = orderPayVoucherService.savePayVoucherInfoAndOrderInfo(payVoucherForm);
        if(i==0){return failure();}
        return success();
    }
}
