package com.hcloud.microserver.bank.controller;

import com.hcloud.microserver.bank.bo.GoodsPayOrderBO;
import com.hcloud.microserver.bank.service.OrderPayService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.client.OrderPayClient;
import com.hcloud.microserver.facade.carbon.forms.GoodsPayOrderForm;
import com.hcloud.microserver.facade.carbon.forms.PaymentForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/order/pay")
public class PayOrderManagerController extends BaseController implements OrderPayClient {

    @Autowired
    private OrderPayService orderPayService;


    /**
     * 查询某订单，支付流水明细
     *
     * @param orderId
     * @return
     */
    @GetMapping("/getListById")
    @Override
    public ResultVO getOrderPayListByOrderId(@RequestParam("orderId") String orderId) {
        List<GoodsPayOrderBO> list =  orderPayService.getPayOrderByOrderId(orderId);
        return success(list);
    }

    /**
     * 保存支付订单信息
     *
     * @param goodsPayOrderForm
     * @return
     */
    @PostMapping("/saveOrderPayInfo")
    @Override
    public ResultVO savePayOrderInfo(@RequestBody GoodsPayOrderForm goodsPayOrderForm) {
       int i =  orderPayService.saveSelective(goodsPayOrderForm);
       if(i==0){return failure();}
        return success();
    }

    /**
     * 更新支付订单明细
     *
     * @param goodsPayOrderForm
     * @return
     */
    @PostMapping("/modifiedOrderPayInfo")
    @Override
    public ResultVO updateOrderPayInfo(@RequestBody GoodsPayOrderForm goodsPayOrderForm) {
        int i = orderPayService.modifyByPrimaryKeySelective(goodsPayOrderForm);
        if(i==0){return failure();}
        return success();
    }

    /**
     * 通过支付订单编号查询支付信息
     *
     * @param payNo
     * @return
     */
    @GetMapping("/getPayOrderInfoByNo")
    @Override
    public ResultVO getPayOrderInfoByPayNo(@RequestParam("payNo") String payNo) {
       GoodsPayOrderForm goodsPayOrderForm = orderPayService.getPayOrderByNo(payNo);
       if(Objects.nonNull(goodsPayOrderForm)){
          return success(goodsPayOrderForm);
       }
        return failure();
    }

    @PostMapping("/queryPaymentByPage")
    @Override
    public ResultVO queryPaymentByPage(@RequestBody PaymentForm paymentForm) {
        return success(orderPayService.queryPaymentByPage(paymentForm));
    }
}
