package com.hcloud.microserver.bank.controller;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.service.ProductBelongService;
import com.hcloud.microserver.bank.service.ProductExpansionService;
import com.hcloud.microserver.bank.service.ProductInfoService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.client.ProductInfoClient;
import com.hcloud.microserver.facade.carbon.entity.*;
import com.hcloud.microserver.facade.carbon.forms.GoodsInfoForm;
import com.hcloud.microserver.facade.carbon.forms.ProductInfoForm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@Slf4j
@RestController
@RequestMapping("/product")
public class ProductInfoManageController extends BaseController implements ProductInfoClient {

    @Autowired
    private ProductInfoService productInfoService;

    @Autowired
    private ProductBelongService productBelongService;

    @Autowired
    private ProductExpansionService productExpansionService;

    @Override
    @PostMapping("/save")
    public ResultVO save(@RequestBody ProductInfoForm productInfoForm) {
        int success = this.productInfoService.save(productInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/delete")
    public ResultVO delete(@RequestBody ProductInfoForm productInfoForm) {
        int success = this.productInfoService.remove(productInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/update")
    public ResultVO update(@RequestBody ProductInfoForm productInfoForm) {
        int success = this.productInfoService.modify(productInfoForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/searchByPage")
    public ResultVO searchByPage(@RequestBody ProductInfoForm productInfoForm) {
        PageInfo pageInfo = this.productInfoService.queryListByPage(productInfoForm);
        if (Objects.isNull(pageInfo)) {
            return success(null, 0);
        }
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    @Override
    @PostMapping("/expansions")
    public ResultVO expansions(@RequestParam("guid") String guid) {
        List<ProductExpansion> expansions = this.productExpansionService.searchByProductGuid(guid);
        return success(expansions);
    }

    @Override
    @PostMapping("/belongs")
    public ResultVO belongs(@RequestParam("guid") String guid) {
        List<ProductBelong> belongs = this.productBelongService.searchByProductGuid(guid);
        return success(belongs);
    }

    @Override
    @PostMapping("/publish")
    public ResultVO publish(@RequestBody JSONObject jsonObject) {
        log.info("参数: {}", jsonObject);
        ProductInfoForm productInfoForm = null;
        GoodsInfoForm goodsInfoForm = null;

        if (jsonObject.containsKey("productInfoForm")) {
            productInfoForm = jsonObject.getObject("productInfoForm", ProductInfoForm.class);
        }
        if (jsonObject.containsKey("goodsInfoForm")) {
            goodsInfoForm = jsonObject.getObject("goodsInfoForm", GoodsInfoForm.class);
        }

        if (Objects.nonNull(productInfoForm) && Objects.nonNull(goodsInfoForm)) {
            int success = this.productInfoService.publish(productInfoForm, goodsInfoForm);
            if (success > 0) {
                return success();
            } else {
                return failure();
            }
        } else {
            return new ResultVO(ResultEnum.PARAMETER_NOT_NULL);
        }
    }

    @Override
    @PostMapping("/publishGoods")
    public ResultVO publishGoods(@RequestBody JSONObject jsonObject) {
        log.info("参数: {}", jsonObject.toJSONString());

        JSONArray jsonArray = jsonObject.getJSONArray("productInfos");
        List<ProductInfo> productInfos = JSONObject.parseArray(jsonArray.toJSONString(), ProductInfo.class);

        GoodsInfoForm goodsInfoForm = jsonObject.getObject("goodsInfoForm", GoodsInfoForm.class);
        log.info("productInfos: {}", productInfos);
        log.info("goodsInfoForm: {}", goodsInfoForm);

        boolean cond1 = Objects.nonNull(productInfos) && productInfos.size() > 0;
        boolean cond2 = Objects.nonNull(goodsInfoForm);

        if (cond1 && cond2) {
            GoodsInfo goodsInfo = new GoodsInfo();
            GoodsSaleInfo goodsSaleInfo = new GoodsSaleInfo();
            goodsInfo.setCarbonVal(goodsInfoForm.getCarbonVal());
            goodsInfo.setPrice(goodsInfoForm.getPrice());
            goodsInfo.setGoodsSaleDate(goodsInfoForm.getGoodsSaleDate());
            goodsInfo.setGoodsUse(goodsInfoForm.getGoodsUse());
            goodsSaleInfo.setSaleTargets(goodsInfoForm.getSaleTargets());
            goodsSaleInfo.setSaleType(goodsInfoForm.getSaleType());

            int success = this.productInfoService.publishGoods(goodsInfo, goodsSaleInfo, productInfos);
            if (success > 0) {
                return success();
            }
        }
        return failure();
    }

    @Override
    @PostMapping("/findByGoodsInfoGuid")
    public ResultVO findByGoodsInfoGuid(@RequestParam("goodsInfoGuid") String goodsInfoGuid) {
        List<ProductInfo> productInfos = this.productInfoService.searchByGoodsInfoGuid(goodsInfoGuid);
        return success(productInfos);
    }

}
