package com.hcloud.microserver.bank.controller;

import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.service.ProductTypeExpansionService;
import com.hcloud.microserver.bank.service.ProductTypeService;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.client.ProductTypeClient;
import com.hcloud.microserver.facade.carbon.entity.ProductType;
import com.hcloud.microserver.facade.carbon.entity.ProductTypeExpansion;
import com.hcloud.microserver.facade.carbon.forms.ProductTypeForm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@Slf4j
@RestController
@RequestMapping("/productType")
public class ProductTypeManageController extends BaseController implements ProductTypeClient {

    @Autowired
    private ProductTypeService productTypeService;

    @Autowired
    private ProductTypeExpansionService productTypeExpansionService;

    @Override
    @PostMapping("/save")
    public ResultVO save(@RequestBody ProductTypeForm productTypeForm) {
        int success = this.productTypeService.add(productTypeForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/delete")
    public ResultVO delete(@RequestBody ProductTypeForm productTypeForm) {
        int success = this.productTypeService.remove(productTypeForm);
        if (success >= 0) {
            return success();
        } else
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/update")
    public ResultVO update(@RequestBody ProductTypeForm productTypeForm) {
        int success = this.productTypeService.modify(productTypeForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @Override
    @PostMapping("/searchByPage")
    public ResultVO searchByPage(@RequestBody ProductTypeForm productTypeForm) {
        PageInfo pageInfo = this.productTypeService.queryListByPage(productTypeForm);
        if (Objects.isNull(pageInfo)){ return success(null, 0);}
        return success(pageInfo.getList(), pageInfo.getTotal());
    }

    @Override
    @PostMapping("/expansions")
    public ResultVO expansions(@RequestParam String guid) {
        log.info("guid: {}", guid);
        List<ProductTypeExpansion> result = this.productTypeExpansionService.searchByProductTypeGuid(guid);
        return success(result);
    }

    @Override
    @PostMapping("/all")
    public ResultVO findAll(@RequestBody(required = false) ProductTypeForm productTypeForm) {
        List<ProductType> productTypes = this.productTypeService.queryAll(productTypeForm);
        return success(productTypes);
    }

    @Override
    public ResultVO checkTypeVal(@RequestParam(name = "typeVal") String typeVal) {
        int count = this.productTypeService.checkTypeVal(typeVal);
        if (count < 1) return success();
        else return new ResultVO(113, "类型值" + typeVal + "已经存在");
    }


}
