package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.dao.CollectiveFarmerMapper;
import com.hcloud.microserver.bank.dao.FarmerInfoMapper;
import com.hcloud.microserver.bank.dao.ProductBelongMapper;
import com.hcloud.microserver.bank.dao.ProductInfoMapper;
import com.hcloud.microserver.bank.service.CollectiveFarmerService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.BusinessCodeRule;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microserver.facade.carbon.entity.CollectiveFarmer;
import com.hcloud.microserver.facade.carbon.entity.FarmerInfo;
import com.hcloud.microserver.facade.carbon.forms.CollectiveFarmerForm;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

@Slf4j
@Service
public class CollectiveFarmerServiceImpl implements CollectiveFarmerService {

    private static String CODE_PREFIX = "C01";
    private static final SimpleDateFormat YEAR_SDF = new SimpleDateFormat("YY");

    @Resource
    private CollectiveFarmerMapper collectiveFarmerMapper;

    @Resource
    private FarmerInfoMapper farmerInfoMapper;

    @Resource
    private ProductInfoMapper productInfoMapper;

    @Resource
    private ProductBelongMapper productBelongMapper;

    @Override
    public int delByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            this.collectiveFarmerMapper.deleteByPrimaryKey(guid);
        }
        return 0;
    }

    @Override
    public int save(CollectiveFarmer record) {
        if (Objects.nonNull(record)) {
            record.setState(1);
            record.setIsEnable(1);
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            return this.collectiveFarmerMapper.insert(record);
        }
        return 0;
    }

    @Override
    public int saveSelective(CollectiveFarmer record) {
        if (Objects.nonNull(record)) {
            record.setState(1);
            record.setIsEnable(1);
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            return this.collectiveFarmerMapper.insertSelective(record);
        }
        return 0;
    }

    @Override
    public CollectiveFarmer queryByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.queryByPrimaryKey(guid);
        }
        return null;
    }

    @Override
    public int modifyByPrimaryKeySelective(CollectiveFarmer record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.collectiveFarmerMapper.updateByPrimaryKeySelective(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(CollectiveFarmer record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.collectiveFarmerMapper.updateByPrimaryKeyWithBLOBs(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(CollectiveFarmer record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.collectiveFarmerMapper.updateByPrimaryKey(record);
            }
        }
        return 0;
    }

    /**
     * 新增贫困集体
     *
     * @param collectiveFarmerForm
     * @return 新的贫困集体
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int save(CollectiveFarmerForm collectiveFarmerForm) {
        int result = 0;
        if (Objects.nonNull(collectiveFarmerForm)) {
            CollectiveFarmer collectiveFarmer = BeanCopyUtil.convertBean(collectiveFarmerForm, CollectiveFarmer.class);
            String collectiveFarmerGuid = UUIDUtils.randomUUID();
            collectiveFarmer.setGuid(collectiveFarmerGuid);
            collectiveFarmer.setModifiedTime(new Date());
            collectiveFarmer.setState(1);
            collectiveFarmer.setCollectiveCode(getMaxCode());

            List<FarmerInfoForm> farmerInfoForms = collectiveFarmerForm.getFarmerInfoForms();
            List<FarmerInfo> farmerInfos = new ArrayList<>();
            if (Objects.nonNull(farmerInfoForms) && farmerInfoForms.size() > 0) {
                for (FarmerInfoForm farmerInfoForm : farmerInfoForms) {
                    FarmerInfo farmerInfo = BeanCopyUtil.convertBean(farmerInfoForm, FarmerInfo.class);
                    farmerInfo.setModifiedTime(new Date());
                    farmerInfo.setFkCollectiveId(collectiveFarmerGuid);
                    farmerInfo.setState(1);
                    farmerInfo.setIsEnable(1);
                    farmerInfos.add(farmerInfo);
                }
                result = farmerInfoMapper.batchUpdate(farmerInfos);
                if (result < 1) throw new RuntimeException("添加贫困集体户失败,批量更新贫困集体户失败");
            }

            collectiveFarmer.setCollectivePersonNum(farmerInfos.size());
            result = this.collectiveFarmerMapper.insertSelective(collectiveFarmer);
        }
        return result;
    }

    /**
     * 删除贫苦集体(软删除)
     *
     * @param collectiveFarmerForm
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int softRemove(CollectiveFarmerForm collectiveFarmerForm) {
        int result = 0;
        if (Objects.nonNull(collectiveFarmerForm)) {
            String guid = collectiveFarmerForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                CollectiveFarmer collectiveFarmer = BeanCopyUtil.convertBean(collectiveFarmerForm, CollectiveFarmer.class);
                // 软删除贫困集体赋值
                collectiveFarmer.setState(0);
                collectiveFarmer.setIsEnable(0);
                collectiveFarmer.setModifiedTime(new Date());
                // 加载贫困集体包含的贫困户
                List<FarmerInfo> farmerInfos = this.farmerInfoMapper.selectByCollectiveId(guid);
                if (Objects.isNull(farmerInfos)) throw new RuntimeException("删除贫困集体失败 -- 加载贫困集体所属贫困户时没有取到值");
                // 贫困户赋值
                for (FarmerInfo farmerInfo : farmerInfos) {
                    farmerInfo.setFkCollectiveId(null);
                    farmerInfo.setModifiedTime(new Date());
                }
                if (Objects.nonNull(farmerInfos) && farmerInfos.size() > 0) {
                    // 批量更新贫困户
                    result = this.farmerInfoMapper.batchUpdate(farmerInfos);
                    if (result < 1) throw new RuntimeException("删除贫困集体失败 -- 批量更新贫困户出错(将贫困户的外键设置为null)");
                }
                // 更新贫困集体
                result = this.collectiveFarmerMapper.updateByPrimaryKey(collectiveFarmer);
            }
        }
        return result;
    }

    /**
     * 修改贫困集体
     *
     * @param collectiveFarmerForm
     * @return
     * @author 朱修贤
     */
    @Override
    @Transactional
    public int modify(CollectiveFarmerForm collectiveFarmerForm) {
        log.info("批量修改的农户数据 =========> {}", collectiveFarmerForm);
        int result = 0;
        if (Objects.nonNull(collectiveFarmerForm)) {
            String guid = collectiveFarmerForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                CollectiveFarmer collectiveFarmer = BeanCopyUtil.convertBean(collectiveFarmerForm, CollectiveFarmer.class);
                collectiveFarmer.setModifiedTime(new Date());
                // 加载贫困集体包含的贫困户
                List<FarmerInfo> farmerInfosInDb = this.farmerInfoMapper.selectByCollectiveId(guid);
                if (Objects.isNull(farmerInfosInDb)) throw new RuntimeException("编辑贫困集体失败 -- 加载贫困集体所属贫困户时没有取到值");

                List<String> farmerInfoGuids = new ArrayList<>();

                // 贫困户赋值
                for (FarmerInfo farmerInfo : farmerInfosInDb) {
                    // 这里设置成空字符串,代表贫困户脱离贫困集体
                    farmerInfo.setFkCollectiveId("#");
                    farmerInfo.setModifiedTime(new Date());
                    // 为了解除贫困户和产品归属的对应关系,必须设置贫困户的guid
                    farmerInfoGuids.add(farmerInfo.getGuid());
                }

                // 批量更新贫困户
                if (Objects.nonNull(farmerInfosInDb) && farmerInfosInDb.size() > 0) {
                    log.info("farmerInfosInDb =========> {}", farmerInfosInDb);
                    result = this.farmerInfoMapper.batchUpdate(farmerInfosInDb);
                    if (result < 0) throw new RuntimeException("编辑贫困集体失败 -- 批量更新贫困户出错(将贫困户的外键设置为null)");
                }

                // 重新关联贫困户
                List<FarmerInfoForm> farmerInfoForms = collectiveFarmerForm.getFarmerInfoForms();
                if (Objects.nonNull(farmerInfoForms) && farmerInfoForms.size() > 0) {
                    List<FarmerInfo> farmerInfosToAdd = new ArrayList<>();
                    for (FarmerInfoForm farmerInfoForm : farmerInfoForms) {
                        FarmerInfo farmerInfo = BeanCopyUtil.convertBean(farmerInfoForm, FarmerInfo.class);
                        farmerInfo.setFkCollectiveId(guid);
                        farmerInfo.setModifiedTime(new Date());
                        farmerInfosToAdd.add(farmerInfo);
                    }
                    log.info("farmerInfosToAdd ==========> {}", farmerInfosToAdd);
                    result = this.farmerInfoMapper.batchUpdate(farmerInfosToAdd);
                    if (result < 1) throw new RuntimeException("编辑贫困集体失败 -- 批量更新贫困户出错(重新关联贫困户时失败)");
                    // 设置贫困户数
                    collectiveFarmer.setCollectivePersonNum(farmerInfosToAdd.size());
                } else {
                    collectiveFarmer.setCollectivePersonNum(0);
                }
                result = this.collectiveFarmerMapper.updateByPrimaryKeySelective(collectiveFarmer);
            }
        }
        return result;
    }

    /**
     * 贫困集体 -- 分页查询
     *
     * @param collectiveFarmerForm
     * @return
     * @author 朱修贤
     */
    @Override
    public PageInfo queryListByPage(CollectiveFarmerForm collectiveFarmerForm) {
        PageHelper.startPage(collectiveFarmerForm.getPageNo(), collectiveFarmerForm.getPageSize());
        Page page = this.collectiveFarmerMapper.selectListByPage(collectiveFarmerForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    /**
     * 审核卡
     *
     * @param collectiveFarmerForm
     * @return
     * @author 朱修贤
     */
    @Override
    public int verify(CollectiveFarmerForm collectiveFarmerForm) {
        // TODO 审核卡
        return 0;
    }

    private Integer getCurrentCode() {
        StringBuffer sbb = new StringBuffer();
        String maxCode = collectiveFarmerMapper.getCurrentCode();
        if (maxCode == null) {
            return 0;
        }
        String yearStr = YEAR_SDF.format(new Date());
        sbb.append(CODE_PREFIX);
        sbb.append(yearStr);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }

    private String getMaxCode() {
        Integer currentNum = getCurrentCode();
        String maxCode = BusinessCodeRule.firstSpecialCodeRule(CODE_PREFIX, currentNum, 25);
        return maxCode;
    }
}
