package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.CredentialInfoBO;
import com.hcloud.microserver.bank.dao.GoodsCredentialInfoMapper;
import com.hcloud.microserver.bank.dao.GoodsOrderInfoMapper;
import com.hcloud.microserver.bank.service.CredentialInfoService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.BusinessCodeRule;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsCredentialInfo;
import com.hcloud.microserver.facade.carbon.entity.GoodsOrderInfo;
import com.hcloud.microserver.facade.carbon.forms.GoodsCredentialInfoForm;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@Service("credentialInfoService")
public class CredentialInfoServiceImpl implements CredentialInfoService {

    /**
     * 证书编码前缀
     */
    private static final String CODE_PREFIX = "CQCSP-PH";

    private static final SimpleDateFormat YEAR_SDF = new SimpleDateFormat("YY");

    @Resource
    private GoodsCredentialInfoMapper goodsCredentialInfoMapper;

    @Resource
    private GoodsOrderInfoMapper goodsOrderInfoMapper;



    /**
     * 生成证书记录
     *
     * @param orderId
     * @return
     */
    @Override
    public int saveCredentialInfo(String orderId) {
        if(null!=orderId && !"".equals(orderId)){
            GoodsCredentialInfoForm goodsCredentialInfoForm = new GoodsCredentialInfoForm();
            //查询订单信息
            GoodsOrderInfo goodsOrderInfo = goodsOrderInfoMapper.selectByPrimaryKey(orderId);
            //查询农户信息
            List<CredentialInfoBO> list =  goodsCredentialInfoMapper.getCredentialFarmerInfoByOrderId(orderId);
           //获取最大证书最大编码
            Integer currentNum = getCurrentCode();
            String maxCode = BusinessCodeRule.notYearCodeRule(CODE_PREFIX,currentNum,15);
            goodsCredentialInfoForm.setFkOrderId(orderId);
            goodsCredentialInfoForm.setFkCustomerId(goodsOrderInfo.getFkCustomerGuid());
            goodsCredentialInfoForm.setBuyDate(new Date());
            goodsCredentialInfoForm.setCarbonAmount(goodsOrderInfo.getOrderCarbonAmount());
            goodsCredentialInfoForm.setCredentialNo(maxCode);
            String farmerNames = "";
            String farmerAreas = "";
            Integer farmerCount=0;
            if(null!=list && list.size()>0){
               for (CredentialInfoBO bo:list) {
                   if(Objects.isNull(bo)){continue;}
                   Integer temp = bo.getFarmerNum()==null?0:bo.getFarmerNum();
                   farmerNames+=bo.getFarmerName()+",";
                   farmerAreas+=bo.getFarmerVillage()+",";
                   farmerCount+=temp;
               }
                goodsCredentialInfoForm.setFarmerNames(farmerNames);
                goodsCredentialInfoForm.setProductAddress(farmerAreas);
                goodsCredentialInfoForm.setFarmerCount(farmerCount);
                goodsCredentialInfoForm.setState(1);
                goodsCredentialInfoForm.setCreateTime(new Date());
            }
            return this.saveSelective(goodsCredentialInfoForm);
        }
        return 0;
    }

    /**
     * 分页查询用户证书信息
     * @param goodsCredentialInfoForm
     * @return
     */
    @Override
    public PageInfo searchCredentialByPage(GoodsCredentialInfoForm goodsCredentialInfoForm) {
        String customId = goodsCredentialInfoForm.getFkCustomerId();
        PageHelper.startPage(goodsCredentialInfoForm.getPageNo(),goodsCredentialInfoForm.getPageSize());
        List<GoodsCredentialInfo> list = goodsCredentialInfoMapper.getCredentialInfoByCustomerId(customId);
        return new PageInfo(list);
    }

    /**
     * 根据订单id查询证书
     *
     * @param orderId
     * @return
     */
    @Override
    public CredentialInfoBO queryByOrderId(String orderId) {
        return this.goodsCredentialInfoMapper.getCredentialInfoByOrderId(orderId);
    }

    /**
     * 获取当前编码流水号的数字形式
     * @return
     */
    private Integer getCurrentCode(){
        StringBuffer sbb = new StringBuffer();
        String maxCode = goodsCredentialInfoMapper.getMaxCode();
        if(maxCode==null){return 500;}
        sbb.append(CODE_PREFIX);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }


    @Override
    public int delByPrimaryKey(String guid) {
        return goodsCredentialInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsCredentialInfoForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        GoodsCredentialInfo goodsCredentialInfo = BeanCopyUtil.convertBean(record,GoodsCredentialInfo.class);
        return goodsCredentialInfoMapper.insert(goodsCredentialInfo);
    }

    @Override
    public int saveSelective(GoodsCredentialInfoForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        GoodsCredentialInfo goodsCredentialInfo = BeanCopyUtil.convertBean(record,GoodsCredentialInfo.class);
        return goodsCredentialInfoMapper.insertSelective(goodsCredentialInfo);
    }

    @Override
    public GoodsCredentialInfoForm queryByPrimaryKey(String guid) {
        GoodsCredentialInfo goodsCredentialInfo = goodsCredentialInfoMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(goodsCredentialInfo,GoodsCredentialInfoForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsCredentialInfoForm record) {
        GoodsCredentialInfo goodsCredentialInfo = BeanCopyUtil.convertBean(record,GoodsCredentialInfo.class);
        return goodsCredentialInfoMapper.updateByPrimaryKeySelective(goodsCredentialInfo);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsCredentialInfoForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsCredentialInfoForm record) {
        GoodsCredentialInfo goodsCredentialInfo = BeanCopyUtil.convertBean(record,GoodsCredentialInfo.class);
        return goodsCredentialInfoMapper.updateByPrimaryKey(goodsCredentialInfo);
    }
}
