package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.dao.CustomerCompanyInfoMapper;
import com.hcloud.microserver.bank.dao.CustomerInfoMapper;
import com.hcloud.microserver.bank.service.CustomerCompanyInfoService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.string.ConvertStringToList;
import com.hcloud.microserver.facade.carbon.entity.CustomerCompanyInfo;
import com.hcloud.microserver.facade.carbon.entity.CustomerInfo;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@Transactional(rollbackFor = Exception.class)
@Service("customerCompanyInfoService")
public class CustomerCompanyInfoServicImpl implements CustomerCompanyInfoService {


    @Resource
    private CustomerCompanyInfoMapper customerCompanyInfoMapper;

    @Resource
    private CustomerInfoMapper customerInfoMapper;

    @Override
    public CustomerCompanyInfoForm getCustomerCompanyInfoByCustomerId(String customerId) {
       CustomerCompanyInfo customerCompanyInfo =  customerCompanyInfoMapper.getCustomerCompanyInfoByCID(customerId);
        if(customerCompanyInfo!=null){
            CustomerCompanyInfoForm companyInfoForm = BeanCopyUtil.convertBean(customerCompanyInfo,CustomerCompanyInfoForm.class);
            String imgStr = customerCompanyInfo.getCompanyImages();
            if(imgStr.indexOf(",")!=-1){
               List<String> imgList = ConvertStringToList.getListFormSplitString(imgStr,",");
                companyInfoForm.setImageList(imgList);
            }
            return companyInfoForm;
        }
        return null;
    }

    @Override
    public PageInfo queryListByPage(CustomerCompanyInfoForm customerCompanyInfoForm) {
        PageHelper.startPage(customerCompanyInfoForm.getPageNo(), customerCompanyInfoForm.getPageSize());
        Page page = this.customerCompanyInfoMapper.selectListByPage(customerCompanyInfoForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    /**
     * 企业认证
     *
     * @param customerCompanyInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int validateCompanyInfo(CustomerCompanyInfoForm customerCompanyInfoForm) {
        String guid = customerCompanyInfoForm.getGuid();
        String customerId = customerCompanyInfoForm.getFkCustomerId();
        CustomerInfo customerInfo = new CustomerInfo();
        customerInfo.setCustomerType(3);
        customerInfo.setGuid(customerId);
        customerInfo.setModifiedTime(new Date());
        customerInfo.setIsCompany(1);
        int k = customerInfoMapper.updateByPrimaryKeySelective(customerInfo);
        if(k>0){
            CustomerCompanyInfo customerCompanyInfo =
                    BeanCopyUtil.convertBean(customerCompanyInfoForm,CustomerCompanyInfo.class);
            if(null!=guid && !"".equals(guid)){
                int i =customerCompanyInfoMapper.updateByPrimaryKeySelective(customerCompanyInfo);
                if(i==0){throw new RuntimeException();}
            }else{
                customerCompanyInfo.setGuid(GetUUID.getUUID());
                customerCompanyInfo.setFkCustomerId(customerId);
                customerCompanyInfo.setState(1);
                customerCompanyInfo.setCreateTime(new Date());
                customerCompanyInfo.setModifiedTime(new Date());
                int j =customerCompanyInfoMapper.insertSelective(customerCompanyInfo);
                if(j==0){throw new RuntimeException();}
            }
        }
        return k;
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return customerCompanyInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(CustomerCompanyInfoForm record) {
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(record,CustomerCompanyInfo.class);
        customerCompanyInfo.setGuid(GetUUID.getUUID());
        return customerCompanyInfoMapper.insert(customerCompanyInfo);
    }

    @Override
    public int saveSelective(CustomerCompanyInfoForm record) {
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(record,CustomerCompanyInfo.class);
        customerCompanyInfo.setGuid(GetUUID.getUUID());
        return customerCompanyInfoMapper.insertSelective(customerCompanyInfo);
    }

    @Override
    public CustomerCompanyInfoForm queryByPrimaryKey(String guid) {
        CustomerCompanyInfo customerCompanyInfo = customerCompanyInfoMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(customerCompanyInfo,CustomerCompanyInfoForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(CustomerCompanyInfoForm record) {
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(record,CustomerCompanyInfo.class);
        return customerCompanyInfoMapper.updateByPrimaryKeySelective(customerCompanyInfo);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(CustomerCompanyInfoForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(CustomerCompanyInfoForm record) {
        CustomerCompanyInfo customerCompanyInfo = BeanCopyUtil.convertBean(record,CustomerCompanyInfo.class);
        return customerCompanyInfoMapper.updateByPrimaryKeySelective(customerCompanyInfo);
    }
}
