package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.hcloud.microserver.bank.bo.GoodsInfoBO;
import com.hcloud.microserver.bank.dao.*;
import com.hcloud.microserver.bank.service.GoodsInfoService;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.BusinessCodeRule;
import com.hcloud.microserver.commoncore.util.UUIDUtils;
import com.hcloud.microserver.facade.carbon.entity.GoodsDetail;
import com.hcloud.microserver.facade.carbon.entity.GoodsInfo;
import com.hcloud.microserver.facade.carbon.entity.GoodsSaleInfo;
import com.hcloud.microserver.facade.carbon.entity.ProductInfo;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsDetailForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsInfoForm;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author zhuxx
 */
@Service
public class GoodsInfoServiceImpl implements GoodsInfoService {

    private static String G_CODE_PREFIX = "G01";
    private static final SimpleDateFormat YEAR_SDF = new SimpleDateFormat("YY");

    @Resource
    private GoodsInfoMapper goodsInfoMapper;

    @Resource
    private GoodsDetailMapper goodsDetailMapper;

    @Resource
    private ProductInfoMapper productInfoMapper;

    @Resource
    private ProductBelongMapper productBelongMapper;

    @Resource
    private ProductTypeExpansionMapper productTypeExpansionMapper;

    @Resource
    private ProductExpansionMapper productExpansionMapper;

    @Resource
    private GoodsSaleInfoMapper goodsSaleInfoMapper;

    @Override
    public int delByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.goodsInfoMapper.deleteByPrimaryKey(guid);
        }
        return 0;
    }

    @Override
    public int save(GoodsInfo record) {
        if (Objects.nonNull(record)) {
            record.setGuid(UUIDUtils.randomUUID());
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            record.setState(1);
            return this.goodsInfoMapper.insert(record);
        }
        return 0;
    }

    @Override
    public int saveSelective(GoodsInfo record) {
        if (Objects.nonNull(record)) {
            record.setGuid(UUIDUtils.randomUUID());
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            record.setState(1);
            return this.goodsInfoMapper.insertSelective(record);
        }
        return 0;
    }

    @Override
    public GoodsInfo queryByPrimaryKey(String guid) {
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.goodsInfoMapper.selectByPrimaryKey(guid);
        }
        return null;
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.goodsInfoMapper.updateByPrimaryKeySelective(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.goodsInfoMapper.updateByPrimaryKeyWithBLOBs(record);
            }
        }
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsInfo record) {
        if (Objects.nonNull(record)) {
            String guid = record.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                record.setModifiedTime(new Date());
                return this.goodsInfoMapper.updateByPrimaryKey(record);
            }
        }
        return 0;
    }

    @Override
    @Transactional
    public int batchShelves(List<GoodsInfoForm> goodsInfoForms) {
        int result = 0;
        if (Objects.nonNull(goodsInfoForms) && goodsInfoForms.size() > 0) {
            List<GoodsInfo> goodsInfos = new ArrayList<>();
            for (GoodsInfoForm goodsInfoForm : goodsInfoForms) {
                String guid = goodsInfoForm.getGuid();
                if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                    GoodsInfo goodsInfo = BeanCopyUtil.convertBean(goodsInfoForm, GoodsInfo.class);
                    goodsInfo.setModifiedTime(new Date());
                    goodsInfos.add(goodsInfo);
                    if (goodsInfo.getIsSale() == 0) { // 下架时,将对应商品改成可生成商品状态
                        List<ProductInfo> productInfos = this.productInfoMapper.selectByGoodsInfoGuid(goodsInfo.getGuid());
                        if (Objects.nonNull(productInfos) && productInfos.size() > 0) {
                            ProductInfo productInfo = productInfos.get(0);
                            productInfo.setIsEnable(1);
                            result = this.productInfoMapper.updateByPrimaryKeySelective(productInfo);
                            if (result < 1) throw new RuntimeException("更新状态商品失败");
                        }
                    } else if (goodsInfo.getIsSale() == 1) { // 上架时,将对应产品改成不可生成状态
                        List<ProductInfo> productInfos = this.productInfoMapper.selectByGoodsInfoGuid(goodsInfo.getGuid());
                        if (Objects.nonNull(productInfos) && productInfos.size() > 0) {
                            ProductInfo productInfo = productInfos.get(0);
                            productInfo.setIsEnable(2);
                            result = this.productInfoMapper.updateByPrimaryKeySelective(productInfo);
                            if (result < 1) throw new RuntimeException("更新状态商品失败");
                        }
                    }
                } else {
                    throw new RuntimeException("批量更新失败");
                }
            }
            result = this.goodsInfoMapper.batchUpdate(goodsInfos);
            if (result < 1) throw new RuntimeException("批量上架下架商品失败");
        }
        return result;
    }

    @Override
    @Transactional
    public int save(GoodsInfoForm goodsInfoForm) {
        int result = 0;
        if (Objects.nonNull(goodsInfoForm)) {
            GoodsInfo goodsInfo = BeanCopyUtil.convertBean(goodsInfoForm, GoodsInfo.class);
            String guid = UUIDUtils.randomUUID();
            goodsInfo.setGuid(guid);
            goodsInfo.setCreateTime(new Date());
            goodsInfo.setModifiedTime(new Date());
            goodsInfo.setState(1);

            List<GoodsDetailForm> goodsDetailForms = goodsInfoForm.getGoodsDetailForms();
            if (Objects.nonNull(goodsDetailForms) && goodsDetailForms.size() > 0) {
                List<GoodsDetail> goodsDetails = new ArrayList<>();
                for (GoodsDetailForm goodsDetailForm : goodsDetailForms) {
                    GoodsDetail goodsDetail = BeanCopyUtil.convertBean(goodsDetailForm, GoodsDetail.class);
                    goodsDetail.setGuid(UUIDUtils.randomUUID());
                    goodsDetail.setFkGoodsGuid(guid);
                    goodsDetail.setState(1);
                    goodsDetail.setCreateTime(new Date());
                    goodsDetail.setModifiedTime(new Date());
                    goodsDetails.add(goodsDetail);
                }
                result = this.goodsDetailMapper.batchInsert(goodsDetails);
                if (result < 0) throw new RuntimeException("新增商品失败 -- 批量新增商品明细失败");
            }
            result = this.goodsInfoMapper.insert(goodsInfo);
        }
        return result;
    }

    @Override
    @Transactional
    public int remove(GoodsInfoForm goodsInfoForm) {
        int result = 0;
        if (Objects.nonNull(goodsInfoForm)) {
            String guid = goodsInfoForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                GoodsInfo goodsInfo = BeanCopyUtil.convertBean(goodsInfoForm, GoodsInfo.class);
                goodsInfo.setState(0);
                goodsInfo.setModifiedTime(new Date());
                result = this.goodsInfoMapper.updateByPrimaryKeySelective(goodsInfo);
                if (result < 0) throw new RuntimeException("软删除商品失败");
                result = this.goodsDetailMapper.deleteByGoodsInfoGuid(guid);
                if (result < 1) throw new RuntimeException("删除商品明细失败");
                result = this.goodsSaleInfoMapper.deleteByGoodsInfoGuid(guid);
                if (result < 1) throw new RuntimeException("删除商品销售信息失败");
            }
        }
        return result;
    }

    @Override
    @Transactional
    public int modify(GoodsInfoForm goodsInfoForm) {
        int result = 0;
        if (Objects.nonNull(goodsInfoForm)) {
            String guid = goodsInfoForm.getGuid();
            if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
                GoodsInfo goodsInfo = BeanCopyUtil.convertBean(goodsInfoForm, GoodsInfo.class);
                goodsInfo.setModifiedTime(new Date());
                result = this.goodsInfoMapper.updateByPrimaryKey(goodsInfo);
                if (result < 1) throw new RuntimeException("更新商品失败");
                // 删除数据库中所有的商品明细
                result = this.goodsDetailMapper.deleteByGoodsInfoGuid(guid);
                if (result < 1) throw new RuntimeException("修改商品失败 -- 删除商品明细失败");

                List<GoodsDetailForm> goodsDetailForms = goodsInfoForm.getGoodsDetailForms();
                if (Objects.nonNull(goodsDetailForms) && goodsDetailForms.size() > 0) {
                    List<GoodsDetail> goodsDetails = new ArrayList<>();
                    for (GoodsDetailForm goodsDetailForm : goodsDetailForms) {
                        GoodsDetail goodsDetail = BeanCopyUtil.convertBean(goodsDetailForm, GoodsDetail.class);
                        goodsDetail.setGuid(UUIDUtils.randomUUID());
                        goodsDetail.setCreateTime(new Date());
                        goodsDetail.setModifiedTime(new Date());
                        goodsDetail.setState(1);
                        goodsDetail.setIsEnable(1);
                        goodsDetails.add(goodsDetail);
                    }
                    result = this.goodsDetailMapper.batchInsert(goodsDetails);
                    if (result < 1) throw new RuntimeException("修改商品失败 -- 新增商品明细失败");
                }
            }
        }
        return result;
    }

    @Override
    public PageInfo queryListByPage(GoodsInfoForm goodsInfoForm) {
        PageHelper.startPage(goodsInfoForm.getPageNo(), goodsInfoForm.getPageSize());
        Page page = this.goodsInfoMapper.selectListByPage(goodsInfoForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    @Override
    public PageInfo queryJoinListByPage(GoodsInfoForm goodsInfoForm) {
        PageHelper.startPage(goodsInfoForm.getPageNo(), goodsInfoForm.getPageSize());
        Page page = this.goodsInfoMapper.selectJoinListByPage(goodsInfoForm);
        if (Objects.nonNull(page) && page.size() > 0){ return new PageInfo(page);}
        return null;
    }

    @Override
    public PageInfo searchGoodsInfoByPage(GoodsInfoForm goodsInfoForm) {
        PageBean pageBean = new PageBean(goodsInfoForm);
        PageHelper.startPage(goodsInfoForm.getPageNo(),goodsInfoForm.getPageSize());
        List<GoodsInfoBO> list = goodsInfoMapper.searchGoodsInfoByPage(pageBean);
        return new PageInfo(list);
    }

    @Override
    public GoodsInfoBO getGoodsInfoByGuid(String guid) {
        GoodsInfoBO goodsInfoBO = null;
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            GoodsInfo goodsInfo = this.goodsInfoMapper.selectByGuid(guid);
            goodsInfoBO = BeanCopyUtil.convertBean(goodsInfo, GoodsInfoBO.class);
            List<ProductInfo> productInfos = this.productInfoMapper.selectByGoodsInfoGuid(guid);
            goodsInfoBO.setProductInfos(productInfos);
        }
        return goodsInfoBO;
    }

    /**
     * 通过需求碳汇量获取商品信息列表
     *
     * @param carbon
     * @return
     */
    @Override
    public List<GoodsInfoForm> getGoodsInfoByCarbon(String carbon) {
        if (null!=carbon && !"".equals(carbon)){
            BigDecimal carbonSkin = BigDecimal.valueOf(Double.valueOf(carbon));
            int compInt = carbonSkin.intValue();
            GoodsInfoForm goodsInfoForm = new GoodsInfoForm();
            goodsInfoForm.setPageSize(30);
            PageBean pageBean = new PageBean(goodsInfoForm);
            PageHelper.startPage(goodsInfoForm.getPageNo(),goodsInfoForm.getPageSize());
            List<GoodsInfoBO> goodsList = goodsInfoMapper.searchGoodsInfoByPage(pageBean);
            if(null!=goodsList && goodsList.size()>0){
                if(carbonSkin.intValue()==0){
                    return BeanCopyUtil.convertList(goodsList,GoodsInfoForm.class);
                }else{
                    List<GoodsInfoForm> list = Lists.newArrayList();
                    int subCount =0;
                    for (GoodsInfoBO bo:goodsList) {
                        int tmp = bo.getUnsaledCarbonSkin().intValue();
                        subCount = tmp-compInt;
                        if(subCount>=0){
                            list.add(BeanCopyUtil.convertBean(bo,GoodsInfoForm.class));
                            break;
                        }else if(subCount<0){
                            list.add(BeanCopyUtil.convertBean(bo,GoodsInfoForm.class));
                            compInt = Math.abs(subCount);
                            continue;
                        }
                    }
                    return list;
                }
            }
        }
        return null;
    }

    @Override
    @Transactional
    public int modifyGoods(GoodsInfo goodsInfo, GoodsSaleInfo goodsSaleInfo, List<ProductInfo> productInfos) {
        int result = 0;
        String guidInfoGuid = goodsInfo.getGuid();
        ProductInfo productInfo = productInfos.get(0);

        goodsInfo.setState(1);
        goodsInfo.setModifiedTime(new Date());

        result = this.goodsSaleInfoMapper.deleteByGoodsInfoGuid(guidInfoGuid);
        if (result < 0) throw new RuntimeException("删除商品销售信息失败");

        result = this.goodsDetailMapper.deleteByGoodsInfoGuid(guidInfoGuid);
        if (result < 0) throw new RuntimeException("删除商品明细失败");

        result = this.goodsInfoMapper.updateByPrimaryKeySelective(goodsInfo);
        if (result < 1) throw new RuntimeException("更新商品失败");

        GoodsDetail goodsDetail = new GoodsDetail();
        goodsDetail.setGuid(UUIDUtils.randomUUID());
        goodsDetail.setFkGoodsGuid(guidInfoGuid);
        goodsDetail.setCreateTime(new Date());
        goodsDetail.setModifiedTime(new Date());
        goodsDetail.setState(1);
        goodsDetail.setIsEnable(1);
        goodsDetail.setCarbonVal(productInfo.getCarbonSink());
        goodsDetail.setFkProductGuid(productInfo.getGuid());
        goodsDetail.setProductName(productInfo.getProductName());
        goodsDetail.setFarmerNums(productInfo.getProductBelongs().size());
        goodsDetail.setSaleUnit("公斤");
        goodsDetail.setProductCode(productInfo.getProductCode());
        result = this.goodsDetailMapper.insert(goodsDetail);
        if (result < 1) throw new RuntimeException("添加商品明细失败");


        goodsSaleInfo.setFkGoodsGuid(guidInfoGuid);
        goodsSaleInfo.setSalePrice(goodsInfo.getPrice());
        goodsSaleInfo.setGuid(UUIDUtils.randomUUID());
        goodsSaleInfo.setIsEnable(1);
        goodsSaleInfo.setState(1);
        goodsSaleInfo.setUnsaledCarbonSkin(productInfo.getCarbonSink());
        goodsSaleInfo.setSaledCarbonSkin(new BigDecimal(0));
        goodsSaleInfo.setModifiedTime(new Date());
        result = goodsSaleInfoMapper.insertSelective(goodsSaleInfo);
        if (result < 1) throw new RuntimeException("添加商品销售信息失败");

        return result;
    }

    private Integer getCurrentGoodsCode() {
        StringBuffer sbb = new StringBuffer();
        String maxCode = this.goodsInfoMapper.getCurrentCode();
        if (maxCode == null) {
            return 0;
        }
        String yearStr = YEAR_SDF.format(new Date());
        sbb.append(G_CODE_PREFIX);
        sbb.append(yearStr);
        String currentNum = maxCode.substring(sbb.toString().length());
        return Integer.valueOf(currentNum);
    }

    private String getMaxGoodsCode() {
        Integer currentNum = getCurrentGoodsCode();
        String maxCode = BusinessCodeRule.firstSpecialCodeRule(G_CODE_PREFIX, currentNum, 20);
        return maxCode;
    }

    /**
     * 通过农户Id查询农户所参与商品信息(分页查询)
     *
     * @param farmerInfoForm
     * @return
     */
    @Override
    public PageInfo searchGoodsInfoByFarmerId(FarmerInfoForm farmerInfoForm) {
        PageHelper.startPage(farmerInfoForm.getPage(),farmerInfoForm.getPageSize());
        List<GoodsInfoBO> list = goodsInfoMapper.searchGoodsInfoByFarmerId(farmerInfoForm.getGuid());
        return new PageInfo(list);
    }
}
