package com.hcloud.microserver.bank.service.impl;

import com.hcloud.microserver.bank.bo.OrderDetailInfoBO;
import com.hcloud.microserver.bank.dao.GoodsOrderDetailMapper;
import com.hcloud.microserver.bank.service.OrderDetailInfoService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsOrderDetail;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderDetailForm;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author xiezt
 */
@Service("orderDetailInfoService")
public class OrderDetailInfoServiceImpl implements OrderDetailInfoService {

    @Resource
    private GoodsOrderDetailMapper goodsOrderDetailMapper;

    /**
     * 通过订单Id查询订单明细
     *
     * @param orderId
     * @return
     */
    @Override
    public List<GoodsOrderDetailForm> getOrderDetailListByOrderId(String orderId) {
        List<OrderDetailInfoBO> list =goodsOrderDetailMapper.getOrderDetailListByOrderId(orderId);
        if(null!=list && list.size()>0){
            return BeanCopyUtil.convertList(list,GoodsOrderDetailForm.class);
        }
        return null;
    }

    /**
     * 批量插入订单明细列表
     *
     * @param batchList
     * @return
     */
    @Override
    public int batchSaveOrderDetailByList(List<GoodsOrderDetailForm> batchList) {
        if(null!= batchList && batchList.size()>0){
            List<GoodsOrderDetail> resultList = BeanCopyUtil.convertList(batchList,GoodsOrderDetail.class);
            return goodsOrderDetailMapper.batchSaveOrderDetailList(resultList);
        }
        return 0;
    }

    @Override
    public List<GoodsOrderDetail> findByGoodsOrderInfoGuid(String goodsOrderInfoGuid) {
        return this.goodsOrderDetailMapper.selectByGoodsOrderInfoGuid(goodsOrderInfoGuid);
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsOrderDetailMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsOrderDetailForm record) {
        record.setGuid(GetUUID.getUUID());
        GoodsOrderDetail goodsOrderDetail = BeanCopyUtil.convertBean(record,GoodsOrderDetail.class);
        return goodsOrderDetailMapper.insert(goodsOrderDetail);
    }

    @Override
    public int saveSelective(GoodsOrderDetailForm record) {
        record.setGuid(GetUUID.getUUID());
        GoodsOrderDetail goodsOrderDetail = BeanCopyUtil.convertBean(record,GoodsOrderDetail.class);
        return goodsOrderDetailMapper.insertSelective(goodsOrderDetail);
    }

    @Override
    public GoodsOrderDetailForm queryByPrimaryKey(String guid) {
        GoodsOrderDetail goodsOrderDetail = goodsOrderDetailMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(goodsOrderDetail,GoodsOrderDetailForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsOrderDetailForm record) {
        GoodsOrderDetail goodsOrderDetail = BeanCopyUtil.convertBean(record,GoodsOrderDetail.class);
        return goodsOrderDetailMapper.updateByPrimaryKeySelective(goodsOrderDetail);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsOrderDetailForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsOrderDetailForm record) {
        GoodsOrderDetail goodsOrderDetail = BeanCopyUtil.convertBean(record,GoodsOrderDetail.class);
        return goodsOrderDetailMapper.updateByPrimaryKey(goodsOrderDetail);
    }
}
