package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.OrderInfoBO;
import com.hcloud.microserver.bank.dao.GoodsOrderInfoMapper;
import com.hcloud.microserver.bank.events.CredentialInfoEvent;
import com.hcloud.microserver.bank.service.*;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GeneratorIdUtils;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsInfo;
import com.hcloud.microserver.facade.carbon.entity.GoodsOrderInfo;
import com.hcloud.microserver.facade.carbon.forms.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author xiezt
 */
@Slf4j
@Transactional(rollbackFor = Exception.class)
@Service("orderInfoService")
public class OrderInfoServiceImpl implements OrderInfoService {

    @Resource
    private GoodsOrderInfoMapper goodsOrderInfoMapper;

    @Autowired
    private OrderDetailInfoService orderDetailInfoService;

    @Autowired
    private OrderPayApplyService orderPayApplyService;

    @Autowired
    private OrderPayService orderPayService;

    @Autowired
    private CustomerInfoService customerInfoService;

    @Autowired
    private ApplicationContext applicationContext;

    @Autowired
    private OrderSaleInfoService orderSaleInfoService;

    @Autowired
    private FarmerInfoService farmerInfoService;

    @Autowired
    private CollectiveFarmerService collectiveFarmerService;

    @Autowired
    private GoodsInfoService goodsInfoService;


    /**
     * 分页查询订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @Override
    public PageInfo searchOrderInfoByPage(GoodsOrderInfoForm goodsOrderInfoForm) {
        PageBean pageBean = new PageBean(goodsOrderInfoForm);
        PageHelper.startPage(goodsOrderInfoForm.getPageNo(),goodsOrderInfoForm.getPageSize());
        List<OrderInfoBO> list =goodsOrderInfoMapper.searchOrderInfoByPage(pageBean);
        return new PageInfo(list);
    }

    /**
     * 通过用户Id查询订单信息(分页信息)
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @Override
    public PageInfo searchOrderInfoByCustomerId(GoodsOrderInfoForm goodsOrderInfoForm) {
        String customerId= goodsOrderInfoForm.getFkCustomerGuid();
        Integer orderStatus = goodsOrderInfoForm.getOrderStatus();
        log.info("page====================>{}",goodsOrderInfoForm.getPage());
        PageHelper.startPage(goodsOrderInfoForm.getPageNo(),goodsOrderInfoForm.getPageSize());
        List<OrderInfoBO> list = goodsOrderInfoMapper.searchOrderInfoByCustomerId(customerId,orderStatus);
        return new PageInfo(list);
    }

    /**
     * 获取订单信息
     *
     * @param orderId
     * @return
     */
    @Override
    public GoodsOrderInfoForm getOrderInfoById(String orderId) {
        GoodsOrderInfoForm goodsOrderInfoForm = new GoodsOrderInfoForm();
        //查询订单信息
        GoodsOrderInfo goodsOrderInfo = goodsOrderInfoMapper.selectByPrimaryKey(orderId);
        if(null!=goodsOrderInfo && !"".equals(goodsOrderInfo.getOrderNo())){
            BeanCopyUtil.copyBean(goodsOrderInfo,goodsOrderInfoForm);
        }
        //查询订单明细信息
        List<GoodsOrderDetailForm> list = orderDetailInfoService.getOrderDetailListByOrderId(orderId);
        goodsOrderInfoForm.setGoodsOrderDetailForm(list);
        //查询支付流水信息
        String payNo = goodsOrderInfoForm.getPayNo();
        if(null!=payNo && !"".equals(payNo)){
           GoodsPayOrderForm goodsPayOrderForm = orderPayService.getPayOrderByNo(payNo);
            goodsOrderInfoForm.setPayOrderInfo(goodsPayOrderForm);
        }
        //客户信息
        CustomerInfoForm customerInfoForm = customerInfoService.getCustomerInfoById(goodsOrderInfoForm.getFkCustomerGuid());
        goodsOrderInfoForm.setCustomerInfoForm(customerInfoForm);
        return goodsOrderInfoForm;
    }

    /**
     * 逻辑删除订单
     *
     * @param orderId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int delOrderInfoByLogical(String orderId) {
        //更新订单信息
        GoodsOrderInfo goodsOrderInfo = new GoodsOrderInfo();
        goodsOrderInfo.setGuid(orderId);
        goodsOrderInfo.setState(0);
        goodsOrderInfo.setModifiedTime(new Date());
        int i = goodsOrderInfoMapper.updateByPrimaryKeySelective(goodsOrderInfo);
        if(i>0){
            //更新离线支付申请
           OrderPayApplyForm orderPayApplyForm =  orderPayApplyService.getPayApplyInfoByOrderId(orderId);
           if(Objects.nonNull(orderPayApplyForm)){
              int j = orderPayApplyService.modifyByPrimaryKeySelective(orderPayApplyForm);
              if(j==0){throw new RuntimeException();}
           }

        }
        return i;
    }

    /**
     * 统一下单(生成订单,订单详情,支付订单)
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public String saveFullOrderInfo(GoodsOrderInfoForm goodsOrderInfoForm) {
        //生成订单信息
        int i = this.saveSelective(goodsOrderInfoForm);
        if(i>0){
            //生成订单明细
           List<GoodsOrderDetailForm> list =  goodsOrderInfoForm.getGoodsOrderDetailForm();
           if(null!=list && list.size()>0){
               for (GoodsOrderDetailForm bean:list) {
                   bean.setFkOrderGuid(goodsOrderInfoForm.getGuid());
                   bean.setState(1);
                   bean.setModifiedTime(new Date());
                   bean.setCreateTime(new Date());
               }
              int j =  orderDetailInfoService.batchSaveOrderDetailByList(list);
              if (j==0){throw new RuntimeException();}
           }else {
               throw new RuntimeException("order detail list is empty!!!!!");
            }
            //更新商品销售表
            if(null!=list && list.size()>0){
                for (GoodsOrderDetailForm bean:list) {
                    BigDecimal saleCarbon = bean.getSaleNum();
                    String goodsGuid = bean.getFkGoodsGuid();
                    GoodsSaleInfoForm goodsSaleInfoForm = orderSaleInfoService.getGoodsSaleInfoByGoodsId(goodsGuid);
                    if(Objects.isNull(goodsSaleInfoForm)){throw new RuntimeException("order sale info is null!!!!!!!");}
                    BigDecimal saledCarbonSkin = goodsSaleInfoForm.getSaledCarbonSkin().add(saleCarbon);
                    BigDecimal unSaleCarbonSkin = goodsSaleInfoForm.getUnsaledCarbonSkin().subtract(saleCarbon);
                    //判断一下商品可售卖的碳汇量是否已经为0,如果为0应该把该商品进行下线操作
                    if(unSaleCarbonSkin.intValue()==0){
                        GoodsInfo goodsInfo = new GoodsInfo();
                        goodsInfo.setGuid(goodsGuid);
                        goodsInfo.setModifiedTime(new Date());
                        goodsInfo.setIsSale(0);
                        int k = goodsInfoService.modifyByPrimaryKeySelective(goodsInfo);
                        if(k==0){throw new RuntimeException();}
                    }
                    goodsSaleInfoForm.setSaledCarbonSkin(saledCarbonSkin);
                    goodsSaleInfoForm.setUnsaledCarbonSkin(unSaleCarbonSkin);
                    goodsSaleInfoForm.setModifiedTime(new Date());
                    int h =orderSaleInfoService.modifyByPrimaryKeySelective(goodsSaleInfoForm);
                    if(h==0){
                        throw new RuntimeException();
                    }
                }
            }
            return goodsOrderInfoForm.getGuid();
        }
        return null;
    }

    /**
     * 支付成功后回调更新（订单信息，支付流水信息，证书信息,农户销售统计信息(集体户统计到集体户上)）
     *
     * @param payNo
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int updateFullOrderInfo(String payNo,Integer state) {
        if(null!=payNo && !"".equals(payNo)){
            //查询支付订单
           GoodsPayOrderForm goodsPayOrderForm = orderPayService.getPayOrderByNo(payNo);
           if(goodsPayOrderForm!=null && !"".equals(goodsPayOrderForm.getFkOrderId())){
               //更新订单信息
               String orderId = goodsPayOrderForm.getFkOrderId();
               GoodsOrderInfo goodsOrderInfo = goodsOrderInfoMapper.selectByPrimaryKey(orderId);
               goodsOrderInfo.setOrderStatus(state);
               goodsOrderInfo.setModifiedTime(new Date());
               int i = goodsOrderInfoMapper.updateByPrimaryKeySelective(goodsOrderInfo);
               if(i>0){
                   //更新支付订单信息
                   goodsPayOrderForm.setPayStatus(state);
                   goodsPayOrderForm.setPayTime(new Date());
                   int j = orderPayService.modifyByPrimaryKeySelective(goodsPayOrderForm);
                   if(j==0){throw new RuntimeException();}
                   //更新农户收入信息
                   updateFarmerSaleInfo(orderId);
                   //生成证书信息
                   applicationContext.publishEvent(new CredentialInfoEvent(new Object(),orderId));
                   return i;
               }else{
                   throw new RuntimeException();
               }
           }
        }
        return 0;
    }

    /**
     * 统计订单详情里商品关联的农户信息，在支付成功后更新农户销售信息
     * @param orderId
     * @throws RuntimeException
     */
    private void updateFarmerSaleInfo(String orderId) throws RuntimeException{
        List<GoodsOrderDetailForm> detailList = orderDetailInfoService.getOrderDetailListByOrderId(orderId);
        if(null!=detailList && detailList.size()>0){
            for (GoodsOrderDetailForm bean:detailList) {
                String goodsGuid = bean.getFkGoodsGuid();
                BigDecimal carbonAmount = bean.getAmount();
                int i =farmerInfoService.updateFarmerSaleInfoByGoodsId(carbonAmount,goodsGuid);
                if(i==0){throw new RuntimeException();}
            }
        }
    }


    /**
     * 支付失败或超过订单支付时间时，进行订单回滚操作
     * @param orderId
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int rollbackOrderInfo(String orderId){
        GoodsOrderInfo goodsOrderInfo = goodsOrderInfoMapper.selectByPrimaryKey(orderId);
        goodsOrderInfo.setOrderStatus(4);
        goodsOrderInfo.setState(0);
        goodsOrderInfo.setModifiedTime(new Date());
        int i = goodsOrderInfoMapper.updateByPrimaryKeySelective(goodsOrderInfo);
        if(i==0){throw new RuntimeException();}
        String guid = goodsOrderInfo.getGuid();
        List<GoodsOrderDetailForm> orderDetailList = orderDetailInfoService.getOrderDetailListByOrderId(guid);
        if(null!=orderDetailList && orderDetailList.size()>0){
            for(GoodsOrderDetailForm bean:orderDetailList) {
                BigDecimal saleCarbon = bean.getSaleNum();
                String goodsGuid = bean.getFkGoodsGuid();
                GoodsSaleInfoForm goodsSaleInfoForm = orderSaleInfoService.getGoodsSaleInfoByGoodsId(goodsGuid);
                BigDecimal saledCarbonSkin = goodsSaleInfoForm.getSaledCarbonSkin().subtract(saleCarbon);
                BigDecimal unSaleCarbonSkin = goodsSaleInfoForm.getUnsaledCarbonSkin().add(saleCarbon);
                goodsSaleInfoForm.setSaledCarbonSkin(saledCarbonSkin);
                goodsSaleInfoForm.setUnsaledCarbonSkin(unSaleCarbonSkin);
                goodsSaleInfoForm.setModifiedTime(new Date());
                int h =orderSaleInfoService.modifiedOrderSaleInfo(goodsSaleInfoForm);
                if(h==0){
                    throw new RuntimeException();
                }
            }
        }
        //取消离线支付订单
        OrderPayApplyForm payApplyForm = orderPayApplyService.getPayApplyInfoByOrderId(orderId);
        if(Objects.nonNull(payApplyForm)){
            payApplyForm.setState(0);
            payApplyForm.setApplyStatus(4);
            payApplyForm.setModifiedTime(new Date());
           int h = orderPayApplyService.modifyByPrimaryKeySelective(payApplyForm);
           if(h==0){throw new RuntimeException();}
        }
        return i;
    }

    /**
     * 通过订单编号查询订单信息
     *
     * @param orderNo
     * @return
     */
    @Override
    public GoodsOrderInfoForm getOrderInfoByNo(String orderNo) {
       GoodsOrderInfo orderInfo = goodsOrderInfoMapper.getOrderInfoByNo(orderNo);
       if(Objects.nonNull(orderInfo)){
           return BeanCopyUtil.convertBean(orderInfo,GoodsOrderInfoForm.class);
       }
        return null;
    }

    /**
     * 为离线支付申请生成订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @Override
    public GoodsOrderInfoForm saveOrderInfoForOffline(GoodsOrderInfoForm goodsOrderInfoForm) {
        int i = this.saveSelective(goodsOrderInfoForm);
        if(i>0){
            //生成订单明细
            List<GoodsOrderDetailForm> detaillist =  goodsOrderInfoForm.getGoodsOrderDetailForm();
            if(null!=detaillist && detaillist.size()>0){
                for (GoodsOrderDetailForm bean:detaillist) {
                    bean.setFkOrderGuid(goodsOrderInfoForm.getGuid());
                    bean.setState(1);
                    bean.setModifiedTime(new Date());
                    bean.setCreateTime(new Date());
                }
                int j =  orderDetailInfoService.batchSaveOrderDetailByList(detaillist);
                if (j==0){throw new RuntimeException();}
            }else {
                throw new RuntimeException("order detail list is empty!!!!!");
            }
            //更新商品销售表
            if(null!=detaillist && detaillist.size()>0){
                for (GoodsOrderDetailForm bean:detaillist) {
                    BigDecimal saleCarbon = bean.getSaleNum();
                    String goodsGuid = bean.getFkGoodsGuid();
                    GoodsSaleInfoForm goodsSaleInfoForm = orderSaleInfoService.queryByPrimaryKey(goodsGuid);
                    BigDecimal saledCarbonSkin = goodsSaleInfoForm.getSaledCarbonSkin().add(saleCarbon);
                    BigDecimal unSaleCarbonSkin = goodsSaleInfoForm.getUnsaledCarbonSkin().subtract(saleCarbon);
                    goodsSaleInfoForm.setSaledCarbonSkin(saledCarbonSkin);
                    goodsSaleInfoForm.setUnsaledCarbonSkin(unSaleCarbonSkin);
                    goodsSaleInfoForm.setModifiedTime(new Date());
                    int h =orderSaleInfoService.modifiedOrderSaleInfo(goodsSaleInfoForm);
                    if(h==0){
                        throw new RuntimeException();
                    }
                }
            }
            return goodsOrderInfoForm;
        }
        return null;
    }


    @Override
    public int delByPrimaryKey(String guid) {
        return goodsOrderInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsOrderInfoForm record) {
        GoodsOrderInfo goodsOrderInfo = BeanCopyUtil.convertBean(record,GoodsOrderInfo.class);
        GeneratorIdUtils generatorIdUtils = new GeneratorIdUtils();
        String orderNum = generatorIdUtils.nextId();
        goodsOrderInfo.setGuid(GetUUID.getUUID());
        goodsOrderInfo.setOrderNo(orderNum);
        record.setState(1);
        record.setCreateTime(new Date());
        record.setModifiedTime(new Date());
        return goodsOrderInfoMapper.insert(goodsOrderInfo);
    }

    @Override
    public int saveSelective(GoodsOrderInfoForm record) {
        GeneratorIdUtils generatorIdUtils = new GeneratorIdUtils();
        String orderNum = generatorIdUtils.nextId();
        //返回生成的订单id和订单编号
        record.setGuid(GetUUID.getUUID());
        record.setOrderNo(orderNum);
        record.setState(1);
        record.setCreateTime(new Date());
        record.setModifiedTime(new Date());
        GoodsOrderInfo goodsOrderInfo = BeanCopyUtil.convertBean(record,GoodsOrderInfo.class);
        return goodsOrderInfoMapper.insertSelective(goodsOrderInfo);
    }

    @Override
    public GoodsOrderInfoForm queryByPrimaryKey(String guid) {
        GoodsOrderInfo goodsOrderInfo = goodsOrderInfoMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(goodsOrderInfo,GoodsOrderInfoForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsOrderInfoForm record) {
        GoodsOrderInfo goodsOrderInfo = BeanCopyUtil.convertBean(record,GoodsOrderInfo.class);
        return goodsOrderInfoMapper.updateByPrimaryKeySelective(goodsOrderInfo);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsOrderInfoForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsOrderInfoForm record) {
        GoodsOrderInfo goodsOrderInfo = BeanCopyUtil.convertBean(record,GoodsOrderInfo.class);
        return goodsOrderInfoMapper.updateByPrimaryKey(goodsOrderInfo);
    }
}
