package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.PayApplyInfoBO;
import com.hcloud.microserver.bank.dao.GoodsOrderInfoMapper;
import com.hcloud.microserver.bank.dao.OrderPayApplyMapper;
import com.hcloud.microserver.bank.service.OrderInfoService;
import com.hcloud.microserver.bank.service.OrderPayApplyService;
import com.hcloud.microserver.bank.service.OrderPayService;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GeneratorIdUtils;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsOrderInfo;
import com.hcloud.microserver.facade.carbon.entity.OrderPayApply;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsPayOrderForm;
import com.hcloud.microserver.facade.carbon.forms.OrderPayApplyForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author xiezt
 */
@Transactional(rollbackFor = Exception.class)
@Service("orderPayApplyService")
public class OrderPayApplyServiceImpl implements OrderPayApplyService {

    @Resource
    private OrderPayApplyMapper orderPayApplyMapper;

    @Autowired
    private GoodsOrderInfoMapper goodsOrderInfoMapper;

    @Autowired
    private OrderInfoService orderInfoService;

    @Autowired
    private OrderPayService orderPayService;

    /**
     * 分页查询离线支付申请信息
     *
     * @param orderPayApplyForm
     * @return
     */
    @Override
    public PageInfo searchPayApplyInfoByPage(OrderPayApplyForm orderPayApplyForm) {
        PageBean pageBean = new PageBean(orderPayApplyForm);
        List<PayApplyInfoBO> list = orderPayApplyMapper.searchPayApplyInfoByPage(pageBean);
        return new PageInfo(list);
    }

    /**
     * 生成离线支付申请信息
     *
     * @param orderInfoForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int savePayApplyInfo(GoodsOrderInfoForm orderInfoForm) {
        //获取已生成的订单信息
        GoodsOrderInfoForm goodsOrderInfoForm = orderInfoService.getOrderInfoById(orderInfoForm.getGuid());
        if(Objects.nonNull(goodsOrderInfoForm)){
            //更新订单状态为待审核订单
            goodsOrderInfoForm.setOrderStatus(3);
            goodsOrderInfoForm.setModifiedTime(new Date());
            int k = orderInfoService.modifyByPrimaryKeySelective(goodsOrderInfoForm);
            if(k==0){throw new RuntimeException();}
            //生成支付订单,支付渠道为线下支付
            GoodsPayOrderForm goodsPayOrderForm = new GoodsPayOrderForm();
            GeneratorIdUtils generatorIdUtils = new GeneratorIdUtils();
            String orderNum = generatorIdUtils.nextId();
            goodsPayOrderForm.setPayNo(orderNum);
            goodsPayOrderForm.setPayStatus(1);
            goodsPayOrderForm.setFkCustomerId(goodsOrderInfoForm.getFkCustomerGuid());
            goodsPayOrderForm.setFkOrderId(goodsOrderInfoForm.getGuid());
            goodsPayOrderForm.setOrderNo(goodsOrderInfoForm.getOrderNo());
            goodsPayOrderForm.setPayChannel(4);
            goodsPayOrderForm.setPayAmount(goodsOrderInfoForm.getOrderAmount());
            goodsPayOrderForm.setPayTime(new Date());
            int h = orderPayService.saveSelective(goodsPayOrderForm);
            if(h==0){throw new RuntimeException();}
            //生成离线支付申请
            OrderPayApplyForm record = new OrderPayApplyForm();
            record.setApplyStatus(1);
            record.setApplyUserName(goodsOrderInfoForm.getCustomerName());
            record.setFkCustomerId(goodsOrderInfoForm.getFkCustomerGuid());
            record.setFkOrderId(goodsOrderInfoForm.getGuid());
            record.setPayAmount(goodsOrderInfoForm.getOrderAmount());
            record.setRemark(orderInfoForm.getApplyRemark());
            record.setState(1);
            record.setCreateTime(new Date());
            record.setModifiedTime(new Date());
            int j = this.saveSelective(record);
            if(j==0){throw new RuntimeException();}
        }else{
            return 0;
        }
        return 1;
    }

    /**
     * 查询离线支付申请信息，通过订单Id
     *
     * @param orderId
     * @return
     */
    @Override
    public OrderPayApplyForm getPayApplyInfoByOrderId(String orderId) {
        OrderPayApply orderPayApply = orderPayApplyMapper.getPayApplyInfoByOrderId(orderId);
        if(Objects.nonNull(orderPayApply)){
            return BeanCopyUtil.convertBean(orderPayApply,OrderPayApplyForm.class);
        }
        return null;
    }

    @Override
    public PageInfo searchByPage(OrderPayApplyForm orderPayApplyForm) {
        PageHelper.startPage(orderPayApplyForm.getPageNo(), orderPayApplyForm.getPageSize());
        Page<Map> page = this.orderPayApplyMapper.selectListByPage(orderPayApplyForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    @Override
    public int audit(OrderPayApply orderPayApply, GoodsOrderInfo goodsOrderInfo) {
        int result = 0;
        // 离线支付记录状态设置为2.已审核
        orderPayApply.setApplyStatus(2);
        // 订单状态改成4,已审核未确认收款
        goodsOrderInfo.setOrderStatus(4);
        result = this.orderPayApplyMapper.updateByPrimaryKeySelective(orderPayApply);
        if (result < 1) throw new RuntimeException("离线支付记录审核失败");
        result = this.goodsOrderInfoMapper.updateByPrimaryKeySelective(goodsOrderInfo);
        if (result < 1) throw new RuntimeException("订单状态修改失败");

        return result;
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return orderPayApplyMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(OrderPayApplyForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        OrderPayApply orderPayApply = BeanCopyUtil.convertBean(record,OrderPayApply.class);
        return orderPayApplyMapper.insert(orderPayApply);
    }

    @Override
    public int saveSelective(OrderPayApplyForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        OrderPayApply orderPayApply = BeanCopyUtil.convertBean(record,OrderPayApply.class);
        return orderPayApplyMapper.insertSelective(orderPayApply);
    }

    @Override
    public OrderPayApplyForm queryByPrimaryKey(String guid) {
        OrderPayApply orderPayApply = orderPayApplyMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(orderPayApply,OrderPayApplyForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(OrderPayApplyForm record) {
        OrderPayApply orderPayApply = BeanCopyUtil.convertBean(record,OrderPayApply.class);
        return orderPayApplyMapper.updateByPrimaryKeySelective(orderPayApply);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(OrderPayApplyForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(OrderPayApplyForm record) {
        OrderPayApply orderPayApply = BeanCopyUtil.convertBean(record,OrderPayApply.class);
        return orderPayApplyMapper.updateByPrimaryKey(orderPayApply);
    }
}
