package com.hcloud.microserver.bank.service.impl;

import com.hcloud.microserver.bank.bo.GoodsPayOrderBO;
import com.hcloud.microserver.bank.bo.PaymentBO;
import com.hcloud.microserver.bank.dao.GoodsPayOrderMapper;
import com.hcloud.microserver.bank.service.OrderPayService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsPayOrder;
import com.hcloud.microserver.facade.carbon.forms.GoodsPayOrderForm;
import com.hcloud.microserver.facade.carbon.forms.PaymentForm;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author xiezt
 */
@Service("orderPayService")
public class OrderPayServiceImpl implements OrderPayService {

    @Resource
    private GoodsPayOrderMapper goodsPayOrderMapper;

    /**
     * 通过订单支付流水号，查询支付流水信息
     *
     * @param payNo
     * @return
     */
    @Override
    public GoodsPayOrderForm getPayOrderByNo(String payNo) {
        GoodsPayOrder goodsPayOrder = goodsPayOrderMapper.getPayOrderByNo(payNo);
        return BeanCopyUtil.convertBean(goodsPayOrder,GoodsPayOrderForm.class);
    }

    /**
     * 获取订单流水列表信息
     * @param orderId
     * @return
     */
    @Override
    public List<GoodsPayOrderBO> getPayOrderByOrderId(String orderId) {
        List<GoodsPayOrder> list = goodsPayOrderMapper.getPayOrderByOrderId(orderId);
        if(null!=list && list.size()>0){
            return BeanCopyUtil.convertList(list,GoodsPayOrderBO.class);
        }
        return null;
    }

    /**
     * 对外提供的支付数据
     * @param paymentForm
     * @return
     */
    @Override
    public List<PaymentBO> queryPaymentByPage(PaymentForm paymentForm) {
        return goodsPayOrderMapper.selectPaymentByPage(paymentForm);
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsPayOrderMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsPayOrderForm record) {
        record.setGuid(GetUUID.getUUID());
        GoodsPayOrder goodsPayOrder = BeanCopyUtil.convertBean(record,GoodsPayOrder.class);
        return goodsPayOrderMapper.insert(goodsPayOrder);
    }

    @Override
    public int saveSelective(GoodsPayOrderForm record) {
        record.setGuid(GetUUID.getUUID());
        GoodsPayOrder goodsPayOrder = BeanCopyUtil.convertBean(record,GoodsPayOrder.class);
        return goodsPayOrderMapper.insertSelective(goodsPayOrder);
    }

    @Override
    public GoodsPayOrderForm queryByPrimaryKey(String guid) {
        GoodsPayOrder goodsPayOrder = goodsPayOrderMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(goodsPayOrder,GoodsPayOrderForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsPayOrderForm record) {
        GoodsPayOrder goodsPayOrder = BeanCopyUtil.convertBean(record,GoodsPayOrder.class);
        return goodsPayOrderMapper.updateByPrimaryKeySelective(goodsPayOrder);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsPayOrderForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsPayOrderForm record) {
        GoodsPayOrder goodsPayOrder = BeanCopyUtil.convertBean(record,GoodsPayOrder.class);
        return goodsPayOrderMapper.updateByPrimaryKey(goodsPayOrder);
    }
}
