package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.dao.GoodsOrderInfoMapper;
import com.hcloud.microserver.bank.dao.OrderPayVoucherMapper;
import com.hcloud.microserver.bank.events.CredentialInfoEvent;
import com.hcloud.microserver.bank.service.OrderPayVoucherService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsOrderInfo;
import com.hcloud.microserver.facade.carbon.entity.OrderPayVoucher;
import com.hcloud.microserver.facade.carbon.forms.OrderPayVoucherForm;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author xiezt
 */
@Transactional(rollbackFor = Exception.class)
@Service("orderPayVoucherService")
public class OrderPayVoucherServiceImpl implements OrderPayVoucherService {


    @Resource
    private OrderPayVoucherMapper orderPayVoucherMapper;

    @Resource
    private GoodsOrderInfoMapper orderInfoMapper;

    @Autowired
    private ApplicationContext applicationContext;

    /**
     * 通过订单Id查询离线支付凭证
     *
     * @param orderId
     * @return
     */
    @Override
    public List<OrderPayVoucherForm> getPayVoucherInfoByOrderId(String orderId) {
        List<OrderPayVoucher> list = orderPayVoucherMapper.getPayVoucherInfoByOrderId(orderId);
        return BeanCopyUtil.convertList(list,OrderPayVoucherForm.class);
    }

    /**
     * 生成离线支付凭证信息
     *
     * @param orderPayVoucherForm
     * @return
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public int savePayVoucherInfoAndOrderInfo(OrderPayVoucherForm orderPayVoucherForm) {
        //更新订单状态，变为已支付
        GoodsOrderInfo orderInfo = new GoodsOrderInfo();
        orderInfo.setGuid(orderPayVoucherForm.getFkOrderId());
        orderInfo.setOrderStatus(2);
        orderInfo.setModifiedTime(new Date());
        int i = orderInfoMapper.updateByPrimaryKeySelective(orderInfo);
        if(i>0){
            //生成离线支付凭证信息
            int j =this.saveSelective(orderPayVoucherForm);
            if(j==0){throw new RuntimeException();}
        }
        applicationContext.publishEvent(new CredentialInfoEvent(new Object(),orderPayVoucherForm.getFkOrderId()));
        return i;
    }

    @Override
    public PageInfo queryListByPage(OrderPayVoucherForm orderPayVoucherForm) {
        PageHelper.startPage(orderPayVoucherForm.getPageNo(), orderPayVoucherForm.getPageSize());
        Page<Map> page = this.orderPayVoucherMapper.selectListByPage(orderPayVoucherForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return orderPayVoucherMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(OrderPayVoucherForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setCreateTime(new Date());
        record.setModifiedTime(new Date());
        record.setState(1);
        OrderPayVoucher orderPayVoucher = BeanCopyUtil.convertBean(record,OrderPayVoucher.class);
        return orderPayVoucherMapper.insert(orderPayVoucher);
    }

    @Override
    public int saveSelective(OrderPayVoucherForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setCreateTime(new Date());
        record.setModifiedTime(new Date());
        record.setState(1);
        OrderPayVoucher orderPayVoucher = BeanCopyUtil.convertBean(record,OrderPayVoucher.class);
        return orderPayVoucherMapper.insertSelective(orderPayVoucher);
    }

    @Override
    public OrderPayVoucherForm queryByPrimaryKey(String guid) {
        OrderPayVoucher orderPayVoucher = orderPayVoucherMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(orderPayVoucher,OrderPayVoucherForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(OrderPayVoucherForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        OrderPayVoucher orderPayVoucher = BeanCopyUtil.convertBean(record,OrderPayVoucher.class);
        return orderPayVoucherMapper.updateByPrimaryKeySelective(orderPayVoucher);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(OrderPayVoucherForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(OrderPayVoucherForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        OrderPayVoucher orderPayVoucher = BeanCopyUtil.convertBean(record,OrderPayVoucher.class);
        return orderPayVoucherMapper.updateByPrimaryKey(orderPayVoucher);
    }
}
