package com.hcloud.microserver.bank.service.impl;

import com.hcloud.microserver.bank.dao.GoodsSaleInfoMapper;
import com.hcloud.microserver.bank.service.OrderSaleInfoService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.facade.carbon.entity.GoodsSaleInfo;
import com.hcloud.microserver.facade.carbon.forms.GoodsSaleInfoForm;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.Objects;

/**
 * @author xiezt
 */
@Service("orderSaleInfoService")
public class OrderSaleInfoServiceImpl implements OrderSaleInfoService {


    @Resource
    private GoodsSaleInfoMapper goodsSaleInfoMapper;

    /**
     * 更新商品销售信息
     *
     * @param goodsSaleInfoForm
     * @return
     */
    @Override
    public int modifiedOrderSaleInfo(GoodsSaleInfoForm goodsSaleInfoForm) {
        return goodsSaleInfoMapper.updateSaleInfo(goodsSaleInfoForm);
    }

    /**
     * 通过商品Id查询商品销售信息
     *
     * @param goodsId
     * @return
     */
    @Override
    public GoodsSaleInfoForm getGoodsSaleInfoByGoodsId(String goodsId) {
        GoodsSaleInfo goodsSaleInfo = goodsSaleInfoMapper.getGoodsSaleInfoByGoodsId(goodsId);
        if(Objects.nonNull(goodsSaleInfo)){
          return  BeanCopyUtil.convertBean(goodsSaleInfo,GoodsSaleInfoForm.class);
        }
        return null;
    }

    @Override
    public int delByPrimaryKey(String guid) {
        return goodsSaleInfoMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(GoodsSaleInfoForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        GoodsSaleInfo goodsSaleInfo = BeanCopyUtil.convertBean(record,GoodsSaleInfo.class);
        return goodsSaleInfoMapper.insert(goodsSaleInfo);
    }

    @Override
    public int saveSelective(GoodsSaleInfoForm record) {
        record.setGuid(GetUUID.getUUID());
        record.setModifiedTime(new Date());
        GoodsSaleInfo goodsSaleInfo = BeanCopyUtil.convertBean(record,GoodsSaleInfo.class);
        return goodsSaleInfoMapper.insertSelective(goodsSaleInfo);
    }

    @Override
    public GoodsSaleInfoForm queryByPrimaryKey(String guid) {
        GoodsSaleInfo goodsSaleInfo = goodsSaleInfoMapper.selectByPrimaryKey(guid);
        return BeanCopyUtil.convertBean(goodsSaleInfo,GoodsSaleInfoForm.class);
    }

    @Override
    public int modifyByPrimaryKeySelective(GoodsSaleInfoForm record) {
        record.setModifiedTime(new Date());
        GoodsSaleInfo goodsSaleInfo = BeanCopyUtil.convertBean(record,GoodsSaleInfo.class);
        return goodsSaleInfoMapper.updateByPrimaryKeySelective(goodsSaleInfo);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(GoodsSaleInfoForm record) {
        //empty implements
        return 0;
    }

    @Override
    public int modifyByPrimaryKey(GoodsSaleInfoForm record) {
        record.setModifiedTime(new Date());
        GoodsSaleInfo goodsSaleInfo = BeanCopyUtil.convertBean(record,GoodsSaleInfo.class);
        return goodsSaleInfoMapper.updateByPrimaryKey(goodsSaleInfo);
    }
}
