package com.hcloud.microserver.bank.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.bank.bo.PovertyAreaBO;
import com.hcloud.microserver.bank.dao.FarmerInfoMapper;
import com.hcloud.microserver.bank.dao.PovertyAreaMapper;
import com.hcloud.microserver.bank.service.PovertyAreaService;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.facade.carbon.entity.FarmerInfo;
import com.hcloud.microserver.facade.carbon.entity.PovertyArea;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.PovertyAreaForm;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;

/**
 * @author xiezt
 */
@Service
public class PovertyAreaServiceImpl implements PovertyAreaService {

    @Resource
    private PovertyAreaMapper povertyAreaMapper;

    @Resource
    private FarmerInfoMapper farmerInfoMapper;

    @Override
    public int save(PovertyAreaForm povertyAreaForm) {
        if (Objects.nonNull(povertyAreaForm)) {
            PovertyArea povertyArea = BeanCopyUtil.convertBean(povertyAreaForm, PovertyArea.class);
            List<PovertyArea> existsList = this.povertyAreaMapper.findByFiveIds(povertyArea);
            if (Objects.isNull(existsList) || (Objects.nonNull(existsList) && existsList.size() > 0)) {
                throw new RuntimeException("新增区域失败, 该区域已经存在");
            }
            povertyArea.setGuid(UUID.randomUUID().toString().replaceAll("-", ""));
            Date now = new Date();
            povertyArea.setCreateTime(now);
            povertyArea.setModifiedTime(now);
            povertyArea.setState(1);
            povertyArea.setIsEnable(1);
            return this.povertyAreaMapper.insert(povertyArea);
        }
        return 0;
    }

    @Override
    public int remove(PovertyAreaForm povertyAreaForm) {
        PovertyArea povertyArea = BeanCopyUtil.convertBean(povertyAreaForm, PovertyArea.class);
        String guid = povertyArea.getGuid();
        povertyArea.setState(0);
        povertyArea.setIsEnable(0);
        povertyArea.setModifiedTime(new Date());
        if (StringUtils.isNotEmpty(guid) && guid.matches("^\\w{32}$")) {
            return this.povertyAreaMapper.updateByPrimaryKey(povertyArea);
        }
        return 0;
    }

    @Override
    public int modify(PovertyAreaForm povertyAreaForm) {
        if (Objects.nonNull(povertyAreaForm) && StringUtils.isNotEmpty(povertyAreaForm.getGuid()) && povertyAreaForm.getGuid().matches("^\\w{32}$")) {
            PovertyArea povertyArea = BeanCopyUtil.convertBean(povertyAreaForm, PovertyArea.class);
            povertyArea.setState(1);
            povertyArea.setIsEnable(1);
            povertyArea.setModifiedTime(new Date());
            return this.povertyAreaMapper.updateByPrimaryKey(povertyArea);
        }
        return 0;
    }

    @Override
    public PageInfo<Page<Map>> queryListByPage(PovertyAreaForm povertyAreaForm) {
        //PageBean<PovertyAreaForm> povertyAreaPageBean = new PageBean<>();
        PageHelper.startPage(povertyAreaForm.getPageNo(), povertyAreaForm.getPageSize());
        Page<Map> page = povertyAreaMapper.selectPovertyAreaListByPage(povertyAreaForm);
        if (Objects.nonNull(page) && page.size() > 0) {
            return new PageInfo(page);
        }
        return null;
    }

    @Override
    public List<PovertyArea> areaTree() {
        return null;
    }

    @Override
    public PovertyArea findByGuid(String guid) {
        return this.povertyAreaMapper.selectByPrimaryKey(guid);
    }

    /**
     * 获取贫困区域明细(含统计信息,含贫困区域农户信息)
     *
     * @param guid
     * @return
     */
    @Override
    public PovertyAreaForm getFullPovertyAreaInfoById(String guid) {
        //查询出贫困统计信息
        PovertyAreaBO povertyArea = povertyAreaMapper.getPovertyAreaAnalysis(guid);
        if(povertyArea!=null && !"".equals(povertyArea.getGuid())){
            PovertyAreaForm povertyAreaForm = BeanCopyUtil.convertBean(povertyArea,PovertyAreaForm.class);
            //查询贫困户列表信息
            List<FarmerInfo> list = farmerInfoMapper.getFarmerInfoByPovertyArea(guid);
            List<FarmerInfoForm> resultList = BeanCopyUtil.convertList(list,FarmerInfoForm.class);
            povertyAreaForm.setFarmerInfoList(resultList);
            return povertyAreaForm;
        }
        return null;
    }
}
