package com.hcloud.microserver.sys.facade.client;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.facade.fallback.impl.SysManagerFallbackFactory;
import com.hcloud.microserver.sys.facade.forms.SysMangerForm;
import com.hcloud.microserver.sys.facade.forms.SysUserForm;
import org.springframework.cloud.netflix.feign.FeignClient;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * @author JYJ
 */
@FeignClient(name = "system-service", path = "/user", fallbackFactory = SysManagerFallbackFactory.class)
public interface SysUserClient {

    /**
     * 通过账号查询用户信息
     *
     * @param account
     * @return
     */
    @PostMapping("/findUserByAccount")
    ResultVO<SysUser> queryUserInfoByAccount(@RequestParam("account") String account);

    /**
     * 用户登录
     *
     * @param account
     * @param password
     * @return
     */
    @PostMapping("/login")
    ResultVO<Map> login(@RequestParam("account") String account, @RequestParam("password") String password);


    /**
     * 退出登录
     * @param accessToken
     * @return
     */
    @GetMapping("/logout")
    ResultVO logout(@RequestParam("token") String accessToken);

    /**
     * 添加用户信息
     *
     * @param sysUserForm
     * @return
     */
    @PostMapping("/saveUserInfo")
    ResultVO saveUserInfo(@RequestBody SysUserForm sysUserForm);

    /**
     * 更新用户信息
     *
     * @param sysUserForm
     * @return
     */
    @PostMapping("/updateUserInfo")
    ResultVO updateUserInfo(@RequestBody SysUserForm sysUserForm);


    /**
     * 分页显示用户信息列表
     *
     * @param sysMangerForm
     * @return
     */
    @PostMapping("/queryUserListByPage")
    ResultVO queryUserListByPage(@RequestBody SysMangerForm sysMangerForm);


    /**
     * 查询用户基本权限信息
     * @param sysMangerForm
     * @return
     */
    @GetMapping("/userPermissionTree")
    ResultVO queryUserPermissionById(@RequestBody SysMangerForm sysMangerForm);

    /**
     * 查询部门及角色信息
     * @param refSysApp
     * @return
     */
    @GetMapping("/selectList")
    ResultVO getDeptAndRoleInfo(@RequestParam("sysApp") String refSysApp);

    /**
     * 更新用户权限信息
     * @param sysUserForm
     * @return
     */
    @PostMapping("/updateUserPermission")
    ResultVO updateUserPermission(@RequestBody SysUserForm sysUserForm);

    /**
     * 删除用户信息
     * @param sysUserForm
     * @return
     */
    @GetMapping("/delUserInfo")
    ResultVO delUserPermission(@RequestBody SysUserForm sysUserForm);

    /**
     * 禁用（启用）用户信息
     * @param sysUserForm
     * @return
     */
    @GetMapping("/enableUserInfo")
    ResultVO enableUserInfo(@RequestBody SysUserForm sysUserForm);
}
