package com.hcloud.microserver.sys.facade.fallback.impl;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.facade.fallback.SysManagerFactory;
import com.hcloud.microserver.sys.facade.forms.SysMangerForm;
import com.hcloud.microserver.sys.facade.forms.SysUserForm;
import com.hcloud.microserver.sys.facade.client.SysUserClient;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.RequestBody;

import java.util.Map;

/**
 * @author xiezt
 */
@Slf4j
@Component
public class SysManagerFallbackFactory implements FallbackFactory<SysUserClient> {
    @Override
    public SysUserClient create(Throwable throwable) {
        return new SysManagerFactory() {

            /**
             * 通过账号查询用户信息
             *
             * @param account
             * @return
             */
            @Override
            public ResultVO<SysUser> queryUserInfoByAccount(String account) {
                log.error("error method is ====================>queryUserInfoByAccount");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 用户登录
             *
             * @param account
             * @param password
             * @return
             */
            @Override
            public ResultVO<Map> login(String account, String password) {
                log.error("error method is ====================>login");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 退出登录
             *
             * @param accessToken
             * @return
             */
            @Override
            public ResultVO logout(String accessToken) {
                log.error("error method is ====================>logout");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 添加用户信息
             *
             * @param sysUserForm
             * @return
             */
            @Override
            public ResultVO saveUserInfo(SysUserForm sysUserForm) {
                log.error("error method is ====================>saveUserInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 更新用户信息
             *
             * @param sysUserForm
             * @return
             */
            @Override
            public ResultVO updateUserInfo(SysUserForm sysUserForm) {
                log.error("error method is ====================>updateUserInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            @Override
            public ResultVO queryUserListByPage(@RequestBody SysMangerForm sysMangerForm) {
                log.error("error method is ====================>queryUserListByPage");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 查询用户基本权限信息
             *
             * @param sysMangerForm
             * @return
             */
            @Override
            public ResultVO queryUserPermissionById(SysMangerForm sysMangerForm) {
                log.error("error method is ====================>queryUserPermissionById");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 查询部门及角色信息
             *
             * @param refSysApp
             * @return
             */
            @Override
            public ResultVO getDeptAndRoleInfo(String refSysApp) {
                log.error("error method is ====================>getDeptAndRoleInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 更新用户权限信息
             *
             * @param sysUserForm
             * @return
             */
            @Override
            public ResultVO updateUserPermission(SysUserForm sysUserForm) {
                log.error("error method is ====================>updateUserPermission");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 删除用户信息
             *
             * @param sysUserForm
             * @return
             */
            @Override
            public ResultVO delUserPermission(SysUserForm sysUserForm) {
                log.error("error method is ====================>delUserPermission");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 禁用（启用）用户信息
             *
             * @param sysUserForm
             * @return
             */
            @Override
            public ResultVO enableUserInfo(SysUserForm sysUserForm) {
                log.error("error method is ====================>enableUserInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }
        };
    }
}
