package com.hcloud.microserver.sys.facade.fallback.impl;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.sys.facade.fallback.SysRoleManagerFactory;
import com.hcloud.microserver.sys.facade.forms.SysRoleForm;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

/**
 * @author JYJ
 */
@Slf4j
@Component
public class SysRoleManagerFallback implements FallbackFactory<SysRoleManagerFactory> {

    @Override
    public SysRoleManagerFactory create(Throwable cause) {
        return new SysRoleManagerFactory(){
            /**
             * 查询角色信息
             *
             * @param sysRoleForm
             * @return
             */
            @Override
            public ResultVO querySysRoleInfo(SysRoleForm sysRoleForm) {
                log.error("error method is ====================>querySysRoleInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 查询角色基本权限信息
             *
             * @param sysRoleForm
             * @return
             */
            @Override
            public ResultVO queryRolePermissionById(SysRoleForm sysRoleForm) {
                log.error("error method is ====================>queryRolePermissionById");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 新增角色基本信息
             *
             * @param sysRoleForm
             * @return
             */
            @Override
            public ResultVO saveRoleInfo(SysRoleForm sysRoleForm) {
                log.error("error method is ====================>saveRoleInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 更新角色基本信息
             *
             * @param sysRoleForm
             * @return
             */
            @Override
            public ResultVO updateRoleInfo(SysRoleForm sysRoleForm) {
                log.error("error method is ====================>queryRolePermissionById");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 更新角色权限信息
             *
             * @param sysRoleForm
             * @return
             */
            @Override
            public ResultVO updateRolePermissionInfo(SysRoleForm sysRoleForm) {
                log.error("error method is ====================>updateRolePermissionInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }

            /**
             * 删除角色基本信息
             *
             * @param sysRoleForm
             * @return
             */
            @Override
            public ResultVO delRolePermissionInfo(SysRoleForm sysRoleForm) {
                log.error("error method is ====================>delRolePermissionInfo");
                return new ResultVO(ResultEnum.SYS_BREAK_ERROR);
            }
        };
    }
}
