package com.hcloud.microserver.sys.service.biz;

import com.hcloud.microserver.commoncore.base.Token;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.service.service.SysDepartmentService;
import com.hcloud.microserver.sys.service.service.SysRoleService;
import com.hcloud.microserver.sys.service.service.SysUserPermissionService;
import com.hcloud.microserver.sys.service.service.SysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * @author JYJ
 */
@Component("loginService")
@Slf4j
public class LoginSystemService {

    //部门及角色缓存
    private static final String DEPT_AND_ROLE_STR="deptAndRole";


    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private SysUserPermissionService sysUserPermissionService;

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private SysDepartmentService sysDepartmentService;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 用户登录接口
     * @param loginName
     * @param password
     * @return
     */
    public Map<String,Object> loginSystemByUserInfo(String loginName,String password){
        Map<String,Object> reslutMap = Collections.emptyMap();
        Token token = null;
        SysUser user =  sysUserService.getSystemByUser(loginName,password);
        if(null!=user && !"".equals(user.getPassword())){
            reslutMap =  new HashMap();
            String userId = user.getGuid();
            List<Map<String,Object>> userPermissionList = sysUserPermissionService.getUserPermissionById(userId);
            ValueOperations<String, Object> operations = redisTemplate.opsForValue();
            token = TokenUtil.createJwtToken(userId, GlobleConstant.CURRENT_USER);
            reslutMap.put(GlobleConstant.ACCESS_TOKEN,token.getAccessToken());
            reslutMap.put(GlobleConstant.AUTH_LIST, userPermissionList);
            reslutMap.put(GlobleConstant.CURRENT_USER, user);
            operations.set(token.getAccessToken(), reslutMap, GlobleConstant.ACCESS_TOKEN_EXPIRES_SECOND, TimeUnit.SECONDS);
        }
        return reslutMap;
    }

    public Map getAllDeptAndRole(){
        Map map = new  HashMap();
        if(!redisTemplate.hasKey(DEPT_AND_ROLE_STR)){
           List<Map<String,String>> roleList =  sysRoleService.queryAllRole();
           List<Map<String,String>> deptList =  sysDepartmentService.queryAllDepartment();
           map.put("dept",deptList);
           map.put("role",roleList);
            ValueOperations<String, Object> operations = redisTemplate.opsForValue();
            operations.set(DEPT_AND_ROLE_STR,map,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
        }else{
           return (Map) redisTemplate.opsForValue().get(DEPT_AND_ROLE_STR);
        }
        return map;
    }


    public Integer deluserInfo(String userId){
        if(null!= userId && !"".equals(userId)){
            SysUser bean = new SysUser();
            bean.setGuid(userId);
            bean.setCstatus("0");
            int success = sysUserService.modifyByPrimaryKeySelective(bean);
            if(success>0){
                sysUserPermissionService.delUserPermissionByUid(userId);
                return success;
            }
        }
        return 0;
    }
}
