package com.hcloud.microserver.sys.service.controller;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.sys.facade.client.SysUserClient;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.facade.forms.SysMangerForm;
import com.hcloud.microserver.sys.facade.forms.SysUserForm;
import com.hcloud.microserver.sys.facade.vo.SysTreeNode;
import com.hcloud.microserver.sys.service.biz.LoginSystemService;
import com.hcloud.microserver.sys.service.service.SysPermissionService;
import com.hcloud.microserver.sys.service.service.SysUserService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 *
 * @author apple
 * @date 2018/9/8
 */
@Slf4j
@RestController
@RequestMapping("/user")
public class SysManagerController extends BaseController implements SysUserClient {


    @Autowired
    private LoginSystemService loginService;

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private SysPermissionService SysPermissionService;

    @Autowired
    private RedisUtils redisUtils;


    @GetMapping("/findUserByAccount")
    @Override
    public ResultVO<SysUser> queryUserInfoByAccount(@RequestParam("account") String account) {
        log.info("account=====================>{}", account);
        return success();
    }

    @PostMapping("/login")
    @Override
    public ResultVO<Map> login(@RequestParam("account") String account, @RequestParam("password") String password) {
        log.info("登录的用户名：[{}]", account);
        Map resultMap = loginService.loginSystemByUserInfo(account, password);
        log.info("登录后的用户信息：[{}]", resultMap);
        if (null != resultMap && resultMap.size() > 0) {
            return success(resultMap);
        }
        return failure();
    }

    @GetMapping("/logout")
    @Override
    public ResultVO logout(@RequestParam("token") String accessToken){
        if(null!=accessToken && !"".equals(accessToken)){
            redisUtils.deleteKey(accessToken);
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    /**
     * @param sysUserForm
     * @return
     */

    @PostMapping("/saveUserInfo")
    @Override
    public ResultVO saveUserInfo(@RequestBody SysUserForm sysUserForm) {
        return sysUserService.saveUserInfoAndPermission(sysUserForm);
    }

    @PostMapping("/updateUserInfo")
    @Override
    public ResultVO updateUserInfo(@RequestBody SysUserForm sysUserForm) {
        return sysUserService.modifyUserInfoAndPermission(sysUserForm);
    }

    @PostMapping("/queryUserListByPage")
    @Override
    public ResultVO queryUserListByPage(@RequestBody SysMangerForm sysMangerForm) {
        PageInfo<Page<Map>> pageInfo = sysUserService.queryUserInfoByPage(sysMangerForm);
        return success(pageInfo.getList(),pageInfo.getTotal());
    }

    @PostMapping("/userPermissionTree")
    @Override
    public ResultVO queryUserPermissionById(@RequestBody SysMangerForm sysMangerForm) {
        log.info("sysMangerForm================>{}", sysMangerForm);
        List<SysTreeNode> list = SysPermissionService.selectPermTree(sysMangerForm.getUserId(), "user");
        return success(list);
    }

    @GetMapping("/selectList")
    @Override
    public ResultVO getDeptAndRoleInfo(@RequestParam("sysApp") String refSysApp) {
        Map resMap = loginService.getAllDeptAndRole();
        if (null != resMap) {
            return success(resMap);
        }
        return new ResultVO(ResultEnum.SYS_ERROR.getCode(), ResultEnum.SYS_ERROR.getMsg());
    }

    @PostMapping("/updateUserPermission")
    @Override
    public ResultVO updateUserPermission(@RequestBody SysUserForm sysUserForm) {
        if (null != sysUserForm) {
            int success = sysUserService.updateUserPermission(sysUserForm);
            if (success > 0) {
                return success();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @PostMapping("/delUserInfo")
    @Override
    public ResultVO delUserPermission(@RequestBody SysUserForm sysUserForm) {
        if (null != sysUserForm) {
            String guid = sysUserForm.getGuid();
            int success = loginService.deluserInfo(guid);
            if (success > 0) {
                return success();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @GetMapping("/enableUserInfo")
    @Override
    public ResultVO enableUserInfo(@RequestBody SysUserForm sysUserForm) {
        if (null != sysUserForm) {
            SysUser user = new SysUser();
            user.setGuid(sysUserForm.getUserId());
            user.setCstatus(sysUserForm.getCstatus());
            int success = sysUserService.modifyByPrimaryKeySelective(user);
            if (success > 0) {
                return success();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

}
