package com.hcloud.microserver.sys.service.controller;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.annotation.AuthValidate;
import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.AuthEnum;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.sys.facade.client.SysRoleClient;
import com.hcloud.microserver.sys.facade.entity.SysRole;
import com.hcloud.microserver.sys.facade.forms.SysRoleForm;
import com.hcloud.microserver.sys.facade.vo.SysTreeNode;
import com.hcloud.microserver.sys.service.service.SysPermissionService;
import com.hcloud.microserver.sys.service.service.SysRolePermissionService;
import com.hcloud.microserver.sys.service.service.SysRoleService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @author xiezt
 */
@RestController
@RequestMapping("/role")
@Slf4j
public class SysManagerRoleController extends BaseController implements SysRoleClient {

    @Autowired
    private SysRoleService sysRoleService;

    @Autowired
    private SysRolePermissionService sysRolePermissionService;

    @Autowired
    private SysPermissionService sysPermissionService;

    @PostMapping("/roleList")
    @Override
    public ResultVO querySysRoleInfo(@RequestBody SysRoleForm sysRoleForm) {
        PageInfo<Page<Map>> pageInfo = sysRoleService.queryRoleListByPage(sysRoleForm);
        if (null != pageInfo) {
            return success(pageInfo.getList(), pageInfo.getTotal());
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @PostMapping("/permissionTree")
    @Override
    public ResultVO queryRolePermissionById(@RequestBody SysRoleForm sysRoleForm) {
        log.info("sysRoleForm================>{}", sysRoleForm);
        List<SysTreeNode> list = sysPermissionService.selectPermTree(sysRoleForm.getGuid(), "role");
        return success(list);
    }

    @PostMapping("/saveInfo")
    @Override
    public ResultVO saveRoleInfo(@RequestBody SysRoleForm sysRoleForm) {
        log.info("sysRoleForm================>{}", sysRoleForm);
        int success = sysRoleService.saveRoleInfo(sysRoleForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @PostMapping("/updateInfo")
    @Override
    public ResultVO updateRoleInfo(@RequestBody SysRoleForm sysRoleForm) {
        log.info("sysRoleForm===============>{}", sysRoleForm);
        if (null != sysRoleForm && !"".equals(sysRoleForm.getGuid())) {
            SysRole sysRole = new SysRole();
            sysRole.setGuid(sysRoleForm.getGuid());
            sysRole.setCname(sysRoleForm.getCname());
            sysRole.setCdesc(sysRoleForm.getCdesc());
            sysRole.setUpdatedate(new Date());
            sysRole.setUpdateuser(sysRoleForm.getRealName());
            int success = sysRoleService.modifyByPrimaryKeySelective(sysRole);
            if (success > 0) {
                return success();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }


    @PostMapping("/updatePermissionInfo")
    @Override
    public ResultVO updateRolePermissionInfo(@RequestBody SysRoleForm sysRoleForm) {
        log.info("sysRoleForm================>{}", sysRoleForm);
        int success = sysRolePermissionService.updateRolePermissionByPermisList(sysRoleForm);
        if (success > 0) {
            return success();
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    @PostMapping("/delRoleInfo")
    @Override
    public ResultVO delRolePermissionInfo(@RequestBody SysRoleForm sysRoleForm) {
        if (null != sysRoleForm) {
            int success = sysRoleService.delRoleInfoAndPermission(sysRoleForm.getGuid());
            if (success > 0) {
                return success();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

}