package com.hcloud.microserver.sys.service.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.facade.forms.SysDeptForm;
import com.hcloud.microserver.sys.service.dao.SysDepartmentMapper;
import com.hcloud.microserver.sys.service.service.SysDepartmentService;
import com.hcloud.microserver.sys.facade.entity.SysDepartment;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestBody;

import javax.annotation.Resource;
import java.lang.reflect.InvocationTargetException;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @author apple
 * @date 2018/9/7
 */
@Service("sysDepartmentService")
@Slf4j
public class SysDepartmentServiceImpl implements SysDepartmentService {

    @Resource
    private SysDepartmentMapper sysDepartmentMapper;

    @Autowired
    private RedisUtils redisUtils;

    @Override
    public int delByPrimaryKey(String guid) {
        return sysDepartmentMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(SysDepartment record) {
        return sysDepartmentMapper.insert(record);
    }

    @Override
    public int saveSelective(SysDepartment record) {
        return sysDepartmentMapper.insertSelective(record);
    }

    @Override
    public SysDepartment queryByPrimaryKey(String guid) {
        return sysDepartmentMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(SysDepartment record) {
        return sysDepartmentMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(SysDepartment record) {
        return sysDepartmentMapper.updateByPrimaryKeyWithBLOBs(record);
    }

    @Override
    public int modifyByPrimaryKey(SysDepartment record) {
        return sysDepartmentMapper.updateByPrimaryKey(record);
    }

    /**
     * 获取所有部门信息
     *
     * @return
     */
    @Override
    public List<Map<String, String>> queryAllDepartment() {
        return sysDepartmentMapper.queryAllDepartment();
    }

    /**
     * 分页查询部门信息
     *
     * @param sysDeptForm
     * @return
     */
    @Override
    public PageInfo<Page<Map>> queryDeptInfoListByPage(SysDeptForm sysDeptForm) {
        log.info("sysDeptForm=======================>{}", sysDeptForm);
        if (null != sysDeptForm) {
            PageHelper.startPage(sysDeptForm.getPageNo(), sysDeptForm.getPageSize());
            PageBean<SysDeptForm> bean = new PageBean<>(sysDeptForm);
            Page<Map> page = sysDepartmentMapper.findDeptInfoListByPage(bean);
            return new PageInfo(page);
        }
        return null;
    }

    /**
     * 保存部门信息
     *
     * @param sysDeptForm
     * @return
     */
    @Override
    public int saveDeptmentInfo(SysDeptForm sysDeptForm) {
        log.info("sysDeptForm=====================>{}", sysDeptForm);
        String cname = sysDeptForm.getCname();
        String cdesc = sysDeptForm.getCdesc();
        log.info("cname: {}", cname);
        if (null != sysDeptForm) {
            SysDepartment record = new SysDepartment();
            try {
                BeanCopyUtil.copyBean(record, sysDeptForm);
                record.setCname(cname);
                record.setCdesc(cdesc);
                record.setGuid(GetUUID.getUUID());
                record.setCcode(getNextMaxCode());
                record.setCreatedate(new Date());
                record.setCreateuser(getCurrentUser(sysDeptForm.getAccessToken()).getRealName());
                return sysDepartmentMapper.insertSelective(record);
            } catch (Exception e) {
                log.error("copy properties error,source===>{}", sysDeptForm);
                e.printStackTrace();
            }
        }
        return 0;
    }

    /**
     * 更新部门信息
     *
     * @param sysDeptForm
     * @return
     */
    @Override
    public int updateDepartmentInfo(SysDeptForm sysDeptForm) {
        log.info("sysDeptForm=====================>{}", sysDeptForm);
        String cname = sysDeptForm.getCname();
        String cdesc = sysDeptForm.getCdesc();
        String guid = sysDeptForm.getGuid();
        String ccode = sysDeptForm.getCcode();
        if (null != sysDeptForm) {
            SysDepartment record = new SysDepartment();
            try {
                BeanCopyUtil.copyBean(record, sysDeptForm);
                record.setCname(cname);
                record.setCdesc(cdesc);
                record.setGuid(guid);
                record.setCcode(ccode);
                record.setUpdatedate(new Date());
                record.setUpdateuser(getCurrentUser(sysDeptForm.getAccessToken()).getRealName());
                return sysDepartmentMapper.updateByPrimaryKeySelective(record);
            } catch (Exception e) {
                log.error("copy properties error,source===>{}", sysDeptForm);
                e.printStackTrace();
            }
        }
        return 0;
    }

    private String getNextMaxCode() {
        String maxCode = sysDepartmentMapper.getMaxCode();
        if (null != maxCode && !"".equals(maxCode)) {
            Integer codeVal = Integer.valueOf(maxCode);
            codeVal++;
            return codeVal.toString();
        } else {
            return "80000";
        }
    }

    /**
     * 获取当前登录用户信息
     *
     * @param accessToken
     * @return
     */
    private SysUser getCurrentUser(String accessToken) {
        Map map = (Map) redisUtils.getValue(accessToken);
        if (null != map) {
            return (SysUser) map.get(GlobleConstant.CURRENT_USER);
        }
        return new SysUser();
    }
}
