package com.hcloud.microserver.sys.service.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.sys.facade.entity.SysPermission;
import com.hcloud.microserver.sys.facade.forms.SysPermissionForm;
import com.hcloud.microserver.sys.facade.vo.SysTreeNode;
import com.hcloud.microserver.sys.service.dao.SysPermissionMapper;
import com.hcloud.microserver.sys.service.service.SysPermissionService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author apple
 * @date 2018/9/7
 */
@Service("sysPermissionService")
@Slf4j
public class SysPermissionServiceImpl implements SysPermissionService {

    @Resource
    private SysPermissionMapper sysPermissionMapper;

    @Override
    public int delByPrimaryKey(String guid) {
        return sysPermissionMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(SysPermission record) {
        return sysPermissionMapper.insertSelective(record);
    }

    @Override
    public int saveSelective(SysPermission record) {
        return sysPermissionMapper.insertSelective(record);
    }

    @Override
    public SysPermission queryByPrimaryKey(String guid) {
        return sysPermissionMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(SysPermission record) {
        return sysPermissionMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(SysPermission record) {
        return sysPermissionMapper.updateByPrimaryKeyWithBLOBs(record);
    }

    @Override
    public int modifyByPrimaryKey(SysPermission record) {
        return sysPermissionMapper.updateByPrimaryKey(record);
    }

    @Override
    public List<SysTreeNode> selectPermTree(String guid, String status) {
        log.info("guid=================>{}", guid);
        log.info("status=================>{}", status);
        List<SysTreeNode> treeList = sysPermissionMapper.selectPermTree(guid, status);
        if (null != treeList && treeList.size() > 0) {
            List<SysTreeNode> convertTree = getSysTree(treeList);
            return convertTree;
        }
        return null;
    }

    @Override
    public PageInfo<Page<Map>> queryPermissionListByPage(SysPermissionForm sysPermissionForm) {
        if (!Objects.isNull(sysPermissionForm)) {
            PageBean<SysPermissionForm> pageBean = new PageBean<>(sysPermissionForm);
            PageHelper.startPage(sysPermissionForm.getPageNo(), sysPermissionForm.getPageSize());
            Page<Map> page = sysPermissionMapper.queryListByPage(pageBean);
            if (Objects.nonNull(page) && page.size() > 0) {
                return new PageInfo(page);
            }
        }
        return null;
    }

    @Override
    public List<SysPermission> queryByCondition(SysPermission sysPermission) {
        //SysPermission sysPermission = BeanCopyUtil.convertBean(sysPermissionForm, SysPermission.class);
        return sysPermissionMapper.selectByCondition(sysPermission);
    }

    /**
     * 组装权限树
     *
     * @param sysTreeList
     * @return
     */
    private List<SysTreeNode> getSysTree(List<SysTreeNode> sysTreeList) {
        List<SysTreeNode> rootTree = new ArrayList<SysTreeNode>();
        for (SysTreeNode tree : sysTreeList) {
            // 当父字段值为null或0时，表示组装成根节点
            if (tree.getPId() == null || tree.getPId().equals("0") || tree.getPId().equals("#")) {
                rootTree.add(tree);
                // 如果是根节点数据，直接执行下一次循环，跳过本次循环
            }
            for (SysTreeNode sysTree : sysTreeList) {
                // 如果是根节点数据，直接执行下一次循环，跳过本次循环
                if (sysTree.getPId() != null && !sysTree.getPId().equals("0")
                        && !sysTree.getPId().equals("#")) {

                    // 如果下列判断成立，那么组装成当前外层循环的子节点
                    if (sysTree.getPId().equals(tree.getId())) {
                        if (tree.getChildren() == null || tree.getChildren().size() == 0) {
                            // 存放子数据
                            List<SysTreeNode> children = new ArrayList<SysTreeNode>();
                            children.add(sysTree);
                            tree.setChildren(children);
                        } else {
                            tree.getChildren().add(sysTree);
                        }
                    }
                }
            }
        }
        return rootTree;
    }
}
