package com.hcloud.microserver.sys.service.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.facade.forms.SysRoleForm;
import com.hcloud.microserver.sys.service.dao.SysRoleMapper;
import com.hcloud.microserver.sys.service.service.SysRolePermissionService;
import com.hcloud.microserver.sys.service.service.SysRoleService;
import com.hcloud.microserver.sys.facade.entity.SysRole;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 *
 * @author apple
 * @date 2018/9/8
 */
@Service("sysRoleService")
public class SysRoleServiceImpl implements SysRoleService {

    @Resource
    private SysRoleMapper sysRoleMapper;

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private SysRolePermissionService sysRolePermissionService;

    @Override
    public int delByPrimaryKey(String guid) {
        return sysRoleMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(SysRole record) {
        return sysRoleMapper.insert(record);
    }

    @Override
    public int saveSelective(SysRole record) {
        return sysRoleMapper.insertSelective(record);
    }

    @Override
    public SysRole queryByPrimaryKey(String guid) {
        return sysRoleMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(SysRole record) {
        return sysRoleMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(SysRole record) {
        return sysRoleMapper.updateByPrimaryKeyWithBLOBs(record);
    }

    @Override
    public int modifyByPrimaryKey(SysRole record) {
        return sysRoleMapper.updateByPrimaryKey(record);
    }

    /**
     * 获取角色信息
     *
     * @return
     */
    @Override
    public List<Map<String, String>> queryAllRole() {
        return sysRoleMapper.queryAllRole();
    }

    /**
     * 分页查询角色信息
     *
     * @param sysRoleForm
     * @return
     */
    @Override
    public PageInfo<Page<Map>> queryRoleListByPage(SysRoleForm sysRoleForm) {
        if (null != sysRoleForm) {
            PageBean<SysRoleForm> pageBean = new PageBean<>(sysRoleForm);
            PageHelper.startPage(sysRoleForm.getPageNo(), sysRoleForm.getPageSize());
            Page<Map> page = sysRoleMapper.queryRoleListByPage(pageBean);
            if (null != page && page.size() > 0) {
                return new PageInfo(page);
            }
        }
        return null;
    }

    /**
     * 删除角色权限信息
     *
     * @param roleId
     * @return
     */
    @Transactional(rollbackFor = RuntimeException.class)
    @Override
    public int delRoleInfoAndPermission(String roleId) {
        SysRole sysRole = new SysRole();
        sysRole.setGuid(roleId);
        sysRole.setCstatus("0");
        int success = sysRoleMapper.updateByPrimaryKeySelective(sysRole);
        if (success > 0) {
            sysRolePermissionService.delPermByRoleId(roleId);
        }
        return success;
    }

    /**
     * 保存角色信息
     *
     * @param sysRoleForm
     * @return
     */
    @Override
    public int saveRoleInfo(SysRoleForm sysRoleForm) {
        if (null != sysRoleForm) {
            SysRole sysRole = new SysRole();
            sysRole.setGuid(GetUUID.getUUID());
            sysRole.setCcode(getMaxCode());
            sysRole.setCname(sysRoleForm.getCname());
            sysRole.setCdesc(sysRoleForm.getCdesc());
            sysRole.setCstatus("2");
            sysRole.setCreatedate(new Date());
            sysRole.setCreateuser(getCurrentUser(sysRoleForm.getAccessToken()).getRealName());
            return sysRoleMapper.insertSelective(sysRole);
        }
        return 0;
    }

    /**
     * 获取最大编号
     *
     * @return
     */
    @Override
    public String getMaxCode() {
        String maxCode = sysRoleMapper.getMaxCode();
        if (null != maxCode && !"".equals(maxCode)) {
            Integer codeVal = Integer.valueOf(maxCode);
            codeVal++;
            return codeVal.toString();
        } else {
            return "60000";
        }
    }

    /**
     * 获取当前登录用户信息
     *
     * @param accessToken
     * @return
     */
    private SysUser getCurrentUser(String accessToken) {
        Map map = (Map) redisUtils.getValue(accessToken);
        if (null != map) {
            return (SysUser) map.get(GlobleConstant.CURRENT_USER);
        }
        return new SysUser();
    }
}
