package com.hcloud.microserver.sys.service.service.impl;

import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hcloud.microserver.commoncore.base.PageBean;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.base.Token;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.GetUUID;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import com.hcloud.microserver.sys.facade.entity.SysUser;
import com.hcloud.microserver.sys.facade.forms.SysMangerForm;
import com.hcloud.microserver.sys.facade.forms.SysUserForm;
import com.hcloud.microserver.sys.service.dao.SysUserMapper;
import com.hcloud.microserver.sys.service.service.SysRolePermissionService;
import com.hcloud.microserver.sys.service.service.SysUserPermissionService;
import com.hcloud.microserver.sys.service.service.SysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 *
 * @author apple
 * @date 2018/9/8
 */
@Slf4j
@Service("sysUserService")
public class SysUserServiceImpl implements SysUserService {

    @Resource
    private SysUserMapper sysUserMapper;

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private SysRolePermissionService sysRolePermissionService;

    @Autowired
    private SysUserPermissionService sysUserPermissionService;

    @Override
    public int delByPrimaryKey(String guid) {
        return sysUserMapper.deleteByPrimaryKey(guid);
    }

    @Override
    public int save(SysUser record) {
        return sysUserMapper.insert(record);
    }

    @Override
    public int saveSelective(SysUser record) {
        return sysUserMapper.insertSelective(record);
    }

    @Override
    public SysUser queryByPrimaryKey(String guid) {
        return sysUserMapper.selectByPrimaryKey(guid);
    }

    @Override
    public int modifyByPrimaryKeySelective(SysUser record) {
        return sysUserMapper.updateByPrimaryKeySelective(record);
    }

    @Override
    public int modifyByPrimaryKeyWithBLOBs(SysUser record) {
        return sysUserMapper.updateByPrimaryKeyWithBLOBs(record);
    }

    @Override
    public int modifyByPrimaryKey(SysUser record) {
        return sysUserMapper.updateByPrimaryKey(record);
    }

    @Override
    public SysUser getSystemByUser(String loginName, String password) {
        log.info("loginName=======>{}", loginName);
        SysUser user = sysUserMapper.getUserByAccount(loginName);
        if (null != user && null != password && !"".equals(password)) {
            String loginPwd = user.getPassword();
            if (loginPwd.equals(Md5Util.toMD5(password))) {
                return user;
            }
        }
        log.info("用户名或命名不正确=======>loginAccount:{},passwd:{}", loginName, password);
        return null;
    }

    @Override
    public PageInfo<Page<Map>> queryUserInfoByPage(SysMangerForm sysMangerForm) {
        log.info("sysMangerForm======================>{}", sysMangerForm);
        PageHelper.startPage(sysMangerForm.getPageNo(),sysMangerForm.getPageSize());
        PageBean<SysMangerForm> pb = new PageBean(sysMangerForm);
        Page<Map> page = sysUserMapper.queryUserInfoByPage(pb);
        return new PageInfo(page);
    }

    /**
     * 保存用户及权限
     *
     * @param sysUserForm
     * @return
     */
    @Transactional(rollbackFor = RuntimeException.class)
    @Override
    public ResultVO saveUserInfoAndPermission(SysUserForm sysUserForm) {
        if (null != sysUserForm) {
            SysUser sysUser = null;
            try {
                //保存用户信息;
                sysUser = BeanCopyUtil.convertBean(sysUserForm, SysUser.class);
                log.info("========================> convertBean是否转换成功: {}", sysUser);
                String realName = getCurrentUser(sysUserForm.getAccessToken()).getRealName();
                sysUser.setGuid(GetUUID.getUUID());
                sysUser.setCreatedate(new Date());
                sysUser.setCreateuser(realName);
                sysUser.setRefSysDepartment(sysUserForm.getDeptId());
                sysUser.setRefZhccSRole(sysUserForm.getRoleId());
                sysUser.setPassword(Md5Util.toMD5(sysUserForm.getPassword()));
                int success = sysUserMapper.insertSelective(sysUser);
                //根据用户角色配置用户权限;
                if (success > 0) {
                    List<String> roleList = sysRolePermissionService.selectPermByRoleId(sysUserForm.getRoleId());
                    int batchRes = sysUserPermissionService.insertBatchPermissionByUid(sysUser.getGuid(), roleList);
                    if (batchRes == roleList.size()) {
                        return new ResultVO(ResultEnum.SUCCESS);
                    }
                }
            } catch (Exception e) {
                log.error("属性拷贝异常=======>{}", sysUserForm);
                e.printStackTrace();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    /**
     * 更新用户信息及权限
     *
     * @param sysUserForm
     * @return
     */
    @Override
    public ResultVO modifyUserInfoAndPermission(SysUserForm sysUserForm) {
        if (null != sysUserForm) {
            SysUser sysUser = null;
            try {
                //保存用户信息;
                SysUser currentUser = getCurrentUser(sysUserForm.getAccessToken());
                sysUser = BeanCopyUtil.convertBean(sysUserForm, SysUser.class);
                log.info("========================> convertBean是否转换成功: {}", sysUser);
                sysUser.setUpdatedate(new Date());
                sysUser.setUpdateuser(currentUser.getRealName());
                sysUser.setPassword(Md5Util.toMD5(sysUserForm.getPassword()));
                sysUser.setRefSysDepartment(sysUserForm.getDeptId());
                if (sysUserForm.getIsChange() == 1) {
                    sysUser.setRefZhccSRole(sysUserForm.getRoleId());
                }
                int success = sysUserMapper.updateByPrimaryKeySelective(sysUser);
                //根据用户角色配置用户权限;
                if (success > 0) {
                    if (sysUserForm.getIsChange() == 1) {
                        List<String> roleList = sysRolePermissionService.selectPermByRoleId(sysUserForm.getRoleId());
                        int batchRes = sysUserPermissionService.insertBatchPermissionByUid(sysUser.getGuid(), roleList);
                        if (batchRes == roleList.size()) {
                            //更新用户缓存信息(只修改登录用户信息时才更新缓存)
                            if (sysUserForm.getGuid().equals(currentUser.getGuid())) {
                                updateUserRedisInfo(sysUserForm.getAccessToken(), sysUserForm.getGuid());
                            }
                            return new ResultVO(ResultEnum.SUCCESS);
                        } else {
                            return new ResultVO(ResultEnum.SYS_ERROR);
                        }
                    }
                    return new ResultVO(ResultEnum.SUCCESS);
                }
            } catch (Exception e) {
                log.error("属性拷贝异常=======>{}", sysUserForm);
                e.printStackTrace();
            }
        }
        return new ResultVO(ResultEnum.SYS_ERROR);
    }

    /**
     * 更新用户权限
     *
     * @param sysUserForm
     * @return
     */
    @Override
    public int updateUserPermission(SysUserForm sysUserForm) {
        int success = 0;
        if (null != sysUserForm && sysUserForm.getPermIds().length > 0) {
            String userId = sysUserForm.getUserId();
            String[] permIdList = sysUserForm.getPermIds();
            List<String> permList = Arrays.asList(permIdList);
            success = sysUserPermissionService.insertBatchPermissionByUid(userId, permList);
        }
        return success;
    }


    /**
     * 获取当前登录用户信息
     *
     * @param accessToken
     * @return
     */
    private SysUser getCurrentUser(String accessToken) {
        Map map = (Map) redisUtils.getValue(accessToken);
        if (null != map) {
            return (SysUser) map.get(GlobleConstant.CURRENT_USER);
        }
        return new SysUser();
    }

    /**
     * 更新用户redis缓存信息
     *
     * @param accessToken
     * @param uid
     */
    private void updateUserRedisInfo(String accessToken, String uid) {
        if (null != accessToken && !"".equals(accessToken)) {
            if (redisUtils.existsKey(accessToken)) {
                redisUtils.deleteKey(accessToken);
                Map resultMap = new HashMap();
                Token token = TokenUtil.createJwtToken(uid, GlobleConstant.CURRENT_MEMBER);
                SysUser bean = sysUserMapper.selectByPrimaryKey(uid);
                List<Map<String, Object>> userPermissionList = sysUserPermissionService.getUserPermissionById(uid);
                resultMap.put(GlobleConstant.ACCESS_TOKEN, token.getAccessToken());
                resultMap.put(GlobleConstant.AUTH_LIST, userPermissionList);
                resultMap.put(GlobleConstant.CURRENT_USER, bean);
                redisUtils.setValue(token.getAccessToken(), resultMap, 3600, TimeUnit.SECONDS);
            }
        }
    }
}
