package com.hcloud.microserver.bank.app;

import com.hcloud.microserver.bank.common.BaseInfoDO;
import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.bank.forms.CompanyValidateForm;
import com.hcloud.microserver.bank.forms.LoginForm;
import com.hcloud.microserver.bank.forms.PersonValidateForm;
import com.hcloud.microserver.bank.forms.SmallCustomer;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.base.Token;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.commoncore.util.JavaBeanUtils;
import com.hcloud.microserver.commoncore.util.Md5Util;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import com.hcloud.microserver.facade.carbon.client.CustomerClient;
import com.hcloud.microserver.facade.carbon.forms.CustomerCompanyInfoForm;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/wap/customer")
@Api(description = "H5端客户服务")
public class CustomerManagerController extends CarbonBaseController {

    private static final String WECHAT_USER_INFO="wechatUserInfo";

    @Autowired
    private CustomerClient customerClient;

    @Autowired
    private RedisTemplate redisTemplate;

    @ApiOperation(value = "通过用户账号或手机登录系统",notes = "通过用户账号或手机登录系统,需要输入密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginForm",value = "登录相关信息", dataType = "LoginForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/customerLogin")
    @AuthCarbonValidate
    public ResponseBase customerLogin(@RequestBody LoginForm loginForm){
        ValueOperations<String, Object> operations = redisTemplate.opsForValue();
        String accountStr = loginForm.getLoginStr();
        Integer loginMethod = loginForm.getLoginMethod();
        ResultVO resultVO =  customerClient.getCustomerInfoByInfo(accountStr);
        //账号密码登录
        if(resultVO.getCode()==0){
            Map userMap = (Map) resultVO.getData();
            CustomerInfoForm customerForm = JavaBeanUtils.mapToEntity(userMap,CustomerInfoForm.class);
            if(loginMethod==1){
                String pwd = loginForm.getPwd();
                String customerPwd = customerForm.getPasswd();
                if(Md5Util.toMD5(pwd).equals(customerPwd)){
                    String token =setCache(customerForm);
                    return responseSuccess(success(token));
                }else {
                    return responseError(new ResultVO(ResultEnum.LOGIN_PWD_MISS));
                }
            }else{
                //手机号登录
                String msgCode = loginForm.getMsgCode();
                String cacheCode = (String) operations.get(accountStr+":"+msgCode);
                //if(cacheCode==null){return responseError(new ResultVO(ResultEnum.CODE_ERROR));}
                //测试阶段加入恒等验证码,测试结束后删除代码
                if(msgCode.equals(cacheCode)|| "000000".equals(msgCode)){
                    String token =setCache(customerForm);
                    return responseSuccess(success(token));
                }
            }
        }else{
            if(loginMethod!=1){
                //TODO 如果是手机快捷登录方式，但是手机号码在系统内尚未注册时自动注册手机
                CustomerInfoForm customerInfoForm = new CustomerInfoForm();
                customerInfoForm.setNickName("手机注册用户");
                customerInfoForm.setCustomerType(1);
                customerInfoForm.setSex(1);
                customerInfoForm.setIsEnable(1);
                customerInfoForm.setPhone(accountStr);
                customerInfoForm.setState(1);
                customerInfoForm.setCreateTime(new Date());
                customerInfoForm.setModifiedTime(new Date());
                ResultVO customerVo = customerClient.saveCustomerInfo(customerInfoForm);
                Map userMap = (Map) customerVo.getData();
                CustomerInfoForm saveCustomer = JavaBeanUtils.mapToEntity(userMap,CustomerInfoForm.class);
                String token =setCache(saveCustomer);
                return responseSuccess(success(token));
            }
        }
        return responseError(new ResultVO(ResultEnum.LOGIN_ACCOUNT_MISS));
    }

    @ApiOperation(value = "获取爱心用户登录信息",notes = "获取爱心用户登录信息,通过token")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "accessToken",value = "登录成功后的Token", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getCustomerByToken")
    @AuthCarbonValidate
    public ResponseBase getCustomerInfoByToken(@RequestParam("accessToken") String accessToken){
        ValueOperations<String, Object> operations = redisTemplate.opsForValue();
        Map loginMap = (Map) operations.get(accessToken);
        if(null!=loginMap && loginMap.size()>0){
            CustomerInfoForm customerInfoForm = (CustomerInfoForm) loginMap.get(WECHAT_USER_INFO);
            String customerId = customerInfoForm.getGuid();
            ResultVO resultVO = customerClient.getCustomerInfoById(customerId);
            return responseSuccess(resultVO);
        }
        return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));
    }


    @ApiOperation(value = "获取爱心用户基本信息",notes = "获取爱心用户基本信息及统计分析信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerId",value = "客户id", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getCustomerById")
    @AuthCarbonValidate
    public ResponseBase getCustomAnalysisInfo(@RequestParam("userId") String customerId){
        ResultVO resultVO =  customerClient.getCustomerInfoById(customerId);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(new ResultVO(ResultEnum.SYS_ERROR));
    }


    @ApiOperation(value = "更新爱心用户基本信息",notes = "更新爱心用户基本信息(昵称，头像等)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "smallCustomer",value = "爱心用户基本信息", dataType = "SmallCustomer")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/updateBaseInfo")
    @AuthCarbonValidate
    public ResponseBase modifiedCustomerInfo(@RequestBody SmallCustomer smallCustomer){
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(Objects.isNull(baseInfoDO)){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
        CustomerInfoForm customerInfoForm = new CustomerInfoForm();
        customerInfoForm.setGuid(baseInfoDO.getCustomerId());
        customerInfoForm.setNickName(smallCustomer.getNickName());
        customerInfoForm.setHeadImage(smallCustomer.getHeadImage());
        customerInfoForm.setModifiedTime(new Date());
        ResultVO resultVO = customerClient.modifiedCustomerInfo(customerInfoForm);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }



    @ApiOperation(value = "个人认证",notes = "爱心用户个人认证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "personValidateForm",value = "认证信息", dataType = "PersonValidateForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/validateCustomer")
    @AuthCarbonValidate
    public ResponseBase validateCustomerInfo(@RequestBody PersonValidateForm personValidateForm){
        CustomerInfoForm customerInfoForm = new CustomerInfoForm();
        customerInfoForm.setGuid(personValidateForm.getCustomerId());
        customerInfoForm.setPid(personValidateForm.getPid());
        customerInfoForm.setCustomerName(personValidateForm.getCustomerName());
        customerInfoForm.setIsValidate(1);
        customerInfoForm.setModifiedTime(new Date());
        ResultVO resultVO =  customerClient.modifiedCustomerInfo(customerInfoForm);
        if(resultVO.getCode()==0){
           return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }

    @ApiOperation(value = "个人认证信息查询",notes = "查询个人认证信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerId",value = "认证信息", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getValidateInfo")
    @AuthCarbonValidate
    public ResponseBase getCustomerValidateInfo(@RequestParam("customerId") String customerId){
        ResultVO resultVO =  customerClient.getCustomerInfoById(customerId);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "企业认证",notes = "爱心用户企业认证")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "companyValidateForm",value = "企业认证信息", dataType = "CompanyValidateForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/validateCompany")
    @AuthCarbonValidate
    public ResponseBase validateCompanyInfo(@RequestBody CompanyValidateForm companyValidateForm){
        CustomerCompanyInfoForm companyInfoForm = BeanCopyUtil.convertBean(companyValidateForm,CustomerCompanyInfoForm.class);
        ResultVO resultVO =  customerClient.validateCustomerCompany(companyInfoForm);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }

    @ApiOperation(value = "查询企业认证信息",notes = "查询企业认证信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerId",value = "爱心用户id", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getCompanyInfo")
    @AuthCarbonValidate
    public ResponseBase getCompanyInfoByCid(@RequestParam("customerId") String customerId){
        ResultVO resultVO =  customerClient.getCustomerCompanyInfo(customerId);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }

    @ApiOperation(value = "绑定手机",notes = "爱心用户手机号绑定")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerInfoForm",value = "爱心用户信息", dataType = "CustomerInfoForm")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/updatePhone")
    @AuthCarbonValidate
    public ResponseBase updateCustomerPhone(@RequestBody CustomerInfoForm customerInfoForm){
        ValueOperations<String, Object> operations = redisTemplate.opsForValue();
        String pcode = customerInfoForm.getMsgCode();
        String phone =customerInfoForm.getPhone();
        if(pcode==null || "".equals(pcode)){return responseError(new ResultVO(ResultEnum.CODE_ERROR));}
        String redisKey = phone+":"+pcode;
        String redisCode = (String) operations.get(redisKey);
        if(pcode.equals(redisCode)||pcode.equals("000000")){
            ResultVO resultVO =  customerClient.bindCustomerPhone(customerInfoForm.getGuid(),phone);
            if(resultVO.getCode()==0){
                return responseSuccess(resultVO);
            }
            return responseError(resultVO);
        }else {
            return responseError(new ResultVO(ResultEnum.CODE_ERROR));
        }
    }


    @ApiOperation(value = "修改密码",notes = "修改密码时如绑定手机,则需要验证手机号码是否是绑定手机号码,并只能同验证码修改密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "loginForm",value = "修改密码信息", dataType = "LoginForm")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/updatePwd")
    @AuthCarbonValidate
    public ResponseBase updateCustomerPassword(@RequestBody LoginForm loginForm){
        ValueOperations<String, Object> operations = redisTemplate.opsForValue();
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(baseInfoDO==null){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
        Integer updateMethod = loginForm.getLoginMethod();
        if(updateMethod==1){
            String upPhone = loginForm.getLoginStr();
            if(upPhone.equals(baseInfoDO.getPhone())){
                String msgCode = loginForm.getMsgCode();
                String redisKey = upPhone+":"+msgCode;
                String redisCode = (String) operations.get(redisKey);
                if(msgCode.equals(redisCode) || "000000".equals(msgCode)){
                    CustomerInfoForm customerInfoForm = new CustomerInfoForm();
                    customerInfoForm.setGuid(loginForm.getCustomerId());
                    customerInfoForm.setPasswd(Md5Util.toMD5(loginForm.getPwd()));
                    customerInfoForm.setModifiedTime(new Date());
                    ResultVO resultVO =customerClient.modifiedCustomerInfo(customerInfoForm);
                    if(resultVO.getCode()==0){return responseSuccess(resultVO);}
                    else {return responseError(resultVO);}
                }else {
                    return responseError(new ResultVO(ResultEnum.CODE_EXPIRED));
                }
                //手机号不是绑定手机号
            }else {
                return responseError(new ResultVO(900,"验证手机号不是绑定手机号"));
            }
            //通过账号和原始密码修改密码
        }else {
            String account = loginForm.getLoginStr();
            String originalPwd = Md5Util.toMD5(loginForm.getOriginalPwd());
            String pwd = Md5Util.toMD5(loginForm.getPwd());
            //查询用户账号是否存在
            ResultVO searchVo =  customerClient.getCustomerInfoByAccount(account);
            if(searchVo.getCode()!=0){return responseError(searchVo);}
            Map userMap = (Map) searchVo.getData();
            CustomerInfoForm customerForm = JavaBeanUtils.mapToEntity(userMap,CustomerInfoForm.class);
            //判断原始密码是否正确
            if(originalPwd.equals(customerForm.getPasswd())){
                CustomerInfoForm customerInfoForm = new CustomerInfoForm();
                customerInfoForm.setGuid(loginForm.getCustomerId());
                customerInfoForm.setPasswd(pwd);
                customerInfoForm.setModifiedTime(new Date());
                ResultVO resultVO =customerClient.modifiedCustomerInfo(customerInfoForm);
                if(resultVO.getCode()==0){return responseSuccess(resultVO);}
                else {return responseError(resultVO);}
            }else{
                return responseError(new ResultVO(ResultEnum.LOGIN_PWD_MISS));
            }
        }
    }
    /**
     * 登录后设置登录缓存信息(专用于用户登录)
     * @param customerInfoForm
     * @return
     */
    private String setCache(CustomerInfoForm customerInfoForm){
        Map resultMap = new HashMap();
        ValueOperations<String, Object> operations = redisTemplate.opsForValue();
        Token token = TokenUtil.createJwtToken(customerInfoForm.getGuid(), GlobleConstant.CURRENT_MEMBER);
        resultMap.put(GlobleConstant.ACCESS_TOKEN,token.getAccessToken());
        resultMap.put(WECHAT_USER_INFO,customerInfoForm);
        operations.set(token.getAccessToken(),resultMap,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
        return token.getAccessToken();
    }

}
