package com.hcloud.microserver.bank.app;

import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.facade.carbon.client.FarmerInfoClient;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.PovertyAreaForm;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.concurrent.TimeUnit;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/wap/farmer")
@Api(description = "H5端贫困户服务")
public class FarmerManagerController extends CarbonBaseController {

    /**
     * 贫困区域列表信息缓存key
     */
    private static final String POVERTY_AREA_KEY="povertyAreaKey-";

    private static final String FARMER_LIST_KEY="farmerKey-";

    @Autowired
    private FarmerInfoClient farmerInfoClient;

    @Autowired
    private RedisUtils redisUtils;


    @ApiOperation(value = "分页查询贫困区域信息",notes = "通过查询条件查询贫困区域信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "povertyAreaForm",value = "查询条件", dataType = "PovertyAreaForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/searchAreaByPage")
    @AuthCarbonValidate
    public ResponseBase searchPovertyArea(@RequestBody PovertyAreaForm povertyAreaForm){
        String cacheKey = POVERTY_AREA_KEY+povertyAreaForm.getPageNo();
        ResultVO resultVO = (ResultVO) redisUtils.getValue(cacheKey);
        if(resultVO!=null){
            return responseSuccess(resultVO);
        }
       ResultVO clientVo = farmerInfoClient.searchPovertyAreaByPage(povertyAreaForm);
        long listSize = clientVo.getCount();
        if(listSize!=0){
            redisUtils.setValue(cacheKey,clientVo,120,TimeUnit.SECONDS);
            return responseSuccess(clientVo);
        }
        return responseError(failure());
    }

    @ApiOperation(value = "获取贫困区域详细信息",notes = "获取贫困区域详细信息(包含统计信息及基本信息)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "areaId",value = "查询条件", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getPovertyAreaById")
    @AuthCarbonValidate
    public ResponseBase getPovertyAreaInfoById(@RequestParam("areaId") String areaId){
        ResultVO resultVO = farmerInfoClient.getPovertyAreaById(areaId);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }

    @ApiOperation(value = "获取贫困区域农户列表",notes = "分页显示贫困区域农户列表信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "farmerInfoForm",value = "查询条件", dataType = "FarmerInfoForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/getPovertyAreaList")
    @AuthCarbonValidate
    public ResponseBase getPovertyAreaListById(@RequestBody FarmerInfoForm farmerInfoForm){
        String cacheKey = FARMER_LIST_KEY+farmerInfoForm.getPageNo();
        ResultVO resultVO = (ResultVO) redisUtils.getValue(cacheKey);
        if(resultVO!=null){
            return responseSuccess(resultVO);
        }
        ResultVO farmerVO = farmerInfoClient.getFarmerInfoByAreaId(farmerInfoForm);
        Long listSize = farmerVO.getCount();
        if(listSize!=0){
            redisUtils.setValue(cacheKey,farmerVO,120,TimeUnit.SECONDS);
            return responseSuccess(farmerVO);
        }
        return responseError(farmerVO);
    }


    @ApiOperation(value = "获取贫困户详细信息",notes = "获取贫困户详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "guid",value = "查询条件", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getById")
    @AuthCarbonValidate
    public ResponseBase getFarmerInfoById(@RequestParam("guid") String farmerId){
       ResultVO resultVO = farmerInfoClient.getFarmerInfoById(farmerId);
       if(resultVO.getCode()==0){
           return responseSuccess(resultVO);
       }
       return  responseError(resultVO);
    }

}
