package com.hcloud.microserver.bank.app;

import com.google.common.collect.Lists;
import com.hcloud.microserver.bank.common.BaseInfoDO;
import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.bank.forms.FarmerQueryForm;
import com.hcloud.microserver.bank.forms.GoodsCarForm;
import com.hcloud.microserver.bank.forms.GoodsQueryForm;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.BeanCopyUtil;
import com.hcloud.microserver.facade.carbon.client.GoodsInfoClient;
import com.hcloud.microserver.facade.carbon.client.ProductTypeClient;
import com.hcloud.microserver.facade.carbon.forms.FarmerInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsInfoForm;
import com.hcloud.microserver.facade.carbon.forms.ProductTypeForm;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author xiezt
 */
@Slf4j
@RestController
@RequestMapping("/wap/goods")
@Api(description = "H5商品信息服务")
public class GoodsManagerController extends CarbonBaseController {


    private static final String INDEX_GOODS_TYPE_All_LIST="goodsList";


    /**
     * 成片林产品列表
     */
    private static final String GOODS_TYPE_ONE_LIST="T001list";

    /**
     * 单株产品列表
     */
    private static final String GOODS_TYPE_TWO_LIST="T002list";

    /**
     * 履约列表
     */
    private static final String INDEX_GOODS_TYPE_ONE_LIST="contractedList";

    /**
     * 中和列表
     */
    private static final String INDEX_GOODS_TYPE_TWO_LIST="neutralizeList";

    /**
     * 普惠列表
     */
    private static final String INDEX_GOODS_TYPE_THREE_LIST="normalList";


    private static final String GOODS_USE_ONE="1";

    private static final String GOODS_USE_TWO="2";

    private static final String GOODS_TYPE_ONE="T001";

    private static final String GOODS_TYPE_TOW="T002";

    @Autowired
    private GoodsInfoClient goodsInfoClient;

    @Autowired
    private ProductTypeClient productTypeClient;

    @Autowired
    private RedisUtils redisUtils;



    @ApiOperation(value = "首页按查询商品用途分页查询信息",notes = "首页分页查询商品信息（商品用户分类查询）")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsQueryForm",value = "商品查询信息", dataType = "GoodsQueryForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/searchForIndex")
    @AuthCarbonValidate
    public ResponseBase searchByPageForIndex(@RequestBody GoodsQueryForm goodsQueryForm){
        String cacheKey = INDEX_GOODS_TYPE_THREE_LIST;
        ResultVO cachePageInfo=null;
        String goodsUse = goodsQueryForm.getGoodsUse();
        if(Objects.nonNull(goodsUse)) {
            if (GOODS_USE_ONE.equals(goodsUse)) {
                cachePageInfo = (ResultVO) redisUtils.getValue(INDEX_GOODS_TYPE_ONE_LIST);
                cacheKey = INDEX_GOODS_TYPE_ONE_LIST;
            }
            if (GOODS_USE_TWO.equals(goodsUse)) {
                cachePageInfo = (ResultVO) redisUtils.getValue(INDEX_GOODS_TYPE_TWO_LIST);
                cacheKey = INDEX_GOODS_TYPE_TWO_LIST;
            }
            if(cachePageInfo==null){
                GoodsInfoForm goodsInfoForm = BeanCopyUtil.convertBean(goodsQueryForm,GoodsInfoForm.class);
                ResultVO resultVO = goodsInfoClient.searchGoodsInfoByPage(goodsInfoForm);
                redisUtils.setValue(cacheKey,resultVO,120,TimeUnit.SECONDS);
                return responseSuccess(resultVO);
            }else {
                redisUtils.setValue(cacheKey,cachePageInfo,120,TimeUnit.SECONDS);
                return responseSuccess(cachePageInfo);
            }
        }else {
            cachePageInfo = (ResultVO) redisUtils.getValue(INDEX_GOODS_TYPE_All_LIST);
            if(Objects.nonNull(cachePageInfo)){return responseSuccess(cachePageInfo);}
            GoodsInfoForm goodsInfoForm = BeanCopyUtil.convertBean(goodsQueryForm,GoodsInfoForm.class);
            ResultVO resultVO = goodsInfoClient.searchGoodsInfoByPage(goodsInfoForm);
            redisUtils.setValue(INDEX_GOODS_TYPE_All_LIST,resultVO,120,TimeUnit.SECONDS);
            return responseSuccess(resultVO);
        }
    }


    @ApiOperation(value = "按类型分页查询商品信息",notes = "获取用户信息,按类型分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsQueryForm",value = "商品查询信息", dataType = "GoodsQueryForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/searchByPage")
    @AuthCarbonValidate
    public ResponseBase searchByPage(@RequestBody GoodsQueryForm goodsQueryForm){
        String cacheKey = INDEX_GOODS_TYPE_All_LIST;
        ResultVO cachePageInfo=null;
        String goodsType = goodsQueryForm.getGoodsType();
        if(Objects.nonNull(goodsType)) {
            if (GOODS_TYPE_ONE.equals(goodsType)) {
                cachePageInfo = (ResultVO) redisUtils.getValue(GOODS_TYPE_ONE_LIST);
                cacheKey = GOODS_TYPE_ONE_LIST;
            }
            if (GOODS_TYPE_TOW.equals(goodsType)) {
                cachePageInfo = (ResultVO) redisUtils.getValue(GOODS_TYPE_TWO_LIST);
                cacheKey = GOODS_TYPE_TWO_LIST;
            }
            if(cachePageInfo==null){
                GoodsInfoForm goodsInfoForm = BeanCopyUtil.convertBean(goodsQueryForm,GoodsInfoForm.class);
                ResultVO resultVO = goodsInfoClient.searchGoodsInfoByPage(goodsInfoForm);
                redisUtils.setValue(cacheKey,resultVO,120,TimeUnit.SECONDS);
                return responseSuccess(resultVO);
            }else {
                redisUtils.setValue(cacheKey,cachePageInfo,120,TimeUnit.SECONDS);
                return responseSuccess(cachePageInfo);
            }
        }else {
            cachePageInfo = (ResultVO) redisUtils.getValue(INDEX_GOODS_TYPE_All_LIST);
            if(Objects.nonNull(cachePageInfo)){return responseSuccess(cachePageInfo);}
            GoodsInfoForm goodsInfoForm = BeanCopyUtil.convertBean(goodsQueryForm,GoodsInfoForm.class);
            ResultVO resultVO = goodsInfoClient.searchGoodsInfoByPage(goodsInfoForm);
            redisUtils.setValue(INDEX_GOODS_TYPE_All_LIST,resultVO,120,TimeUnit.SECONDS);
            return responseSuccess(resultVO);
        }
    }

    @ApiOperation(value = "获取商品信息详情",notes = "通过商品Id去查询商品明细")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsId",value = "商品id", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getById")
    @AuthCarbonValidate
    public ResponseBase getGoodsInfoById(@RequestParam("goodsId") String goodsId){
        ResultVO resultVO = goodsInfoClient.getGoodsInfoById(goodsId);
        return responseSuccess(resultVO);
    }



    @ApiOperation(value = "查询购物车列表",notes = "查询购物列表")
    @ApiResponse(code =0,message = "操作成功")
    @GetMapping("/getGoodsCarInfo")
    @AuthCarbonValidate
    public ResponseBase getGoodsCarInfo(){
       BaseInfoDO baseInfoDO =  this.initiSaveObject();
       if(baseInfoDO==null){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
       String customerId = baseInfoDO.getCustomerId();
       List<GoodsInfoForm> list = (List<GoodsInfoForm>) redisUtils.getValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId);
       return responseSuccess(success(list));
    }

    @ApiOperation(value = "添加用户购物车",notes = "在goodsCarForm里的goodsList属性，存储客户添加用户的购物车列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsCarForm",value = "购物车操作信息", dataType = "GoodsCarForm")
    })
    @ApiResponse(code =0,message = "修改成功")
    @PostMapping("/addCar")
    @AuthCarbonValidate
    public ResponseBase addGoodsCar(@RequestBody GoodsCarForm goodsCarForm){
        String customerId = goodsCarForm.getCustomerId();
        String userId = getUserId();
        log.info(userId);
        if(null!=customerId && !"".equals(customerId)){
           List<GoodsInfoForm> list = (List<GoodsInfoForm>) redisUtils.getValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId);
           List<GoodsInfoForm> goodsList = goodsCarForm.getGoodsList();
           if(null!=list && list.size()>0){
               //购物车里有历史商品列表
               if(null!=goodsList && goodsList.size()>0){
                   for (GoodsInfoForm bean:goodsList) {
                       if(compareGoodsFromCar(bean,list)){
                           //有新的商品列表需要加入购物车
                           list.add(bean);
                       }
                   }
               }
               //把新的商品列表加入缓存进行持久化
               redisUtils.setValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId,list,30,TimeUnit.DAYS);
               return responseSuccess(success(list));
           }else{
               //购物车内无历史商品列表
               if(null!=goodsList && goodsList.size()>0){
                   //需要加入新的商品信息列表进入缓存进行持久化操作
                   redisUtils.setValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId,goodsList,30,TimeUnit.DAYS);
                   return responseSuccess(success(goodsList));
               }
           }
        }
        return responseError(failure());
    }

    /**
     * 判断是否重复产品加入购物车
     * @param goodsBean
     * @param carList
     * @return
     */
    private boolean compareGoodsFromCar(GoodsInfoForm goodsBean,List<GoodsInfoForm> carList){
        if(null!=carList && carList.size()>0){
            for (GoodsInfoForm bean:carList) {
                String goodsId = goodsBean.getGuid();
                String carGoodsId = bean.getGuid();
                if(goodsId.equals(carGoodsId)){
                    return false;
                }
            }
        }
        return true;
    }


    @ApiOperation(value = "修改用户购物车",notes = "修改用户购物车")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsCarForm",value = "购物车操作信息", dataType = "GoodsCarForm")
    })
    @ApiResponse(code =0,message = "修改成功")
    @PostMapping("/modifiedCar")
    @AuthCarbonValidate
    public ResponseBase modifiedGoodsCar(@RequestBody GoodsCarForm goodsCarForm){
        String customerId = goodsCarForm.getCustomerId();

        String goodsId = goodsCarForm.getGoodsId();
        Integer goodsNum = goodsCarForm.getCarbonNum();
        if(null!=customerId && !"".equals(customerId)){
            List<GoodsInfoForm> list = (List<GoodsInfoForm>) redisUtils.getValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId);
            if(null!=list && list.size()>0){
                List<GoodsInfoForm> carList = Lists.newArrayList();
                carList.addAll(list);
                for (GoodsInfoForm info:carList) {
                    String tmpId = info.getGuid();
                    if(tmpId.equals(goodsId)){
                        if(goodsNum<=0){
                            list.remove(info);
                        }else {
                            Integer subCarbon = info.getCarbonVal().intValue();
                            Integer tempCarbon = subCarbon-goodsNum;
                            info.setCarbonVal(BigDecimal.valueOf(Long.valueOf(tempCarbon.toString())));
                        }
                        break;
                    }
                }
            }
            //需要加入新的商品信息列表进入缓存进行持久化操作
            redisUtils.setValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId,list,30,TimeUnit.DAYS);
            return responseSuccess(success());
        }
        return responseError(failure());
    }


    @ApiOperation(value = "删除购物车列表项",notes = "删除购物车列表项，传入goodsList")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsCarForm",value = "购物车操作信息", dataType = "GoodsCarForm")
    })
    @ApiResponse(code =0,message = "操作成")
    @PostMapping("/delGoodsCar")
    @AuthCarbonValidate
    public ResponseBase delGoodsCar(@RequestBody GoodsCarForm goodsCarForm){
        List<GoodsInfoForm> delList = goodsCarForm.getGoodsList();
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(Objects.isNull(baseInfoDO)){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
            String customerId = baseInfoDO.getCustomerId();
            List<GoodsInfoForm> list = (List<GoodsInfoForm>) redisUtils.getValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId);
            if(null!=list && list.size()>0){
                List<GoodsInfoForm> carList = Lists.newArrayList();
                carList.addAll(list);
                for (GoodsInfoForm bean:carList) {
                    String goodsId = bean.getGuid();
                    for(GoodsInfoForm vo:delList){
                        if(goodsId.equals(vo.getGuid())){
                            list.remove(bean);
                            break;
                        }
                    }
                }
                if(list.size()>0){
                    redisUtils.setValue(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId,list,30,TimeUnit.DAYS);
                }else{
                    redisUtils.deleteKey(GlobleConstant.CUSTOMER_CAR_PREFIX +customerId);
                }
            }
        return responseSuccess(success());
    }



    @ApiOperation(value = "通过需求碳汇量,获取可以购买的商品列表",notes = "商品列表是随机的,根据客户需求")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "carbonSkin",value = "客户需求碳汇量", dataType = "String",paramType = "query")
    })
    @ApiResponse(code =0,message = "查询成功")
    @GetMapping("/getGoodsInfoList")
    @AuthCarbonValidate
    public ResponseBase getGoodsInfoListByCarbon(@RequestParam("carbonSkin") String carbonSkin){
       ResultVO resultVO = goodsInfoClient.getGoodsInfoByCarbon(carbonSkin);
        return responseSuccess(resultVO);
    }


    @ApiOperation(value = "查询产品类型列表",notes = "查询产品类型列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "productTypeForm",value = "类型查询条件", dataType = "ProductTypeForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/getGoodsTypeList")
    @AuthCarbonValidate
    public ResponseBase getGoodsTypeList(@RequestBody ProductTypeForm productTypeForm){
        ResultVO resultVO = productTypeClient.findAll(productTypeForm);
        return responseSuccess(resultVO);
    }


    @ApiOperation(value = "通过农户Id查询商品信息",notes = "通过农户Id查询商品信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "farmerQueryForm",value = "类型查询条件", dataType = "FarmerQueryForm")
    })
    @ApiResponse(code =0,message = "查询成功")
    @PostMapping("/getGoodsByFarmerId")
    @AuthCarbonValidate
    public ResponseBase getGoodsInfoByFarmerId(@RequestBody FarmerQueryForm farmerQueryForm){
        FarmerInfoForm farmerInfoForm = BeanCopyUtil.convertBean(farmerQueryForm,FarmerInfoForm.class);
        ResultVO resultVO = goodsInfoClient.getGoodsInfoByFarmerId(farmerInfoForm);
        return responseSuccess(resultVO);
    }

}
