package com.hcloud.microserver.bank.app;

import com.hcloud.microserver.bank.common.BaseInfoDO;
import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.bank.forms.OrderQueryForm;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.facade.carbon.client.OrderClient;
import com.hcloud.microserver.facade.carbon.client.OrderPayApplyClient;
import com.hcloud.microserver.facade.carbon.forms.GoodsCredentialInfoForm;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import com.hcloud.microserver.facade.carbon.forms.OrderPayVoucherForm;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author xiezt
 */
@Slf4j
@RestController
@RequestMapping("/wap/order")
@Api(description = "H5订单服务")
public class OrderManagerController extends CarbonBaseController {

    @Autowired
    private OrderClient orderClient;

    @Autowired
    private OrderPayApplyClient orderPayApplyClient;


    @ApiOperation(value = "查询爱心用户订单信息",notes = "分类查询爱心用户订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderType",value = "爱心用户信息", dataType = "Integer",paramType = "query")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/searchByPage")
    @AuthCarbonValidate
    public ResponseBase searchCustomerOrderByClass(@RequestBody OrderQueryForm queryForm){
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(baseInfoDO==null){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
        GoodsOrderInfoForm goodsOrderInfoForm = new GoodsOrderInfoForm();
        goodsOrderInfoForm.setFkCustomerGuid(baseInfoDO.getCustomerId());
        goodsOrderInfoForm.setOrderStatus(queryForm.getOrderStatus());
        goodsOrderInfoForm.setPageNo(queryForm.getPage());
        goodsOrderInfoForm.setPageSize(queryForm.getPageSize());
        ResultVO resultVO = orderClient.getOrderInfoByCustomerId(goodsOrderInfoForm);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "查询爱心用户证书信息",notes = "分类查询爱心用户证书")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "customerInfoForm",value = "爱心用户信息", dataType = "CustomerInfoForm")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/searchCredentialList")
    @AuthCarbonValidate
    public ResponseBase searchCredentialInfo(@RequestBody GoodsCredentialInfoForm goodsCredentialInfoForm){
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(baseInfoDO==null){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
        goodsCredentialInfoForm.setFkCustomerId(baseInfoDO.getCustomerId());
        ResultVO resultVO = orderClient.getCustomerCredentialList(goodsCredentialInfoForm);
        return responseSuccess(resultVO);
    }


    @ApiOperation(value = "生成订单信息",notes = "生成订单信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId",value = "订单Id", dataType = "String")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/saveOrderInfo")
    @AuthCarbonValidate
    public ResponseBase saveOrderInfo(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm){
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(baseInfoDO==null){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
        log.info("order info is ================>{}",goodsOrderInfoForm);
        log.info("baseInfo===============>{}",baseInfoDO);
        goodsOrderInfoForm.setFkCustomerGuid(baseInfoDO.getCustomerId());
        ResultVO resultVO = orderClient.saveOrderInfoAndPayOrderInfo(goodsOrderInfoForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }


    @ApiOperation(value = "取消订单",notes = "取消订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId",value = "订单Id", dataType = "String")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/cancelOrderInfo")
    @AuthCarbonValidate
    public ResponseBase cancelOrderInfo(@RequestParam("orderId") String orderId){
        ResultVO resultVO = orderClient.callbackOrderInfo(orderId);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
       return responseError(resultVO);
    }

    @ApiOperation(value = "删除订单",notes = "删除订单")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId",value = "订单Id", dataType = "String")
    })
    @ApiResponse(code =0,message = "操作成功")
    @GetMapping("/delOrderInfo")
    @AuthCarbonValidate
    public ResponseBase delOrderInfo(@RequestParam("orderId") String orderId){
        ResultVO resultVO = orderClient.delOrderInfo(orderId);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }


    @ApiOperation(value = "生成离线支付申请",notes = "生成离线支付申请，用于用户线下支付购碳资金")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "goodsOrderInfoForm",value = "订单信息", dataType = "GoodsOrderInfoForm")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/saveOffLineApply")
    @AuthCarbonValidate
    public ResponseBase saveOffLienPayApply(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm){
        ResultVO resultVO = orderPayApplyClient.savePayApplyInfo(goodsOrderInfoForm);
       if (resultVO.getCode()==0){
           return responseSuccess(resultVO);
       }
       return responseError(resultVO);
    }



    @ApiOperation(value = "生成离线支付凭证",notes = "生成离线支付凭证，用于线下支付后的凭证记录")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderPayVoucherForm",value = "凭证信息", dataType = "OrderPayVoucherForm")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/saveOffLineVoucher")
    @AuthCarbonValidate
    public ResponseBase saveOffLienPayVoucher(@RequestBody OrderPayVoucherForm orderPayVoucherForm){
        ResultVO resultVO = orderPayApplyClient.saveOffLienPayVoucher(orderPayVoucherForm);
        if (resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }


    @ApiOperation(value = "快速购买某一农户下的可售商品",notes = "快速购买某一农户下的可售商品")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderQueryForm",value = "购买参数", dataType = "OrderQueryForm")
    })
    @ApiResponse(code =0,message = "操作成功")
    @PostMapping("/saveOrderForFarmer")
    @AuthCarbonValidate
    public ResponseBase insertOrderInfoByFarmerId(@RequestBody OrderQueryForm orderQueryForm){
        BaseInfoDO baseInfoDO = this.initiSaveObject();
        if(baseInfoDO==null){return responseError(new ResultVO(ResultEnum.TOKEN_EXPIRED));}
        return null;
    }

    @ApiOperation(value = "根据证书guid获取证书信息",notes = "根据证书guid获取证书信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "guid",value = "证书guid", dataType = "string")
    })
    @ApiResponse(code =0,message = "操作成功")
    @GetMapping("/getCredential")
    @AuthCarbonValidate
    public ResponseBase getCredentialInfo(@RequestParam(name = "guid") String guid) {
        ResultVO resultVO = this.orderClient.getCredentialInfo(guid);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "根据订单id获取证书信息",notes = "根据订单id获取证书信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "orderId",value = "订单guid", dataType = "string")
    })
    @ApiResponse(code =0,message = "操作成功")
    @GetMapping("/getCredentialByOrderId")
    @AuthCarbonValidate
    public ResponseBase getCredentialByOrderId(@RequestParam(name = "orderId") String orderId) {
        ResultVO resultVO = this.orderClient.getCredentialByOrderId(orderId);
        return responseSuccess(resultVO);
    }
}
