package com.hcloud.microserver.bank.biz;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.util.JavaBeanUtils;
import com.hcloud.microserver.facade.carbon.client.OrderClient;
import com.hcloud.microserver.facade.carbon.forms.GoodsOrderInfoForm;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.connection.MessageListener;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @author xiezt
 * redis key 过期事件监听
 */
@Slf4j
@Component("redisMessageListener")
public class RedisMessageListener implements MessageListener {

    private static final String ORDER_START_STR="ord";

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private OrderClient orderClient;

    @Override
    public void onMessage(Message message, byte[] pattern) {
        String str = (String) redisTemplate.getValueSerializer().deserialize(message.getBody());
        log.info("redis key value===================>{}",str);
        //判断是否是订单数据
        if(str.startsWith(ORDER_START_STR)){
            String [] orderStr = str.split(":");
            log.info("order no==============>{}",orderStr[1]);
            ResultVO resultVO = orderClient.getOrderInfoByOrderNo(orderStr[1]);
            if(resultVO.getCode()==0){
                Map orderMap = (Map) resultVO.getData();
                GoodsOrderInfoForm orderInfoForm = JavaBeanUtils.mapToEntity(orderMap,GoodsOrderInfoForm.class);
                String orderId = orderInfoForm.getGuid();
                ResultVO orderVo = orderClient.callbackOrderInfo(orderId);
                if(orderVo.getCode()==0){
                    redisTemplate.delete(str);
                }
            }
        }
    }
}
