package com.hcloud.microserver.bank.biz;

import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.enums.SMSEnum;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.sms.NoteInfo;
import com.hcloud.microserver.commoncore.util.CodeUtil;
import com.hcloud.microserver.commoncore.util.SMSUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * @author xiezt
 */
@Slf4j
@Component("smsComponent")
public class SmsComponent {

    private static final String SEND_SUCCESS="OK";

    @Autowired
    private RedisUtils redisUtils;

    @Autowired
    private SMSUtil smsUtil;

    /**
     * 根据电话号码发送验证码
     * @param phone
     * @return
     */
    public ResultVO sendPhoneMsm(String phone,SMSEnum smsEnum){
        //获取随机验证码
        String code = CodeUtil.getSearchId();
        //发送验证码
        NoteInfo noteInfo = smsUtil.sendNote(phone, smsEnum.getCode(), code);
        String recode = noteInfo.getCode();
        ResultVO resultVO=null;
        //发送成功
        if (SEND_SUCCESS.equalsIgnoreCase(recode)){
            resultVO = new ResultVO(ResultEnum.SUCCESS);
        }else{
            resultVO = new ResultVO(ResultEnum.FAILURE);
            resultVO.setMsg("短信发送失败");
        }
        //放入redis存储5分钟销毁
        redisUtils.setValue(phone + ":" + code, code, 5, TimeUnit.MINUTES);
        resultVO.setData(code);
        log.info("sms send code is ======================>{}",code);
        return resultVO;
    }
}
