package com.hcloud.microserver.bank.biz;

import com.hcloud.microserver.bank.config.WxMpConfiguration;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.base.Token;
import com.hcloud.microserver.commoncore.constant.GlobleConstant;
import com.hcloud.microserver.commoncore.service.RedisUtils;
import com.hcloud.microserver.commoncore.util.JavaBeanUtils;
import com.hcloud.microserver.commoncore.util.TokenUtil;
import com.hcloud.microserver.facade.carbon.client.CustomerClient;
import com.hcloud.microserver.facade.carbon.forms.CustomerInfoForm;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.api.WxMpService;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;

/**
 * @author xiezt
 */
@Slf4j
@Component("weChatManager")
public class WechatManagerComponent{

    private static final String WECHAT_USER_INFO="wechatUserInfo";

    private static final Integer LOGIN_CHANNEL_WECHAT=1;

    private static final String CUSTOMER_INFO_PREFIX="customer_";

    @Value("${wx.mp.appId}")
    private String appid;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private CustomerClient customerClient;

    @Autowired
    private RedisUtils redisUtils;

    public Map getWechatAccessToken(WxMpUser wxMpUser){
        log.info("wxMpUser======================>{}",wxMpUser);
        Map map = new HashMap();
        if(null!=wxMpUser){
            if(getRedisUserCache(wxMpUser,map)){
                return map;
            }
            productAccessToken(wxMpUser,map);
            return map;
        }
        log.info("没有获取到微信用户基本信息，请联系开发人员");
        return null;
    }

    public int updateCustomerWeChatInfo(String customerId,WxMpUser wxMpUser){
        CustomerInfoForm customerInfoForm = new CustomerInfoForm();
        customerInfoForm.setGuid(customerId);
        customerInfoForm.setWxId(wxMpUser.getOpenId());
        customerInfoForm.setUnionId(wxMpUser.getUnionId());
        customerInfoForm.setModifiedTime(new Date());
        customerInfoForm.setChannelType(1);
        ResultVO resultVO = customerClient.updateCustomerInfoForWeChat(customerInfoForm);
        if(resultVO.getCode()==0){
            return 1;
        }
        return 0;
    }


    /**
     * 获取用户信息,通过订单用户外键
     * @param fkCustomerId
     * @return
     */
    public CustomerInfoForm getCustomerInfoByCustomerId(String fkCustomerId){
       if(null!=fkCustomerId && !"".equals(fkCustomerId)){
           CustomerInfoForm customerInfoForm = (CustomerInfoForm) redisTemplate.opsForValue().get(CUSTOMER_INFO_PREFIX+fkCustomerId);
           if(customerInfoForm!=null){return customerInfoForm;}
           ResultVO resultVO =  customerClient.getCustomerInfoById(fkCustomerId);
           Map customerInfoMap = (Map) resultVO.getData();
           CustomerInfoForm customerInfo = JavaBeanUtils.mapToEntity(customerInfoMap,CustomerInfoForm.class);
           redisTemplate.opsForValue().set(CUSTOMER_INFO_PREFIX+fkCustomerId,customerInfo,600,TimeUnit.SECONDS);
           return customerInfo;
        }
       return null;
    }

    /**
     * 缓存过期后的处理
     * @param wxMpUser
     * @param resultMap
     */
    private void productAccessToken(WxMpUser wxMpUser, Map resultMap){
        String code =wxMpUser.getUnionId();
        if(Objects.isNull(code)){
            code = wxMpUser.getOpenId();
        }
        ResultVO resultVO = customerClient.getCustomerInfoByChannelInfo(LOGIN_CHANNEL_WECHAT,code);
        if(resultVO.getCode()==0){
            Map userMap = (Map) resultVO.getData();
            CustomerInfoForm customerInfoForm = JavaBeanUtils.mapToEntity(userMap,CustomerInfoForm.class);
            //更新微信关键数字
            this.updateCustomerWeChatInfo(customerInfoForm.getGuid(),wxMpUser);
            String guid = customerInfoForm.getGuid();
            if(null!=guid && !"".equals(guid)){
                //重新查询客户信息,以便进行缓存
                customerInfoForm = this.getCustomerInfoByCustomerId(guid);
                ValueOperations<String, Object> operations = redisTemplate.opsForValue();
                Token token = TokenUtil.createJwtToken(guid, GlobleConstant.CURRENT_MEMBER);
                resultMap.put(GlobleConstant.ACCESS_TOKEN,token.getAccessToken());
                resultMap.put(WECHAT_USER_INFO,customerInfoForm);
                resultMap.put(GlobleConstant.WECHAT_INFO_PREFIX,wxMpUser);
                operations.set(token.getAccessToken(),resultMap,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                if(customerInfoForm.getUnionId()!=null){
                    operations.set(customerInfoForm.getUnionId(),customerInfoForm,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                }else{
                    operations.set(customerInfoForm.getWxId(),customerInfoForm,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                }
            }else{
                CustomerInfoForm customerForm = new CustomerInfoForm();
                customerForm.setLoginType(LOGIN_CHANNEL_WECHAT);
                customerForm.setWxId(wxMpUser.getOpenId());
                customerForm.setUnionId(wxMpUser.getUnionId());
                customerForm.setSex(wxMpUser.getSex());
                customerForm.setHeadImage(wxMpUser.getHeadImgUrl());
                customerForm.setNickName(wxMpUser.getNickname());
                customerForm.setCustomerType(1);
                customerForm.setIsEnable(1);
                customerForm.setState(1);
                customerForm.setCreateTime(new Date());
                customerForm.setModifiedTime(new Date());
                ResultVO innerVO = customerClient.saveCustomerInfo(customerForm);
                if(innerVO.getCode()==0){
                    ValueOperations<String, Object> operations = redisTemplate.opsForValue();
                    Map customerMap = (Map) innerVO.getData();
                    CustomerInfoForm customerInfo = JavaBeanUtils.mapToEntity(customerMap,CustomerInfoForm.class);
                    Token token = TokenUtil.createJwtToken(customerInfo.getGuid(), GlobleConstant.CURRENT_MEMBER);
                    resultMap.put(GlobleConstant.ACCESS_TOKEN,token.getAccessToken());
                    resultMap.put(WECHAT_USER_INFO,customerInfo);
                    resultMap.put(GlobleConstant.WECHAT_INFO_PREFIX,wxMpUser);
                    operations.set(token.getAccessToken(),resultMap,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                    if(customerForm.getUnionId()!=null){
                        operations.set(customerInfo.getUnionId(),customerInfo,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                    }else{
                        operations.set(customerInfo.getWxId(),customerInfo,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                    }
                }
                log.info("GoodsTracedUserForm======================>{}",customerForm);
            }
        }
    }


    /**
     * 缓存未过期的处理
     * @param wxMpUser
     * @param resultMap
     * @return
     */
    private boolean getRedisUserCache(WxMpUser wxMpUser,Map resultMap) {
        String unionId = wxMpUser.getUnionId();
        String openId = wxMpUser.getOpenId();
        if (null != unionId && !"".equals(unionId)) {
            Object obj = redisTemplate.opsForValue().get(unionId);
            log.info("obj========================>{}", obj);
            if (obj != null) {
                ValueOperations<String, Object> operations = redisTemplate.opsForValue();
                CustomerInfoForm comp = (CustomerInfoForm) obj;
                Token token = TokenUtil.createJwtToken(comp.getGuid(), GlobleConstant.CURRENT_MEMBER);
                resultMap.put(GlobleConstant.ACCESS_TOKEN,token.getAccessToken());
                resultMap.put(WECHAT_USER_INFO, comp);
                resultMap.put(GlobleConstant.WECHAT_INFO_PREFIX,wxMpUser);
                operations.set(token.getAccessToken(),resultMap,GlobleConstant.TOKEN_EXPIRES_SECOND,TimeUnit.SECONDS);
                operations.set(unionId, comp, GlobleConstant.ACCESS_TOKEN_EXPIRES_SECOND, TimeUnit.SECONDS);
                return Boolean.TRUE;
            }
        }else if(null != openId && !"".equals(openId)){
            Object obj = redisTemplate.opsForValue().get(openId);
            log.info("obj========================>{}", obj);
            if (obj != null) {
                ValueOperations<String, Object> operations = redisTemplate.opsForValue();
                CustomerInfoForm comp = (CustomerInfoForm) obj;
                Token token = TokenUtil.createJwtToken(comp.getGuid(), GlobleConstant.CURRENT_MEMBER);
                resultMap.put(GlobleConstant.ACCESS_TOKEN, token.getAccessToken());
                resultMap.put(WECHAT_USER_INFO, comp);
                resultMap.put(GlobleConstant.WECHAT_INFO_PREFIX, wxMpUser);
                operations.set(token.getAccessToken(), resultMap, GlobleConstant.TOKEN_EXPIRES_SECOND, TimeUnit.SECONDS);

                operations.set(openId, comp, GlobleConstant.ACCESS_TOKEN_EXPIRES_SECOND, TimeUnit.SECONDS);
                return Boolean.TRUE;
            }
        }
        return Boolean.FALSE;
    }


    /**
     * 获取访问微信接口的API TOKEN
     * @throws WxErrorException
     */
    public String getApiToken() throws WxErrorException {
        final WxMpService wxMpService = WxMpConfiguration.getMpServices().get(appid);
        String apiToken = (String) redisUtils.getValue(GlobleConstant.WECHAT_API_TOKEN);
        if(Objects.isNull(apiToken) || "".equals(apiToken)){
            apiToken = wxMpService.getAccessToken(true);
            redisUtils.setValue(GlobleConstant.WECHAT_API_TOKEN,apiToken,7200,TimeUnit.SECONDS);
        }
        return apiToken;
    }

}
