package com.hcloud.microserver.bank.common;

import com.hcloud.microserver.bank.biz.SmsComponent;
import com.hcloud.microserver.bank.biz.WechatManagerComponent;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.SMSEnum;
import com.hcloud.microserver.commoncore.util.EasyHttpUtils;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;

/**
 * @author JYJ
 */
@Slf4j
@RestController
@RequestMapping("/web/normal")
@Api(description = "公共操作服务")
public class NormalController extends CarbonBaseController {

    private static final String TARGET_IMAGE_DIC_NAME="images/";
    private static final String TARGET_FILE_DIC_NAME="files/";

    @Value("${img.location}")
    private String UPLOAD_FILE_PATH;
    /**
     * 微信下载图片远端地址
     */
    private static final String DOWNLOAD_IMAGES_FOR_WX="http://file.api.weixin.qq.com/cgi-bin/media/get?";

    @Autowired
    private SmsComponent smsComponent;
    @Autowired
    private WechatManagerComponent weChatManager;

    @ApiOperation(value = "图片上传",notes = "上传问题图片，图片限制大小10MB")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file",value = "上传内容",dataType = "MultipartFile")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @AuthCarbonValidate
    @PostMapping("/uploadPics")
    public ResponseBase uploadPics(@RequestParam("file") MultipartFile file, HttpServletRequest request){
        ResultVO resultVO = this.uploadImageFiles(file,TARGET_IMAGE_DIC_NAME);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "文件上传",notes = "上传文件，文件限制大小10MB")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file",value = "上传内容",dataType = "MultipartFile")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @AuthCarbonValidate
    @PostMapping("/uploadFiles")
    public ResponseBase uploadFiles(@RequestParam("file") MultipartFile file, HttpServletRequest request){
        log.info("==================>{}",file);
        ResultVO resultVO = this.uploadImageFiles(file,TARGET_FILE_DIC_NAME);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "上传文件(微信浏览器专用)",notes = "上传文件(微信浏览器专用)")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "imageSign",value = "微信图片密文",dataType = "String",paramType = "query")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @AuthCarbonValidate
    @GetMapping("/uploadWeChatImg")
    public ResponseBase uploadFilesForWeChat(@RequestParam("imageSign") String signImageStr){
        StringBuffer stb = new StringBuffer(DOWNLOAD_IMAGES_FOR_WX);
        try {
            String token = weChatManager.getApiToken();
            stb.append("access_token=");
            stb.append(token);
            stb.append("&media_id=");
            stb.append(signImageStr);
            log.info("0t55HYhg1JEXqV_YJ0ox6f8uq0zu1SmU9rF0kzzMgq8KuGIXkrT8ot5KAOWttVId");
            log.info("curl===================>{}",stb.toString());
            String curl = stb.toString();
            String path = UPLOAD_FILE_PATH+TARGET_IMAGE_DIC_NAME;
            String imageType = EasyHttpUtils.LoadInstance().getUrlToImg(curl,signImageStr,path);
            if(null!=imageType && !"".equals(imageType)){
                String imageUrl =TARGET_IMAGE_DIC_NAME+signImageStr+"."+imageType;
                return responseSuccess(success(imageUrl));
            }
        } catch (Exception e) {
            log.info("wx get accessToken error please call programmer!!!!");
            e.printStackTrace();
        }
        return responseError(failure());
    }


    @ApiOperation(value = "绑定手机发送短信",notes = "绑定用户手机号")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone",value = "手机号码",dataType = "String",paramType = "query")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @AuthCarbonValidate
    @GetMapping("/sendPhoneMsg")
    public ResponseBase sendPhoneMsg(@RequestParam("phone") String phone){
        ResultVO resultVO = smsComponent.sendPhoneMsm(phone,SMSEnum.AUTHENTICATION);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "找回密码发送短信",notes = "通过用户手机号，修改遗忘密码")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "phone",value = "手机号码",dataType = "String",paramType = "query")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @AuthCarbonValidate
    @GetMapping("/sendUpdatePasswdMsg")
    public ResponseBase sendUpdatePasswdMsg(@RequestParam("phone") String phone){
        ResultVO resultVO = smsComponent.sendPhoneMsm(phone,SMSEnum.CHANGE_PASSWORD);
        return responseSuccess(resultVO);
    }
}
