package com.hcloud.microserver.bank.web;

import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.CarbonAuthEnum;
import com.hcloud.microserver.sys.facade.client.SysDepartmentClient;
import com.hcloud.microserver.sys.facade.forms.SysDeptForm;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/sys/dept")
@Api(description = "系统部门信息服务")
public class SysDeptController extends CarbonBaseController {

    @Autowired
    private SysDepartmentClient sysDepartmentClient;

    @ApiOperation(value = "查询部门信息", notes = "分页查询信息,默认10行1页")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysDeptForm", value = "部门管理页面参数对象", dataType = "SysDeptForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("/searchByPage")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_DEPT_QUERY)
    public ResponseBase searchByPage(@RequestBody SysDeptForm sysDeptForm){
       ResultVO resultVO =  sysDepartmentClient.queryDeptInfoByPage(sysDeptForm);
       return responseSuccess(resultVO);
    }

    @ApiOperation(value = "新增部门信息", notes = "根据部门基本信息新增部门")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysDeptForm", value = "部门管理页面参数对象", dataType = "SysDeptForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_DEPT_ADD)
    @PostMapping("/save")
    public ResponseBase saveDepartmentInfo(@RequestBody SysDeptForm sysDeptForm){
        ResultVO resultVO = sysDepartmentClient.saveDeptInfo(sysDeptForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }

    @ApiOperation(value = "更新部门信息", notes = "根据部门id更新部门信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysDeptForm", value = "部门管理页面参数对象", dataType = "SysDeptForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("update")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_DEPT_EDIT)
    public ResponseBase updateDeptInfo(@RequestBody SysDeptForm sysDeptForm){
        ResultVO resultVO = sysDepartmentClient.updateDeptInfo(sysDeptForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }

    @ApiOperation(value = "删除部门信息", notes = "根据部门id删除部门信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysDeptForm", value = "部门管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_DEPT_DELETE)
    @PostMapping("del")
    public ResponseBase delDeptInfo(@RequestBody SysDeptForm sysDeptForm){
        ResultVO resultVO = sysDepartmentClient.delDeptInfo(sysDeptForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }
}
