package com.hcloud.microserver.bank.web;

import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.CarbonAuthEnum;
import com.hcloud.microserver.sys.facade.client.SysRoleClient;
import com.hcloud.microserver.sys.facade.forms.SysRoleForm;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/sys/role")
@Api(description = "系统角色服务")
public class SysRoleController extends CarbonBaseController {

    @Autowired
    private SysRoleClient sysRoleClient;

    @ApiOperation(value = "分页查询角色信息", notes = "默认分页参数pageNo，pageSize")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysRoleForm", value = "角色管理页面参数对象", dataType = "SysRoleForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_ROLE_QUERY)
    @PostMapping("/searchByPage")
    public ResponseBase querySysRoleInfo(@RequestBody SysRoleForm sysRoleForm){
        ResultVO resultVO = sysRoleClient.querySysRoleInfo(sysRoleForm);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "查询角色基本权限信息", notes = "以树型结构递归显示")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysRoleForm", value = "角色管理页面参数对象", dataType = "SysRoleForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_PERMISSION_ROLE_QUERY)
    @PostMapping("/getAuth")
    public ResponseBase getAuthInfoByRole(@RequestBody SysRoleForm sysRoleForm){
        ResultVO resultVO = sysRoleClient.queryRolePermissionById(sysRoleForm);
        return responseSuccess(resultVO);
    }
    @ApiOperation(value = "新增角色基本信息", notes = "保存角色基本信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysRoleForm", value = "角色管理页面参数对象", dataType = "SysRoleForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("/save")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_ROLE_ADD)
    public ResponseBase saveRoleInfo(@RequestBody SysRoleForm sysRoleForm){
        ResultVO resultVO = sysRoleClient.saveRoleInfo(sysRoleForm);
        if(resultVO.getCode()==0){
            return responseSuccess(resultVO);
        }
        return responseError(resultVO);
    }

    @ApiOperation(value = "更新角色基本信息", notes = "更新角色基本信息,不更新权限信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysRoleForm", value = "角色管理页面参数对象", dataType = "SysRoleForm")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("/update")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_ROLE_EDIT)
    public ResponseBase updateRoleInfo(@RequestBody SysRoleForm sysRoleForm){
       ResultVO resultVO =  sysRoleClient.updateRoleInfo(sysRoleForm);
       if (resultVO.getCode()==0){return responseSuccess(resultVO);}
       return responseError(resultVO);
    }

    @ApiOperation(value = "更新角色权限信息", notes = "更新角色权限基本信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysRoleForm", value = "角色管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_PERMISSION_ROLE_UPDATE)
    @PostMapping("/updateAuth")
    public ResponseBase updateRoleAuth(@RequestBody SysRoleForm sysRoleForm){
       ResultVO resultVO =  sysRoleClient.updateRolePermissionInfo(sysRoleForm);
       if(resultVO.getCode()==0){return responseSuccess(resultVO);}
       return responseError(resultVO);
    }
    @ApiOperation(value = "删除角色基本信息", notes = "删除角色基本信息,包含权限信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysRoleForm", value = "角色管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("/del")
    @AuthCarbonValidate(value = CarbonAuthEnum.SYS_ROLE_DELETE)
    public ResponseBase delRoleInfo(@RequestBody SysRoleForm sysRoleForm){
        ResultVO resultVO = sysRoleClient.delRolePermissionInfo(sysRoleForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }
}
