package com.hcloud.microserver.bank.web;


import com.hcloud.microserver.bank.common.CarbonBaseController;
import com.hcloud.microserver.commoncore.annotation.AuthCarbonValidate;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.CarbonAuthEnum;
import com.hcloud.microserver.sys.facade.client.SysUserClient;
import com.hcloud.microserver.sys.facade.forms.SysMangerForm;
import com.hcloud.microserver.sys.facade.forms.SysUserForm;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * @author xiezt
 */
@RestController
@RequestMapping("/sys/user")
@Api(description = "系统用户服务")
public class SysUserController extends CarbonBaseController {

    @Autowired
    private SysUserClient sysUserClient;

    @ApiOperation(value = "用户登录", notes = "根据用户名及密码完成登录动作，")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "account", value = "用户名", dataType = "String"),
            @ApiImplicitParam(name = "password", value = "用户密码", dataType = "String")
    })
    @ApiResponse(code = 600, message = "账号或密码不正确")
    @RequestMapping("/login")
    @AuthCarbonValidate
    public ResponseBase login(@RequestParam("account") String account, @RequestParam("password") String password){
        ResultVO resultVO = sysUserClient.login(account,password);
        return responseSuccess(resultVO);
    }


    @ApiOperation(value = "登录用户登出",notes = "根据用户accessToken信息注销")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "accessToken",value = "用户accessToken",dataType = "String")
    })
    @ApiResponse(code = 0,message = "操作成功")
    @RequestMapping("/logout")
    @AuthCarbonValidate
    public ResponseBase logout(@RequestParam("token") String accessToken){
        ResultVO resultVO =  sysUserClient.logout(accessToken);
        return responseSuccess(resultVO);

    }

    @ApiOperation(value = "保存用户信息", notes = "保存用户信息时，添加用户所选角色的相应权限")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysMangerForm", value = "用户管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @RequestMapping("/add")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_USER_CREATE)
    public ResponseBase addUser(@RequestBody SysUserForm sysUserForm){
       ResultVO resultVO =  sysUserClient.saveUserInfo(sysUserForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
       return responseError(resultVO);
    }

    @ApiOperation(value = "更新用户信息", notes = "更新用户信息时，变更角色后会删除缓存信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysMangerForm", value = "用户管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @RequestMapping("/update")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_USER_EDIT)
    public ResponseBase modifiedUser(@RequestBody SysUserForm sysUserForm){
        ResultVO resultVO = sysUserClient.updateUserInfo(sysUserForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }


    @ApiOperation(value = "分页查询用户基本信息", notes = "分页查询用户信息，默认每页每页10")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysMangerForm", value = "用户管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @RequestMapping("/searchByPage")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_USER_QUERY)
    public ResponseBase searchByPage(@RequestBody SysMangerForm sysMangerForm){
        ResultVO resultVO = sysUserClient.queryUserListByPage(sysMangerForm);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "查询用户基本权限信息", notes = "以树型结构递归显示")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysMangerForm", value = "用户管理页面参数对象", dataType = "DefinedObject")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("/getAuthTreeById")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_PERMISSION_USER_QUERY)
    public ResponseBase searchUserAuthById(@RequestBody SysMangerForm sysMangerForm){
        ResultVO resultVO = sysUserClient.queryUserPermissionById(sysMangerForm);
        return responseSuccess(resultVO);
    }

    @ApiOperation(value = "查询部门及角色信息", notes = "系统内所有部门及角色信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "refSysApp", value = "引用系统app标记，默认是1", dataType = "String")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @GetMapping("/selectList")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_DEPT_ROLE_INFO)
    public ResponseBase getDeptAndRoleInfo(@RequestParam("sysApp") String refSysApp){
        ResultVO resultVO = sysUserClient.getDeptAndRoleInfo(refSysApp);
        return responseSuccess(resultVO);
    }


    @ApiOperation(value = "更新用户权限信息", notes = "根据用户变更的权限列表，更新用户权限")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysUserForm", value = "用户管理页面参数对象", dataType = "String")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @RequestMapping("/updateAuth")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_PERMISSION_USER_UPDATE)
    public ResponseBase modifiedUserAuth(@RequestBody SysUserForm sysUserForm){
        ResultVO resultVO = sysUserClient.updateUserPermission(sysUserForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }

    @ApiOperation(value = "删除用户信息", notes = "根据用户id删除用户信息，含权限信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysUserForm", value = "用户管理页面参数对象", dataType = "String")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @PostMapping("/del")
    @AuthCarbonValidate(CarbonAuthEnum.SYS_USER_DELETE)
    public ResponseBase delUserInfo(@RequestBody SysUserForm sysUserForm){
        ResultVO resultVO =  sysUserClient.delUserPermission(sysUserForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }

    @ApiOperation(value = "禁用（启用）用户信息", notes = "根据用户id启用（禁用）用户信息，不删除权限信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "sysUserForm", value = "用户管理页面参数对象", dataType = "String")
    })
    @ApiResponse(code = 0, message = "操作成功")
    @RequestMapping("/forbidden")
    @AuthCarbonValidate
    public ResponseBase forbiddenUser(@RequestBody SysUserForm sysUserForm){
       ResultVO resultVO = sysUserClient.enableUserInfo(sysUserForm);
        if(resultVO.getCode()==0){return responseSuccess(resultVO);}
        return responseError(resultVO);
    }

}
