package com.hcloud.microserver.bank.web.provide;

import com.hcloud.microserver.commoncore.base.BaseController;
import com.hcloud.microserver.commoncore.base.ResponseBase;
import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.facade.carbon.client.*;
import com.hcloud.microserver.facade.carbon.forms.*;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@RequestMapping("data")
@Api(tags = "数据接口")
public class DataProvideController extends BaseController {

    @Autowired
    private CustomerClient customerClient;

    @Autowired
    private FarmerInfoClient farmerInfoClient;

    @Autowired
    private CollectiveFarmerClient collectiveFarmerClient;

    @Autowired
    private GoodsInfoClient goodsInfoClient;

    @Autowired
    private OrderClient orderClient;

    @Autowired
    private OrderPayClient orderPayClient;

    /**
     * 客户账号信息
     *
     * @return
     */
    @PostMapping("customers")
    @ApiOperation(value = "客户账号信息", httpMethod = "POST", notes = "客户账号信息")
    public ResponseBase customers(@RequestBody CustomerInfoForm customerInfoForm) {
        ResultVO resultVO = this.customerClient.searchCustomerInfoByPage(customerInfoForm);
        return responseSuccess(resultVO);
    }

    /**
     * 企业信息
     *
     * @param customerCompanyInfoForm
     * @return
     */
    @ApiOperation(value = "企业信息", httpMethod = "POST", notes = "注册认证企业信息")
    @PostMapping("companies")
    public ResponseBase companies(@RequestBody CustomerCompanyInfoForm customerCompanyInfoForm) {
        ResultVO resultVO = this.customerClient.searchCompanyByPage(customerCompanyInfoForm);
        return responseSuccess(resultVO);
    }

    /**
     * 产品归属
     * @param productGuid
     * @return
     */
    @PostMapping("belongs")
    @ApiOperation(value = "归属信息", httpMethod = "POST", notes = "归属")
    public ResponseBase farmers(@RequestParam String productGuid) {
        ResultVO resultVO = this.farmerInfoClient.searchByProductGuid(productGuid);
        return responseSuccess(resultVO);
    }

    /**
     * 农户信息
     *
     * @param farmerInfoForm
     * @return
     */
    @PostMapping("farmers")
    @ApiOperation(value = "村民信息", httpMethod = "POST", notes = "归属")
    public ResponseBase farmers(@RequestBody FarmerInfoForm farmerInfoForm) {
        ResultVO resultVO = this.farmerInfoClient.searchByPage(farmerInfoForm);
        return responseSuccess(resultVO);
    }

    /**
     * 集体信息
     *
     * @param collectiveFarmerForm
     * @return
     */
    @ApiOperation(value = "村名组信息", httpMethod = "POST", notes = "归属")
    @PostMapping("collectives")
    public ResponseBase collectives(@RequestBody CollectiveFarmerForm collectiveFarmerForm) {
        ResultVO resultVO = this.collectiveFarmerClient.searchByPage(collectiveFarmerForm);
        return responseSuccess(resultVO);
    }

    /**
     * 商品信息
     *
     * @param goodsInfoForm
     * @return
     */
    @ApiOperation(value = "商品信息", httpMethod = "POST", notes = "森林碳汇开发信息里的森林碳汇商品数据")
    @PostMapping("goods")
    public ResponseBase goods(@RequestBody GoodsInfoForm goodsInfoForm) {
        ResultVO resultVO = this.goodsInfoClient.searchByPage(goodsInfoForm);
        return responseSuccess(resultVO);
    }

    /**
     * 订单信息
     *
     * @param goodsOrderInfoForm
     * @return
     */
    @ApiOperation(value = "订单数据", httpMethod = "POST", notes = "交易订单数据里的订单数据")
    @PostMapping("orders")
    public ResponseBase orders(@RequestBody GoodsOrderInfoForm goodsOrderInfoForm) {
        ResultVO resultVO = this.orderClient.searchOrderInfoByPage(goodsOrderInfoForm);
        return responseSuccess(resultVO);
    }

    /**
     * 证书信息
     *
     * @param goodsCredentialInfoForm
     * @return
     */
    @ApiOperation(value = "证书信息", httpMethod = "POST", notes = "交易订单数据里的激励证书数据")
    @PostMapping("customer/credentials")
    public ResponseBase credentials(@RequestBody GoodsCredentialInfoForm goodsCredentialInfoForm) {
        ResultVO resultVO = this.orderClient.getCustomerCredentialList(goodsCredentialInfoForm);
        return responseSuccess(resultVO);
    }

    @PostMapping("order/credentials")
    public ResponseBase credentials(@RequestParam String orderId) {
        ResultVO resultVO = this.orderClient.getCredentialByOrderId(orderId);
        return responseSuccess(resultVO);
    }

    /**
     * 在线支付信息
     *
     * @param
     * @return
     */
    @ApiOperation(value = "支付信息", httpMethod = "POST", notes = "交易订单数据里的支付信息数据")
    @PostMapping("order/payment")
    public ResponseBase payment(@RequestBody PaymentForm paymentForm) {
        /*ResultVO resultVO = this.orderPayClient.getOrderPayListByOrderId(orderId);*/
        ResultVO resultVO = this.orderPayClient.queryPaymentByPage(paymentForm);
        return responseSuccess(resultVO);
    }
}
