package com.hcloud.microserver.commoncore.base;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.web.BodyReaderHttpServletRequestWrapper;

import java.io.PrintWriter;
import java.lang.annotation.Annotation;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

/**
 * 权限拦截器定义
 *
 * @author yys
 */
@Slf4j
@Configuration
public abstract class BaseInterceptor extends HandlerInterceptorAdapter {


    /**
     * 实现每个系统不同的鉴权方式
     */
    public abstract ResultEnum authUserPermission(String accessToken, RedisTemplate redisTemplate,
                                                  Annotation authValidate);

    /**
     * 获取系统AccessToken
     */
    protected String getAccessToken(HttpServletRequest request) {
        log.info("uri====================>{}", request.getRequestURI());
        log.info("url====================>{}", request.getRequestURL());
        log.info("realIp========================>{}", getIpAddress(request));
        String accessToken = null;
        //从头部提取accessToken信息
        if (accessToken == null || "".equalsIgnoreCase(accessToken)) {
            accessToken = request.getHeader("accessToken");
        }
        if (accessToken == null || "".equalsIgnoreCase(accessToken)) {
            Map<String, String[]> maps = request.getParameterMap();
            if (null != maps && maps.size() > 0) {
                for (Map.Entry<String, String[]> entry : maps.entrySet()) {
                    if ("accessToken".equalsIgnoreCase(entry.getKey())) {
                        log.info("非requestBody提交请求，获取accessToken");
                        accessToken = entry.getValue()[0];
                    }
                }
            }
        }
        if (accessToken == null || "".equalsIgnoreCase(accessToken)) {
            JSONObject parameterMap = JSON.parseObject(
                    new BodyReaderHttpServletRequestWrapper(request).getBodyString(request));
            if (null != parameterMap) {
                log.info("requestBody提交请求，获取accessToken");
                accessToken = parameterMap.getString("accessToken");
            }
        }
        log.info("accessToken============>{}", accessToken);
        return accessToken;
    }

    protected String getAccessTokenFromHeader(HttpServletRequest request) {
        log.info("uri====================>{}", request.getRequestURI());
        log.info("url====================>{}", request.getRequestURL());
        log.info("realIp========================>{}", getIpAddress(request));
        String accessToken = null;
        //从头部提取accessToken信息
        if (accessToken == null || "".equalsIgnoreCase(accessToken)) {
            accessToken = request.getHeader("accessToken");
        }
        log.info("accessToken============>{}", accessToken);
        return accessToken;
    }


    /**
     * 权限验证不通过时，返回提示信息
     *
     * @author yys
     */
    protected boolean responseMsg(ResultEnum resultEnum, HttpServletResponse response) {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        PrintWriter out = null;
        try {
            RetHead retHead = new RetHead(resultEnum);
            ResponseBase responseBase = new ResponseBase(retHead);
            out = response.getWriter();
            out.append(JSONObject.toJSONString(responseBase));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 获取真实IP地址
     */
    protected String getIpAddress(HttpServletRequest request) {
        String ip = request.getHeader("x-forwarded-for");
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ip == null || ip.length() == 0 || "unknown".equalsIgnoreCase(ip)) {
            ip = request.getRemoteAddr();
        }
        if (ip.contains(",")) {
            return ip.split(",")[0];
        } else {
            return ip;
        }
    }

}
