package com.hcloud.microserver.commoncore.enums;

import lombok.Getter;

/**
 * 权限枚举定义
 *
 * @author yys
 */
@Getter
public enum AuthEnum {
    /**
     * 默认外网游客可访问
     */
    DEFAULT("0000", "游客可访问", "", ""),
    COMMON_PRO_SERVICE("COM0001", "公共问题API查询服务", "", ""),
    NO_AUTH("NO_AUTH0000", "权限开放", "", ""),
    SYS_LOGIN("LOGIN000", "登录接口", "", ""),
    /**
     * 平台权限信息
     **/
    PLATFORM_AUTH_ADD("AUTH0001", "新增平台权限", "", "web:auth:saveAuthInfo"),
    PLATFORM_AUTH_QUERY("AUTH0002", "查询平台权限", "", "web:auth:searchAuthInfo"),
    PLATFORM_AUTH_EDIT("AUTH0003", "修改平台权限", "", "web:auth:editAuthInfo"),
    PLATFORM_AUTH_DELETE("AUTH0004", "删除平台权限", "", "web:auth:delAuthInfo"),

    /**
     * 系统服务权限定义
     */
    SYS_USER_CREATE("SYS0000", "新增系统用户", "", "api:sys:saveUserInfo"),
    SYS_USER_EDIT("SYS0001", "编辑用户信息", "", "api:sys:updateUserInfo"),
    SYS_USER_QUERY("SYS0002", "查询用户信息", "", "api:sys:queryUserListByPage"),
    SYS_USER_DELETE("SYS0003", "删除用户信息", "", "api:sys:delUserInfo"),
    SYS_USER_ENABLE("SYS0008", "激活用户信息", "", "api:sys:enableUserInfo"),
    SYS_PERMISSION_USER_UPDATE("SYS0019", "更用户新权限信息", "", "api:sys:updateUserPermission"),
    SYS_PERMISSION_USER_QUERY("SYS0020", "查询用户权限信息", "", "api:sys:userPermissionTree"),
    SYS_PERMISSION_ADD("SYS0004", "添加权限信息", "", "api:sys:permission:add"),
    SYS_PERMISSION_UPDATE("SYS0005", "更新权限信息", "", "api:sys:permission:update"),
    SYS_PERMISSION_QUERY("SYS0006", "查询权限信息", "", "api:sys:permission:query"),
    SYS_PERMISSION_DELETE("SYS0007", "删除权限信息", "", "api:sys:permission:del"),
    SYS_DEPT_DELETE("SYS0008", "删除部门信息", "", "api:sys:dept:delDept"),
    SYS_DEPT_ADD("SYS0009", "新增部门信息", "", "api:sys:dept:saveDept"),
    SYS_DEPT_QUERY("SYS0010", "查询部门信息", "", "api:sys:dept:listByPage"),
    SYS_DEPT_EDIT("SYS0011", "编辑部门信息", "", "api:sys:dept:modifyDept"),
    SYS_ROLE_DELETE("SYS0012", "删除角色信息", "", "api:sys:role:delRoleInfo"),
    SYS_ROLE_ADD("SYS0013", "新增角色信息", "", "api:sys:role:saveInfo"),
    SYS_ROLE_QUERY("SYS0014", "查询角色信息", "", "api:sys:role:roleList"),
    SYS_PERMISSION_ROLE_QUERY("SYS00015", "查询角色权限信息", "", "api:sys:role:permissionTree"),
    SYS_ROLE_EDIT("SYS0016", "编辑角色信息", "", "api:sys:role:updateInfo"),
    SYS_DEPT_ROLE_INFO("SYS0017", "查询部门及角色基本信息", "", "api:sys:selectList"),
    SYS_PERMISSION_ROLE_UPDATE("SYS00018", "更新角色权限信息", "", "api:sys:role:updatePermissionInfo"),
    /**
     * 问题权限定义
     */
    PROBLEM_CREATE("PRO0000", "新增投诉问题", "", "busin:pro:add"),
    PROBLEM_EDIT("PRO0001", "编辑投诉问题", "", "busin:pro:edit"),
    PROBLEM_DETAIL("PRO0002", "查询问题明细", "", "api:problem:detail"),
    PROBLEM_QUERY("PRO0003", "查询问题列表", "", "api:problem:queryProblemListByPage"),
    PROBLEM_DEAL("PRO0004", "问题处理", "", "api:problem:deal"),
    PROBLEM_DELETE("PRO0005", "删除问题", "", "api:problem:del"),
    PROBLEM_ASSAY_QUERY("PRO0006", "查询问题检验申请记录", "", "api:problem:assay:listByPage"),
    PROBLEM_ASSAY_ADD("PRO0007", "添加问题检验申请记录", "", "api:problem:assay:apply"),
    PROBLEM_ASSAY_DETAIL("PRO0008", "查询问题检验申请明细", "", "api:problem:assay:detail");

    /**
     * 权限编码
     */
    private String authCode;
    /**
     * 权限名称
     */
    private String authName;

    /**
     * 权限描述字符
     */
    private String authTarget;
    /**
     * 权限描述
     */
    private String description;


    AuthEnum(String authCode, String authName, String description, String authTarget) {
        this.authCode = authCode;
        this.authName = authName;
        this.description = description;
        this.authTarget = authTarget;
    }
}
