package com.hcloud.microserver.commoncore.exception;


import com.hcloud.microserver.commoncore.base.ResultVO;
import com.hcloud.microserver.commoncore.enums.ResultEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;

import javax.servlet.http.HttpServletRequest;
import java.io.FileNotFoundException;
import java.util.List;


@Slf4j
@CrossOrigin
@RestControllerAdvice
public class GlobalDefaultExceptionHandler {

    @ExceptionHandler(value = Exception.class)
    public ResultVO defaultErrorHandler(
            HttpServletRequest req, Exception e) {

        log.error(req.getRequestURL() + "<:=====:>" +
                req.getMethod() + "<:=====:>" +
                req.getQueryString() + "<:=====:>" +
                e.getMessage());
        e.printStackTrace();

        ResultVO resultVO = new ResultVO();

        if (e instanceof RuntimeException) {
            log.error(e.getClass().toString());
            resultVO.setCode(1);
            resultVO.setMsg(e.getMessage());
        } else if (e instanceof GlobalException) {
            log.error(e.getClass().toString());
            GlobalException globalException = (GlobalException) e;
            Integer code = globalException.getCode();
            String msg = globalException.getMessage();
            resultVO.setCode(code);
            resultVO.setMsg(msg);
        } else if (e instanceof MethodArgumentNotValidException) {
            log.error(e.getClass().toString());
            List<ObjectError> allErrors = ((MethodArgumentNotValidException) e).getBindingResult()
                    .getAllErrors();
            if (!CollectionUtils.isEmpty(allErrors)) {
                String errorMessage = allErrors.get(0).getDefaultMessage();
                log.error(errorMessage);
                resultVO.setCode(ResultEnum.FAILURE.getCode());
                resultVO.setMsg(errorMessage);
            }
        } else if (e instanceof MissingServletRequestParameterException) {
            log.error(e.getClass().toString());
            resultVO.setCode(ResultEnum.PARAMETER_NOT_NULL.getCode());
            resultVO.setMsg(ResultEnum.PARAMETER_NOT_NULL.getMsg());
        } else if (e instanceof HttpMessageNotReadableException) {
            log.error(e.getClass().toString());
            resultVO.setCode(ResultEnum.PARAMETER_NOT_NULL.getCode());
            resultVO.setMsg(ResultEnum.PARAMETER_NOT_NULL.getMsg());
        } else if (e instanceof FileNotFoundException) {
            log.error(e.getClass().toString());
            resultVO.setCode(ResultEnum.FILE_NOT_FOUND.getCode());
            resultVO.setMsg(ResultEnum.FILE_NOT_FOUND.getMsg());
        } else {
            log.error(e.getClass().toString() + ":" + e.getMessage());
            resultVO.setCode(ResultEnum.FAILURE.getCode());
            resultVO.setMsg(ResultEnum.FAILURE.getMsg());
        }
        return resultVO;
    }
}
