package com.hcloud.microserver.commoncore.service;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.PropertyAccessor;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;
import javax.annotation.PostConstruct;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.annotation.Bean;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import org.springframework.stereotype.Component;


/**
 * @author xiezt
 */
@Component("redisUtils")
@Slf4j
public class RedisUtils {

    @Autowired
    @Qualifier("redisTemplate")
    RedisTemplate template;

    @Autowired
    @Qualifier("stringRedisTemplate")
    StringRedisTemplate stringRedisTemplate;

    @Bean
    CountDownLatch latch() {
        return new CountDownLatch(1);
    }

//    @Bean
//    StringRedisTemplate stringTemplate(RedisConnectionFactory connectionFactory) {
//        return new StringRedisTemplate(connectionFactory);
//    }

    @PostConstruct
    public void init() {
        RedisSerializer<String> stringSerializer = new StringRedisSerializer();
        Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<Object>(
                Object.class);
        ObjectMapper om = new ObjectMapper();
        om.setVisibility(PropertyAccessor.ALL, JsonAutoDetect.Visibility.ANY);
        om.enableDefaultTyping(ObjectMapper.DefaultTyping.NON_FINAL);
        jackson2JsonRedisSerializer.setObjectMapper(om);
        template.setKeySerializer(stringSerializer);
        template.setValueSerializer(jackson2JsonRedisSerializer);
        template.setHashKeySerializer(stringSerializer);
        template.setHashValueSerializer(jackson2JsonRedisSerializer);
        template.afterPropertiesSet();
    }

    public void setValue(String key, Object val) {
        template.opsForValue().set(key, val);
    }

    public void setValue(String key, Object val, int time, TimeUnit unit) {
        template.opsForValue().set(key, val, time, unit);
    }

    public void setValue(String key, Object val, long time, TimeUnit unit) {
        template.opsForValue().set(key, val, time, unit);
    }

    public Object getValue(String key) {
        return template.opsForValue().get(key);
    }

    public void multiSet(Map<String, Object> map) {
        template.opsForValue().multiSet(map);
    }

    public List<Object> multiGet(Collection<String> keys) {
        return template.opsForValue().multiGet(keys);
    }

    public long incr(String key, long delta) {
        return template.opsForValue().increment(key, delta);
    }

    public void lpush(String key, String value) {
        template.opsForList().leftPush(key, value);
    }

    public List<Object> range(String key, int start, int end) {
        return template.opsForList().range(key, start, end);
    }

    public Object rpop(String key) {
        return template.opsForList().rightPop(key);
    }

    public void setHash(String key, Map<String, Object> map) {
        template.opsForHash().putAll(key, map);
    }

    public Object getHash(String key, String prop) {
        return template.opsForHash().get(key, prop);
    }

    /**
     * 删除key
     */
    public void deleteKey(String key) {
        template.delete(key);
    }

    public boolean existsKey(String key) {
        return template.hasKey(key);
    }

    public void lengthenExpire(String key) {
        template.expire(key, 3600, TimeUnit.SECONDS);
    }

    public Map getHashAll(String key) {
        Map map = new HashMap();
        map.put("keys", template.opsForHash().keys(key));
        map.put("vals", template.opsForHash().values(key));
        return map;
    }

    public Set<String> keys(String key) {
        return stringRedisTemplate.keys(key);
    }
}
