package com.hcloud.microserver.commoncore.util;

import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.exception.GlobalException;
import org.springframework.util.StringUtils;

import java.io.File;
import java.math.BigDecimal;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class BasicsUtils {

    final static Map<Integer, String> ZONE_NUM = new HashMap<>();

    private static Pattern CELLPHONE = Pattern
            .compile("^(13[0-9]{9})|(18[0-9]{9})|(14[0-9]{9})|(17[0-9]{9})|(15[0-9]{9})$");

    private static Pattern EMAIL = Pattern.compile("^([a-zA-Z0-9_-])+@([a-zA-Z0-9_-])+(\\.([a-zA-Z0-9_-])+)+$");

    private static Pattern INTEGER = Pattern.compile("[0-9]*");

    static {
        ZONE_NUM.put(11, "北京");
        ZONE_NUM.put(12, "天津");
        ZONE_NUM.put(13, "河北");
        ZONE_NUM.put(14, "山西");
        ZONE_NUM.put(15, "内蒙古");
        ZONE_NUM.put(21, "辽宁");
        ZONE_NUM.put(22, "吉林");
        ZONE_NUM.put(23, "黑龙江");
        ZONE_NUM.put(31, "上海");
        ZONE_NUM.put(32, "江苏");
        ZONE_NUM.put(33, "浙江");
        ZONE_NUM.put(34, "安徽");
        ZONE_NUM.put(35, "福建");
        ZONE_NUM.put(36, "江西");
        ZONE_NUM.put(37, "山东");
        ZONE_NUM.put(41, "河南");
        ZONE_NUM.put(42, "湖北");
        ZONE_NUM.put(43, "湖南");
        ZONE_NUM.put(44, "广东");
        ZONE_NUM.put(45, "广西");
        ZONE_NUM.put(46, "海南");
        ZONE_NUM.put(50, "重庆");
        ZONE_NUM.put(51, "四川");
        ZONE_NUM.put(52, "贵州");
        ZONE_NUM.put(53, "云南");
        ZONE_NUM.put(54, "西藏");
        ZONE_NUM.put(61, "陕西");
        ZONE_NUM.put(62, "甘肃");
        ZONE_NUM.put(63, "青海");
        ZONE_NUM.put(64, "新疆");
        ZONE_NUM.put(71, "台湾");
        ZONE_NUM.put(81, "香港");
        ZONE_NUM.put(82, "澳门");
        ZONE_NUM.put(91, "外国");
    }

    final static int[] PARITYBIT = {'1', '0', 'X', '9', '8', '7', '6', '5', '4', '3', '2'};
    final static int[] POWER_LIST = {7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2};
    final static char[] UPPER_LIST = {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L',
            'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'};

    /**
     * 身份证验证
     */
    public static boolean isIDCard(String certNo) {
        if (certNo == null || (certNo.length() != 15 && certNo.length() != 18)) {
            return false;
        }
        final char[] cs = certNo.toUpperCase().toCharArray();
        //校验位数
        int power = 0;
        for (int i = 0; i < cs.length; i++) {
            if (i == cs.length - 1 && cs[i] == 'X') {
                break;//最后一位可以 是X或x
            }
            if (cs[i] < '0' || cs[i] > '9') {
                return false;
            }
            if (i < cs.length - 1) {
                power += (cs[i] - '0') * POWER_LIST[i];
            }
        }

        //校验区位码
        if (!ZONE_NUM.containsKey(Integer.valueOf(certNo.substring(0, 2)))) {
            return false;
        }

        //校验年份
        String year = certNo.length() == 15 ? getIdcardCalendar() + certNo.substring(6, 8)
                : certNo.substring(6, 10);

        final int iyear = Integer.parseInt(year);
        if (iyear < 1900 || iyear > Calendar.getInstance().get(Calendar.YEAR)) {
            return false;
            //1900年的PASS，超过今年的PASS
        }

        //校验月份
        String month = certNo.length() == 15 ? certNo.substring(8, 10) : certNo.substring(10, 12);
        final int imonth = Integer.parseInt(month);
        if (imonth < 1 || imonth > 12) {
            return false;
        }

        //校验天数
        String day = certNo.length() == 15 ? certNo.substring(10, 12) : certNo.substring(12, 14);
        final int iday = Integer.parseInt(day);
        if (iday < 1 || iday > 31) {
            return false;
        }

        //校验"校验码"
        if (certNo.length() == 15) {
            return true;
        }
        return cs[cs.length - 1] == PARITYBIT[power % 11];
    }

    private static int getIdcardCalendar() {
        GregorianCalendar curDay = new GregorianCalendar();
        int curYear = curDay.get(Calendar.YEAR);
        int year2bit = Integer.parseInt(String.valueOf(curYear).substring(2));
        return year2bit;
    }

    /**
     * 通过文件路径得到文件后缀名。如：hello.txt得到.txt
     *
     * @param filePath 文件路径
     */
    public static String getFileSuffix(String filePath) {
        int pointIndex = filePath.lastIndexOf(".");
        return filePath.substring(pointIndex, filePath.length());
    }

    public static String getFileNameWithoutSuffix(File file) {
        String fileName = file.getName();
        int pointIndex = fileName.lastIndexOf(".");
        return fileName.substring(0, pointIndex);
    }

    public static boolean isEmail(String str) {
        if (StringUtils.isEmpty(str)) {
            return false;
        }

        Matcher m = EMAIL.matcher(str);
        return m.matches();
    }

    public static boolean isCellPhone(String str) {
        if (StringUtils.isEmpty(str)) {
            return false;
        }

        Matcher m = CELLPHONE.matcher(str);
        return m.matches();
    }

    /**
     * 生成手机验证码
     *
     * @return 长度为6的数字字符串
     */
    public static String getVilidateCode() {
        char[] codeSequence = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
        StringBuffer codeStr = new StringBuffer();
        for (int i = 0; i < 6; i++) {
            int j = (int) (Math.random() * 10);
            codeStr.append(codeSequence[j]);
        }
        return codeStr.toString();
    }

    /**
     * MD5加密
     */
    public static String md5(String encryptedObject) {
        try {
            char[] hexDigits = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C',
                    'D', 'E', 'F'};
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(encryptedObject.getBytes());

            byte[] digest = md5.digest();
            int j = digest.length;
            char[] chars = new char[j * 2];
            int k = 0;
            for (int i = 0; i < j; i++) {
                byte byte0 = digest[i];
                chars[k++] = hexDigits[byte0 >>> 4 & 0xf];
                chars[k++] = hexDigits[byte0 & 0xf];
            }
            return new String(chars);
        } catch (Exception e) {
            throw new GlobalException(ResultEnum.ENCRYPTION_ERROR.getCode(), ResultEnum.ENCRYPTION_ERROR.getMsg());
        }
    }

    public static String getToken() {

        return null;
    }

    public static boolean isInteger(String str) {
        if (StringUtils.isEmpty(str)) {
            return false;
        }
        Matcher isNum = INTEGER.matcher(str);
        return isNum.matches();
    }

    /**
     * 小数点保留方法
     *
     * @param value 需要处理的数值
     * @param scale 需要保留的小数位数
     */
    public static float round(double value, int scale) {
        if (scale < 0) {
            throw new IllegalArgumentException("The scale must be a positive integer or zero");
        }
        BigDecimal bigDecimal = new BigDecimal(Double.toString(value));
        BigDecimal one = new BigDecimal("1");
        return bigDecimal.divide(one, scale, BigDecimal.ROUND_HALF_UP).floatValue();
    }

    public static String sHA1(String decript) {
        try {
            MessageDigest digest = MessageDigest.getInstance("SHA-1");
            digest.update(decript.getBytes());
            byte[] messageDigest = digest.digest();
            StringBuffer hexString = new StringBuffer();
            for (int i = 0; i < messageDigest.length; i++) {
                String shaHex = Integer.toHexString(messageDigest[i] & 0xFF);
                if (shaHex.length() < 2) {
                    hexString.append(0);
                }
                hexString.append(shaHex);
            }
            return hexString.toString();

        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        return "";
    }


    public static String getFieldName(String fieldName) {
        return fieldName.replaceFirst(fieldName.substring(0), fieldName.substring(0).toLowerCase());
    }

    /**
     * 根据oauth2入口参数找到定向到前端的路由
     */
    public static String translateRoute(String route) {
        route = StringUtils.isEmpty(route) ? "" : route;
        String redirectRoute = route.replaceAll("!", "/").replaceAll("~", "?");
        return redirectRoute;
    }

    public synchronized static String getRandomUpper(int lenght) {
        Random random = new Random();
        StringBuffer upperStr = new StringBuffer();
        for (int i = 0; i < lenght; i++) {
            upperStr.append(UPPER_LIST[random.nextInt(UPPER_LIST.length)]);
        }
        return upperStr.toString();
    }


}
