package com.hcloud.microserver.commoncore.util;


import static com.hcloud.microserver.commoncore.enums.CommonEnum.FILE_CONTENT_NOT_NULL;

import com.hcloud.microserver.commoncore.enums.CommonEnum;
import com.hcloud.microserver.commoncore.exception.GlobalException;
import com.hcloud.microserver.commoncore.util.string.StringUtils;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.lang.reflect.Method;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import javax.servlet.ServletOutputStream;
import javax.servlet.http.HttpServletResponse;

import jxl.Cell;
import jxl.CellType;
import jxl.CellView;
import jxl.NumberCell;
import jxl.Sheet;
import jxl.Workbook;
import jxl.format.Alignment;
import jxl.format.Border;
import jxl.format.BorderLineStyle;
import jxl.format.CellFormat;
import jxl.format.Colour;
import jxl.format.VerticalAlignment;
import jxl.write.Label;
import jxl.write.NumberFormats;
import jxl.write.WritableCellFormat;
import jxl.write.WritableFont;
import jxl.write.WritableSheet;
import jxl.write.WritableWorkbook;
import jxl.write.WriteException;
import jxl.write.biff.RowsExceededException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.util.CollectionUtils;
import org.springframework.web.multipart.MultipartFile;

@Slf4j
public class ExcelUtils {

    /**
     * 导出excel
     */
    public static void exportExcel(List<?> list, HttpServletResponse response, String tableName,
                                   String tableTitle, String sheetName, LinkedHashMap<String, String> filedNames) {
        try {
            OutputStream os = response.getOutputStream();
            response.reset();
            response.setContentType("application/vnd.ms-excel;charset=utf-8");
            //中文文件名做iso-8859-1转码
            String convertCode = new String(tableName.getBytes("gbk"), "iso8859-1") + ".xls";
            response.setHeader("Content-Disposition", "attachment;filename=" + convertCode);
            //创建Excel文件
            WritableWorkbook workbook = Workbook.createWorkbook(os);
            //创建Sheet页
            WritableSheet sheet = workbook.createSheet(sheetName, 0);

            // 设置标题字体,字体为Arial,字号大小为22,采用黑体显示
            WritableFont titleBold = new WritableFont(WritableFont.ARIAL, 22, WritableFont.BOLD);

            // 设置列名字体,字体为COURIER,字号大小为14,采用黑体显示
            WritableFont columnBold = new WritableFont(WritableFont.COURIER, 18, WritableFont.BOLD);

            // 设置单元格字体,字体为COURIER,字号大小为14,采用黑体显示
            WritableFont cellBold = new WritableFont(WritableFont.COURIER, 14,
                    WritableFont.NO_BOLD);

            // 标题单元格样式控制对象
            WritableCellFormat titleFormat = new WritableCellFormat(titleBold);
            titleFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
            titleFormat.setAlignment(Alignment.CENTRE);
            titleFormat.setWrap(true);//是否自动换行
            titleFormat.setBackground(Colour.WHITE);//单元格背景色
            titleFormat.setBorder(Border.ALL, BorderLineStyle.THIN); //给单元格加边框

            // 列名单元格样式控制对象
            WritableCellFormat columnFormat = new WritableCellFormat(columnBold);
            columnFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
            columnFormat.setAlignment(Alignment.CENTRE);
            columnFormat.setBackground(Colour.GRAY_25);//单元格背景色
            columnFormat.setBorder(Border.ALL, BorderLineStyle.THIN); //给单元格加边框
            //columnFormat.setShrinkToFit(true);//设置缩放字体适应单元格大小
            //columnFormat.setOrientation(Orientation.VERTICAL);//设置字体方向

            // 列名单元格样式控制对象
            WritableCellFormat cellFormat = new WritableCellFormat(cellBold, NumberFormats.TEXT);
            cellFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
            cellFormat.setAlignment(Alignment.CENTRE);
            cellFormat.setWrap(true);//是否自动换行
            cellFormat.setBorder(Border.ALL, BorderLineStyle.THIN); //给单元格加边框

            // 添加表头
            Label label0 = new Label(0, 0, tableTitle, titleFormat);
            sheet.setRowView(0, 1300, false); //设置行高
            sheet.addCell(label0);
            //添加列名
            List<String> column = new ArrayList<>(filedNames.values());
            addCell(sheet, columnFormat, column);
            //添加数据
            List<String> fields = new ArrayList<>(filedNames.keySet());
            addData(list, sheet, fields, cellFormat);

            //合并单元格，第一个参数是起始列，第二个参数是起始行，第三个参数是终止列，第四个参数是终止行
            sheet.mergeCells(0, 0, column.size() - 1, 0);

            //将数据写入文件
            workbook.write();
            workbook.close();
        } catch (IOException e) {
            e.printStackTrace();
        } catch (RowsExceededException e) {
            e.printStackTrace();
        } catch (WriteException e) {
            e.printStackTrace();
        }
    }


    /**
     * 导入excel
     */
    public static List<Map<String, String>> importExcel(MultipartFile file, String filePath)
            throws GlobalException {
        if (file == null) {
            throw new GlobalException(CommonEnum.FILE_NOT_NULL.getIndex(),
                    CommonEnum.FILE_NOT_NULL.getValue());
        }
        String fileName = file.getOriginalFilename();
        String suffixName = fileName.substring(fileName.lastIndexOf("."));

        if (!".xlsx".equals(suffixName) && !".xls".equals(suffixName)) {
            throw new GlobalException(CommonEnum.FILE_TYPE_ERROR.getIndex(),
                    CommonEnum.FILE_TYPE_ERROR.getValue());
        }
        File dest = new File(filePath + fileName);
        // 检测是否存在目录
        if (dest.exists()) {
            throw new GlobalException(CommonEnum.FILE_REPEATED.getIndex(),
                    CommonEnum.FILE_REPEATED.getValue());
        } else if (!dest.getParentFile().exists()) {
            dest.getParentFile().mkdirs();
        }
        try {
            file.transferTo(dest);
        } catch (Exception e) {
            throw new GlobalException(CommonEnum.FILE_UPLOAD_FAIL.getIndex(),
                    CommonEnum.FILE_UPLOAD_FAIL.getValue());
        }
        List<Map<String, String>> list = new ArrayList<>();
        //打开文件
        Workbook book = null;
        try {
            book = Workbook.getWorkbook(dest);
        } catch (Exception e) {
            e.printStackTrace();
            throw new GlobalException(CommonEnum.FILE_UPLOAD_FAIL.getIndex(),
                    CommonEnum.FILE_UPLOAD_FAIL.getValue());
        }
        Sheet sheet = book.getSheet(0);
        boolean sw = true;
        //sheet.getRows()获取行数
        for (int i = 2; i < sheet.getRows(); i++) {
            Map<String, String> map = new HashMap<>();
            for (int j = 0; j < sheet.getColumns(); j++) {
                Cell cell = sheet.getCell(j, i);
                if (!StringUtils.isEmpty(cell.getContents().trim())) {
                    sw = false;
                }
            }
            if (sw) {
                break;
            }
            analyseData(sheet, i, map);
            list.add(map);
            sw = true;
        }
        book.close();
        if (CollectionUtils.isEmpty(list)) {
            throw new GlobalException(FILE_CONTENT_NOT_NULL.getIndex(),
                    FILE_CONTENT_NOT_NULL.getValue());
        }
        return list;
    }

    private static void addData(List<?> list, WritableSheet sheet, List<String> fields,
                                CellFormat cellFormat) throws WriteException {
        for (int j = 0; j < list.size(); j++) {
            for (int i = 0; i < fields.size(); i++) {
                Object fieldValueByName = getFieldValueByName(fields.get(i), list.get(j));
                Label label = new Label(i, j + 2,
                        fieldValueByName == null ? " " : fieldValueByName.toString(), cellFormat);
                sheet.addCell(label);
            }
        }
    }

    private static Object getFieldValueByName(String fieldName, Object o) {
        try {
            String firstLetter = fieldName.substring(0, 1).toUpperCase();
            String methodName = "get" + firstLetter + fieldName.substring(1);
            Method method = o.getClass().getMethod(methodName, new Class[]{});
            Class<?> returnType = method.getReturnType();
            if ("java.util.Date".equals(returnType.getTypeName())) {
                if (!Optional.ofNullable(method.invoke(o, new Object[]{})).isPresent()) {
                    return null;
                }
                Date date = (Date) (method.invoke(o, new Object[]{}));
                return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(date);
            } else {
                return method.invoke(o, new Object[]{});
            }
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    public static String getValue(Map<String, String> map, String key) {
        String value;
        try {
            value = StringUtils.isEmpty(map.get(key)) ? "" : map.get(key);
        } catch (Exception e) {
            value = "";
        }
        return value;
    }

    /**
     * 解析数据
     */
    private static void analyseData(Sheet sheet, int i, Map<String, String> map) {
        //sheet.getColumns()获取列数
        for (int j = 0; j < sheet.getColumns(); j++) {
            Cell cell = sheet.getCell(j, i);
            //取数字的时候强转一下,否则默认只取出小数点后3位
            if (cell.getType() == CellType.NUMBER) {
                NumberCell numberCell = (NumberCell) cell;
                double value = numberCell.getValue();
                map.put(sheet.getCell(j, 1).getContents(), String.valueOf(value));
                continue;
            }
            map.put(sheet.getCell(j, 1).getContents(), cell.getContents());
        }
    }

    /**
     * 导出excel生成文件形式
     */
    public static File exportExcelForFile(List<?> list, String tableTitle,
                                          String sheetName, LinkedHashMap<String, String> filedNames, String filePath,
                                          String fileName) {
        try {
            new File(filePath).mkdirs();
            File file = new File(filePath + fileName);
            //创建Excel文件
            WritableWorkbook workbook = Workbook.createWorkbook(file);
            //创建Sheet页
            WritableSheet sheet = workbook.createSheet(sheetName, 0);

            // 设置标题字体,字体为Arial,字号大小为22,采用黑体显示
            WritableFont titleBold = new WritableFont(WritableFont.ARIAL, 22, WritableFont.BOLD);

            // 设置列名字体,字体为COURIER,字号大小为14,采用黑体显示
            WritableFont columnBold = new WritableFont(WritableFont.COURIER, 18, WritableFont.BOLD);

            // 设置单元格字体,字体为COURIER,字号大小为14,采用黑体显示
            WritableFont cellBold = new WritableFont(WritableFont.COURIER, 14,
                    WritableFont.NO_BOLD);

            // 标题单元格样式控制对象
            WritableCellFormat titleFormat = new WritableCellFormat(titleBold);
            titleFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
            titleFormat.setAlignment(Alignment.CENTRE);
            titleFormat.setWrap(true);//是否自动换行
            titleFormat.setBackground(Colour.WHITE);//单元格背景色
            titleFormat.setBorder(Border.ALL, BorderLineStyle.THIN); //给单元格加边框

            // 列名单元格样式控制对象
            WritableCellFormat columnFormat = new WritableCellFormat(columnBold);
            columnFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
            columnFormat.setAlignment(Alignment.CENTRE);
            columnFormat.setBackground(Colour.GRAY_25);//单元格背景色
            columnFormat.setBorder(Border.ALL, BorderLineStyle.THIN); //给单元格加边框
            // 列名单元格样式控制对象
            WritableCellFormat cellFormat = new WritableCellFormat(cellBold);
            cellFormat.setVerticalAlignment(VerticalAlignment.CENTRE);
            cellFormat.setAlignment(Alignment.CENTRE);
            cellFormat.setWrap(true);//是否自动换行
            cellFormat.setBorder(Border.ALL, BorderLineStyle.THIN); //给单元格加边框

            // 添加表头
            Label label0 = new Label(0, 0, tableTitle, titleFormat);
            sheet.setRowView(0, 1300, false); //设置行高
            sheet.addCell(label0);

            //添加列名
            List<String> column = new ArrayList<>(filedNames.values());
            addCell(sheet, columnFormat, column);

            //添加数据
            List<String> fields = new ArrayList<>(filedNames.keySet());
            addData(list, sheet, fields, cellFormat);

            //合并单元格，第一个参数是起始列，第二个参数是起始行，第三个参数是终止列，第四个参数是终止行
            sheet.mergeCells(0, 0, column.size() - 1, 0);

            //将数据写入文件
            workbook.write();
            workbook.close();
            return file;
        } catch (IOException e) {
            e.printStackTrace();
        } catch (RowsExceededException e) {
            e.printStackTrace();
        } catch (WriteException e) {
            e.printStackTrace();
        }
        throw new GlobalException(CommonEnum.FAILURE.getIndex(), CommonEnum.FAILURE.getValue());
    }

    /**
     * 增加列名
     */
    private static void addCell(WritableSheet sheet, WritableCellFormat columnFormat,
                                List<String> column) throws WriteException {
        for (int i = 0; i < column.size(); i++) {
            Label label = new Label(i, 1, column.get(i), columnFormat);
            CellView cellView = new CellView();
            cellView.setAutosize(true); //设置自动大小
            sheet.setColumnView(i, column.get(i).getBytes().length + 8);//根据内容自动设置列宽
//            setColumnAutoSize(sheet, 10); //设置行宽
            sheet.addCell(label);
        }
    }

    /**
     * 自动调节格式
     */
    private static void setColumnAutoSize(WritableSheet ws, int extraWith) {
        // 获取本列的最宽单元格的宽度
        for (int i = 0; i <= ws.getColumns(); i++) {
            int colWith = 0;
            for (int j = 0; j < ws.getRows(); j++) {
                String content = ws.getCell(i, j).getContents();
                byte[] bstrLength = content.getBytes();
                int cellWith = bstrLength.length;
                if (colWith < cellWith) {
                    colWith = cellWith;
                }
            }
            // 设置单元格的宽度为最宽宽度+额外宽度
            ws.setColumnView(i, colWith + extraWith);
        }
    }
}
