package com.hcloud.microserver.commoncore.util.element;


import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;

import java.util.ArrayList;
import java.util.List;

/**
 * 作用：对一个网页的基本解析操作Tiddler
 * 作者：
 * 时间：2018-08-2018/8/3 17：54
 * 类名：BaseParser
 **/
public class BaseParser {
    private final static Logger log= LogManager.getLogger(BaseParser.class);
    private Document document;
    public BaseParser(Document document) {
        this.document = document;
    }

    /**
     * 获取网页中meta标签的keywords值
     * @return
     */
    public String getMetaKeyword(){
        Elements attr = document.select("meta[name=keywords]");
        if(attr!=null){
            Element element = attr.get(0);
            return element.attr("content");
        }
        return null;
    }

    /**
     * 获取网页中meta标签的description值
     * @return
     */
    public String getMetaDescription(){
        Elements attr = document.select("meta[name=description]");
        if(attr!=null){
            Element element = attr.get(0);
            return element.attr("content");
        }
        return null;
    }

    /**
     * 获取页面中所有的a标签
     * @return
     */
    public List<HrefData> getHrefs(){
        Elements attr = document.getElementsByTag("a");
        List<HrefData> hrefDataList=new ArrayList();
        for (Element element : attr) {
            //实体对象构造方法，代码省略
            HrefData hrefData=new HrefData(element.attr("href"),element.attr("title"),element.text());
            hrefDataList.add(hrefData);
        }
        return hrefDataList;
    }

    /**
     * 获取页面中所有img标签
     * @return
     */
    public List<ImgData> getImgs(){
        Elements attr = document.getElementsByTag("img");
        List<ImgData> imgDataList=new ArrayList<ImgData>();
        for (Element element : attr) {
            ImgData imgData=new ImgData(element.attr("src"),element.attr("alt"));//实体对象构造方法，代码省略
            imgDataList.add(imgData);
        }
        return imgDataList;
    }

    /**
     * 获取页面中的h标签
     * @return
     */
    public List<HData> getHs(){
        List<HData> hDataList=new ArrayList<HData>();
        Elements attr1 = document.getElementsByTag("h1");
        for (Element element : attr1) {
            HData hData=new HData("h1",element.text());//实体对象构造方法，代码省略
            hDataList.add(hData);
        }
        Elements attr2 = document.getElementsByTag("h2");
        for (Element element : attr2) {
            HData hData=new HData("h2",element.text());
            hDataList.add(hData);
        }
        return hDataList;
    }


}