package com.hcloud.microserver.commoncore.util.imgUtils;

import java.io.*;

/**
 * @ClassName ImgBase64
 * @Description: TODO
 * @Author qb
 * @Date 2020/5/15
 * @Version V1.0
 * @ProjectName gzcarbon2.0
 **/

public class ImgBase64 {

    public static String bytesToBase64(byte[] bytes) {
        // 返回Base64编码过的字节数组字符串
        return org.apache.commons.codec.binary.Base64.encodeBase64String(bytes);
    }

    /**
     * 将图片文件转化为字节数组字符串，并对其进行Base64编码处理
     *
     * @param path 图片路径
     * @return base64字符串
     */
    public static String imageToBase64(String path) throws IOException {
        // 将图片文件转化为字节数组字符串，并对其进行Base64编码处理
        byte[] data = null;
        // 读取图片字节数组
        InputStream in = null;
        try {
            in = new FileInputStream(path);
            data = new byte[in.available()];
            in.read(data);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }
        // 返回Base64编码过的字节数组字符串
        return org.apache.commons.codec.binary.Base64.encodeBase64String(data);
    }

    /**
     * 处理Base64解码并写图片到指定位置
     *
     * @param base64 图片Base64数据
     * @param path   图片保存路径
     * @return
     */
    public static boolean base64ToImageFile(String base64, String path) throws IOException {
        // 对字节数组字符串进行Base64解码并生成图片
        // 生成jpeg图片
        try {
            OutputStream out = new FileOutputStream(path);
            return base64ToImageOutput(base64, out);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * 处理Base64解码并输出流
     *
     * @param base64
     * @param out
     * @return
     */
    public static boolean base64ToImageOutput(String base64, OutputStream out) throws IOException {
        // 图像数据为空
        if (base64 == null) {
            return false;
        }
        try {
            // Base64解码

            byte[] bytes = org.apache.commons.codec.binary.Base64.decodeBase64(base64);
            // 调整异常数据
            for (int i = 0; i < bytes.length; ++i) {
                if (bytes[i] < 0) {
                    bytes[i] += 256;
                }
            }
            // 生成jpeg图片
            out.write(bytes);
            out.flush();
            return true;
        } finally {
            try {
                out.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
}
