package com.hcloud.microserver.commoncore.validation;

import com.hcloud.microserver.commoncore.enums.ResultEnum;
import com.hcloud.microserver.commoncore.exception.GlobalException;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * @author vic
 */
@Component
@Aspect
public class FormValidation {

    //注解式表单验证
    @Around("@annotation(com.hcloud.microserver.commoncore.validation.Validate)")
    @Order(1)
    public Object formValidation(ProceedingJoinPoint point) throws Throwable {
        //获得对象方法
        Method method = ((MethodSignature) point.getSignature()).getMethod();
        Validate annotation = method.getAnnotation(Validate.class);
        //通过反射调用注解中的方法
        Class<?> clazz = Class.forName(annotation.value().getName());
        Method clazzMethod = clazz.getMethod("validate", Object[].class);
        Object invoke = null;
        try {
            invoke = clazzMethod.invoke(clazz.newInstance(), (Object) point.getArgs());
        } catch (InvocationTargetException e) {
            throw e.getTargetException();
        } catch (Exception e) {
            throw new GlobalException(ResultEnum.SYS_ERROR.getCode(), ResultEnum.SYS_ERROR.getMsg());
        }
        if (null != invoke) {
            return invoke;
        }
        return point.proceed();
    }
}
